#ifndef INCL_VIRTUAL_CALENDAR_CONVERTER
#define INCL_VIRTUAL_CALENDAR_CONVERTER

#include "base/fscapi.h"
#include "vocl/iCalendar/Calendar.h"
#include "vocl/VConverter.h"

#define EVENT_PROPERTIES_LIST TEXT("BEGIN,CLASS,CREATED,DESCRIPTION,DTSTART,GEO,LAST-MODIFIED,LOCATION,ORGANIZER,PRIORITY,") \
                                   TEXT("DTSTAMP,SEQUENCE,STATUS,SUMMARY,TRANSP,UID,URL,RECURRENCE-ID,DTEND,DURATION,ATTACH,ATTENDEE,") \
                                   TEXT("CATEGORIES,COMMENT,CONTACT,EXDATE,EXRULE,REQUEST-STATUS,RELATED,RESOURCES,RDATE,RRULE,END,GROUP")
   
#define TODO_PROPERTIES_LIST  TEXT("BEGIN,CLASS,COMPLETED,CREATED,DESCRIPTION,DTSTAMP,DTSTART,GEO,LAST-MODIFIED,LOCATION,ORGANIZER,PERCENT-COMPLETE,") \
                                   TEXT("PRIORITY,RECURRENCE-ID,SEQUENCE,STATUS,SUMMARY,UID,URL,DUE,DURATION,ATTACH,ATTENDEE,") \
                                   TEXT("CATEGORIES,COMMENT,CONTACT,EXDATE,EXRULE,REQUEST-STATUS,RELATED,RESOURCES,RDATE,RRULE,END,GROUP")

#define CLASS_PROPERTY_VALUE  TEXT("PUBLIC,PRIVATE,CONFIDENTIAL")

#define COMMENT_PARAM_LIST      TEXT("ALTREP,LANGUAGE")
#define DT_PARAM_LIST           TEXT("VALUE,TZID")
#define ORGANIZER_PARAM_LIST    TEXT("CN,DIR,SENT-BY,LANGUAGE,ROLE")
#define ATTACH_PARAM_LIST       TEXT("FMTTYPE,ENCODING,VALUE")
#define ATTENDEE_PARAM_LIST     TEXT("CUTYPE,MEMBER,ROLE,PARTSTAT,RSVP,DELEGATED-TO,DELEGATED-FROM,SENT-BY,CN,DIR,LANGUAGE")
#define RDATE_PARAM_LIST        TEXT("VALUE,DATE-TIME,DATE,PERIOD,TZID")
#define RECRUL_ITEMS_LIST       TEXT("FREQ,UNTIL,COUNT,INTERVAL,BYSECOND,BYMINUTE,BYHOUR,BYDAY,BYMONTHDAY,BYYEARDAY,BYWEEKNO,BYMONTH,BYSETPOS,WKST")
#define FREQUENCY_VALUES_LIST   TEXT("SECONDLY,MINUTELY,HOURLY,DAILY,WEEKLY,MONTHLY,YEARLY")

#define ERROR_SUCCESS                0L
#define ERROR_KEY_PROPERTY_MISSING   100L
#define ERROR_ILLEGAL_VERSION_NUMBER 101L
#define ERROR_ILLEGAL_PROPERTY_NAME  102L
#define ERROR_UNSUPPORTED_ENCODING   103L
#define ERROR_ILLEGAL_TYPE_PARAMETER 104L
#define ERROR_ILLEGAL_PARAMETER      105L
#define ERROR_INVALID_PROPERTY_VALUE 106L
#define ERROR_PARSING_ERROR          200L
#define ERROR_INVALID_EVENT_BLOCK    301L
#define ERROR_INVALID_TODO_BLOCK     302L

//#define isdigit(c) ((c) >= '0' && (c)<= '9') FIXME


class iCalConverter { 
private:
    wchar_t* iCalendar;
    Calendar* calendar; 
    bool validateEvent(Event*, wchar_t* error, long* errorCode);
    bool validateTodo(ToDo*, wchar_t* error, long* errorCode);
    bool validatePropery(VProperty*, wchar_t* error, long* errorCode);
    Event* extractEvent(VObject* vo, wchar_t* errorDescription, long* errorCode);
    ToDo* extractTask(VObject* vo, wchar_t* errorDescription, long* errorCode);
    bool validateGeo(wchar_t* geo);
    bool validateDT(wchar_t* dt);
    bool validateDate(wchar_t* date);
    bool validateRecur(wchar_t* recur);
    void extractAlarm(VObject* vo);

public:
    iCalConverter();
    ~iCalConverter();
    void setSource(wchar_t* inputICalendar);
    void setSource(Calendar& inputCalendar);
    void getICalendar(wchar_t* vCard);
    void getCalendar(Calendar** outputContact);
    bool convert(wchar_t* error, long* errorCode);
};
#endif
