/*
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <windows.h>
#include "Winbase.h"

#include "pimstore.h"
#include <stdio.h>

#include "base/Log.h"
#include "spds/spdsutils.h"
#include "spds/Constants.h"
#include "spdm/spdmutils.h"
#include "pim/Utils.h"
#include "pim/FILEClient2Server.h"
#include "pim/FILEFileManagement.h"
#include "spds/FileData.h"

#include "shellapi.h"

/*
to get My documents foleder
SHGetSpecialFolderPath(NULL, internalPath, CSIDL_PERSONAL, FALSE);
*/

void setAllItemsFILE(Container* c, int dataType, const wchar_t* path, const wchar_t* dir) {    	        
    
    LOG.debug("Enter in setAllItemsFILE... ");
    
    wchar_t localTemp [DIM_LOCAL_TEMP];

    wchar_t **ptrArrayOfFileName    = NULL;   
    wchar_t  *ptrFileContent        = NULL;
    char* fileContent = NULL;
    char* pp = NULL;
    wchar_t   filterPattern [256];
    wchar_t   internalPath  [DIM_LOCAL_TEMP];

    int numBytes            = 0;
    int count               = 0;
    int j                   = 0;
    unsigned long sizeFile  = 0;    
    
    BOOL recursiveInDirectory = FALSE;
    filterPattern[0] = 0;
    internalPath[0]  = 0;

    //
    // decide the path of the files to read
    //

    if (dataType == constFiles) 
        wsprintf(internalPath, TEXT("%s%s"), dir, TEXT("\\"));
    else if (dataType == constNotes) {
        wsprintf(internalPath,  TEXT("%s%s"), dir, TEXT("\\"));
        wsprintf(filterPattern, PWI_EXTENSION_PATTERN);
    }
    else if (dataType == constFavorites) {     
        SHGetSpecialFolderPath(NULL, localTemp, CSIDL_FAVORITES, FALSE);
        wsprintf(internalPath, TEXT("%s%s"), localTemp, TEXT("\\"));
        recursiveInDirectory = TRUE;          
    }
   
    // to retrieve num of files
    getFileList(internalPath, ptrArrayOfFileName, &count, TRUE, recursiveInDirectory, filterPattern);    
        
    ptrArrayOfFileName = new wchar_t*[count];

    for ( int k = 0; k < count; k++) {
        ptrArrayOfFileName[k] = 0;
    }
   	
    // to retrieve fileName
    getFileList(internalPath, ptrArrayOfFileName, &count, FALSE, recursiveInDirectory, filterPattern);        
       	
    SyncItem* syncItem = NULL;
	
    for (j = 0; j < count; j++) {
    
        // fileName with complete path (e.g. \\Program Files\\SyncClient PIM\\store\\example.bin)
        wsprintf(localTemp, ptrArrayOfFileName[j]);

        if (dataType == constFiles) {
            
            unsigned long sizeFile = 0;
            getFileSize(ptrArrayOfFileName[j], &sizeFile);
            if (sizeFile == 0)
                return; 
            fileContent = new char[sizeFile]; 
            readByteFromFile(ptrArrayOfFileName[j], (byte*)fileContent, &numBytes); 
            FileData fileData;
            getFileName(ptrArrayOfFileName[j], localTemp);        
            fileData.setName(localTemp);
            fileData.setSize(numBytes);
            fileData.setEnc(TEXT("base64"));
            fileData.setBody(fileContent, numBytes);
            fileContent = fileData.format();
            numBytes = strlen(fileContent);
            
        } else if (dataType == constNotes) {
            
            pp = readTextFromBinaryFileChar(ptrArrayOfFileName[j]);       
            getFileName(ptrArrayOfFileName[j], localTemp);
            if (pp != NULL) {
                getXMLNoteChar(&pp, localTemp);
                fileContent = pp;
                numBytes = strlen(fileContent);
            }
            /*          
            ptrFileContent = readTextFromBinaryFile(ptrArrayOfFileName[j]);
                                    
            // only file name without path. (e.g. example.bin)
            getFileName(ptrArrayOfFileName[j], localTemp);
            
            // to send to the server the xml structure <note><Subject>fileName.pwi</Subject><Body>the body</Body></note>
            if (ptrFileContent != NULL) {
                
                getXMLNote(&ptrFileContent, localTemp);
                // translateText2Base64(&ptrFileContent);                
                fileContent = toMultibyte(ptrFileContent);
                numBytes = strlen(fileContent);
            } 
            */

        }
        else if (dataType == constFavorites) {
            
            ptrFileContent = readTextFromBinaryFile(ptrArrayOfFileName[j]);                          

            if (ptrFileContent != NULL) {
                translateText2Base64(&ptrFileContent);                
            }             
            fileContent = toMultibyte(ptrFileContent);
            numBytes = strlen(fileContent);

            wsprintf(localTemp, ptrArrayOfFileName[j]);
            
            wchar_t* t = wcsstr(ptrArrayOfFileName[j], internalPath);            
            
            if (t != NULL) {                   
                wsprintf(localTemp, t + wcslen(internalPath));
                delete [] t; t = NULL;
            }
            if (FILE_SYSTEM_SYNC_SOURCE == TRUE)
                convertSlashAt(localTemp);
        }                 
        
        encodeKey (localTemp);
        
        syncItem = new SyncItem(localTemp);	                

        if (fileContent == NULL) {         
            syncItem->setData("", 1*sizeof(char));	
        }
        else {            
            syncItem->setData(fileContent, numBytes);	
        }
        
        c->addItemToAllItems(syncItem);

        if (ptrFileContent != NULL) {
            delete [] ptrFileContent; ptrFileContent = NULL;
        }
        if (syncItem) {
            delete syncItem; syncItem = NULL;
        }
        if (fileContent != NULL) {
            delete [] ptrFileContent; ptrFileContent = NULL;
        }
        pp = NULL;

	}

    // delete array and pointer...
    for (j = 0; j < count; j++) {
        if (ptrArrayOfFileName[j] != NULL)
            delete [] ptrArrayOfFileName[j]; 
    }

    if (ptrArrayOfFileName != NULL)
        delete [] ptrArrayOfFileName;
    	
	wsprintf(localTemp, SET_ALL_ITEMS, count); 
	LOG.debug("Exiting from setAllItemsFILE... ");
}

void setAllItemsFILEKey(Container* c, int dataType, const wchar_t* path, const wchar_t* dir) {    	        		
    
    LOG.debug("Enter in setAllItemsFILEKey... ");
    wchar_t localTemp [DIM_LOCAL_TEMP];

    wchar_t **ptrArrayOfFileName    = NULL;   
   
    wchar_t   filterPattern [256];
    wchar_t   internalPath  [DIM_LOCAL_TEMP];

    int numBytes            = 0;
    int count               = 0;
    int j                   = 0;
    unsigned long sizeFile  = 0;    
    
    BOOL recursiveInDirectory = FALSE;
    filterPattern[0] = 0;
    internalPath[0]  = 0;

    //
    // decide the path of the files to read
    //

    if (dataType == constFiles) 
        wsprintf(internalPath, TEXT("%s%s"), dir, TEXT("\\"));
    else if (dataType == constNotes) {
        wsprintf(internalPath,  TEXT("%s%s"), dir, TEXT("\\"));
        wsprintf(filterPattern, PWI_EXTENSION_PATTERN);
    }
    else if (dataType == constFavorites) {     
        SHGetSpecialFolderPath(NULL, localTemp, CSIDL_FAVORITES, FALSE);
        wsprintf(internalPath, TEXT("%s%s"), localTemp, TEXT("\\"));
        recursiveInDirectory = TRUE;          
    }
   
    // to retrieve num of files
    getFileList(internalPath, ptrArrayOfFileName, &count, TRUE, recursiveInDirectory, filterPattern);    
    
    ptrArrayOfFileName = new wchar_t*[count];

    for ( int k = 0; k < count; k++) {
        ptrArrayOfFileName[k] = 0;
    }
   	
    // to retrieve fileName
    getFileList(internalPath, ptrArrayOfFileName, &count, FALSE, recursiveInDirectory, filterPattern);        

    SyncItem* syncItem = NULL;
	
    for (j = 0; j < count; j++) {
    
        // fileName with complete path (e.g. \\Program Files\\SyncClient PIM\\store\\example.bin)
        wsprintf(localTemp, ptrArrayOfFileName[j]);

        if (dataType == constFiles) 
            getFileName(ptrArrayOfFileName[j], localTemp);
        else if (dataType == constNotes)
            getFileName(ptrArrayOfFileName[j], localTemp);
        else if (dataType == constFavorites) {
                                    
            wsprintf(localTemp, ptrArrayOfFileName[j]);            
            wchar_t* t = wcsstr(ptrArrayOfFileName[j], internalPath);            
            
            if (t != NULL) {                   
                wsprintf(localTemp, t + wcslen(internalPath));
                delete [] t;
            }
            if (FILE_SYSTEM_SYNC_SOURCE == TRUE)
                convertSlashAt(localTemp);
        }
        
        encodeKey (localTemp);
    	                
        syncItem = new SyncItem(localTemp);	                
        syncItem->setData("", 1*sizeof(char));	
      
        c->addItemToAllItems(syncItem);

        if (syncItem) {
            delete syncItem; syncItem = NULL;
        }
	}

    // delete array and pointer...
    for (j = 0; j < count; j++) {
        if (ptrArrayOfFileName[j] != NULL)
            delete ptrArrayOfFileName[j]; 
    }
    if (ptrArrayOfFileName != NULL)
        delete [] ptrArrayOfFileName;  
    LOG.debug("Exiting from setAllItemsFILEKey... ");
}


/*
* The method replace the text file content with new one in xml format.
*/
void getXMLNoteChar (char** ptrFileContent, wchar_t* fileName) {

    char* t  = NULL;
    wchar_t  fileNameNoPwiExt [DIM_LOCAL_TEMP];
    string s = string();
    int position = 0;
    
    string element_s = string();
    string key_s = string();

    int filelen = strlen(*ptrFileContent);
   
    wsprintf(fileNameNoPwiExt, fileName);
       
    s += (wchar_t*)fileNameNoPwiExt;
    position = pos(PWI_EXTENSION, s);
    if (position != -1) {
        fileNameNoPwiExt[position] = 0;
        s = string((wchar_t*)fileNameNoPwiExt);
    }       
    //encodeSpecialChar(s); 

    char* noteFileName = NULL;
    noteFileName = toMultibyte((wchar_t*)s);

    wchar_t tempDate[64];
    doubleToSystemTime(tempDate, getSystemTime());
    char* date = toMultibyte(tempDate);

    t = new char [250 + filelen];
    t[0] = 0;
    strcat(t, "<?xml version=\"1.0\" encoding=\"UTF-8\"?><note><Subject>");
    strcat(t, noteFileName); // filename (with no .pwi extension)
    strcat(t, "</Subject><Body>");
    strcat(t, *ptrFileContent);
    strcat(t, "</Body><Date>");
    strcat(t, date);
    strcat(t, "</Date></note>");
    
    if (noteFileName) {delete [] noteFileName; noteFileName = NULL; }
    if (date)         {delete [] date;         date = NULL;         }

    *ptrFileContent = t;     
}



/*
* The method replace the text file content with new one in xml format.
*/
void getXMLNote (wchar_t** ptrFileContent, wchar_t* fileName) {

    wchar_t* t  = NULL;
    wchar_t  fileNameNoPwiExt [DIM_LOCAL_TEMP];
    string s = string();
    int position = 0;
    
    string element_s = string();
    string key_s = string();

    element_s = string(*ptrFileContent); 
    encodeSpecialChar(element_s);    

    wsprintf(fileNameNoPwiExt, fileName);
       
    s += (wchar_t*)fileNameNoPwiExt;
    position = pos(PWI_EXTENSION, s);
    if (position != -1) {
        fileNameNoPwiExt[position] = 0;
        s = string((wchar_t*)fileNameNoPwiExt);
    }       
    encodeSpecialChar(s); 
    
    wchar_t tempDate[64];
    doubleToSystemTime(tempDate, getSystemTime());

    t = new wchar_t [250 + length(element_s)];
    t[0] = 0;
    wcscat(t, TEXT("<?xml version=\"1.0\" encoding=\"UTF-8\"?><note><Subject>"));
    wcscat(t, (wchar_t*)s); // filename (with no .pwi extension)
    wcscat(t, TEXT("</Subject><Body>"));
    wcscat(t, (wchar_t*)element_s);
    wcscat(t, TEXT("</Body><Date>"));
    wcscat(t, tempDate);
    wcscat(t, TEXT("</Date></note>"));
    
    delete *ptrFileContent;

    *ptrFileContent = t;     
}

/*
* text pointed by ptrData is translated in base64 and assigned to ptrData
* 
*/
void translateText2Base64(wchar_t** ptrFileContent) {
    
    wchar_t* ptr = NULL;
    ptr = encodeBase64UTF8(*ptrFileContent);    
    if (*ptrFileContent != NULL) {
        delete *ptrFileContent; 
        *ptrFileContent = NULL;
    }
    *ptrFileContent = ptr;      
}


/*
* get the data content of a file by filename and return these data into base64 format.
* It's useful to read data to set into setAllItems, setModifiedItem, setNewItems
*/


char* getB64DataContentByFilename(wchar_t* filename) {

    unsigned long sizeFile      = 0;
    int      numBytes           = 0;
    byte*    ptrFileContent     = NULL;    
    char*    base64             = NULL;
    wchar_t* ptrFileContentb64  = NULL;
    int encodeLen               = 0; 
    getFileSize(filename, &sizeFile);

    if (sizeFile == 0)
        goto finally;

    ptrFileContent = new byte[sizeFile]; 

    // read bytes from file

    readByteFromFile(filename, ptrFileContent, &numBytes);

    if (numBytes != (int)sizeFile) {        	            
		LOG.error("The number of byte read and size are different");            	    
        goto finally;
    }		    

    encodeLen = lengthForB64(numBytes); 
    base64 = new char[encodeLen + 1];

    if (base64 == NULL) {
        LOG.error("not enough memory to instantiate array...");
    }

    memset(base64, 0, encodeLen + 1);
    
    b64_encode(base64, ptrFileContent, numBytes);                
    
finally:

    if (ptrFileContent != NULL)
        delete ptrFileContent; ptrFileContent = NULL;
    
    return base64;
}



/*
* Method to understand if file are new, deleted or updated.
*/

void setModifiedItemsFILE(Container* c, int dataType, const wchar_t* path, const wchar_t* dir) {

    LOG.debug("Enter in setModifiedItemsFILE... ");
    
    wchar_t   localTemp     [DIM_LOCAL_TEMP];    
    wchar_t   internalPath  [DIM_LOCAL_TEMP];
    wchar_t   filterPattern [DIM_LOCAL_TEMP];
    wchar_t   wtime         [DIM_LOCAL_TEMP]; 
    
    localTemp[0]        = 0;
    internalPath[0]     = 0;
    wtime[0]            = 0;
    filterPattern[0]    = 0;

    wchar_t** ptrFilenameInDir      = NULL;   
    wchar_t*  ptrFileContent        = NULL;        
    char* fileContent               = NULL;
    wchar_t** ptrFilenameFromFile   = NULL;
    char* pp = NULL;

    SyncItem* syncItem = NULL;

    int numBytes            = 0;    
    int countFileInDir      = 0;
    int countFileInFile     = 0;
    int countNewItems       = 0;
    int countUpdatedItems   = 0;
    int countDeletedItems   = 0;

    int j                   = 1;
    int i                   = 0;
    unsigned long sizeFile  = 0;    
    DATE dtime              = 0.0;
    DATE timeInFile         = 0.0;
         
    BOOL found = FALSE;
    BOOL recursiveInDirectory = FALSE;
    
    filterPattern[0] = 0;
    
    string key_s = string();

    if (dataType == constFiles)
        wsprintf(internalPath, TEXT("%s%s"), dir, TEXT("\\"));
    else if (dataType == constNotes) {
        wsprintf(internalPath, TEXT("%s%s"), dir, TEXT("\\"));
        wsprintf(filterPattern, PWI_EXTENSION_PATTERN);
    }
    else if (dataType == constFavorites) {        
        SHGetSpecialFolderPath(NULL, internalPath, CSIDL_FAVORITES, FALSE);
        wcscat(internalPath, TEXT("\\"));    
        recursiveInDirectory = TRUE;    
    }

    // to retrieve num of files in directory
    getFileList(internalPath, ptrFilenameInDir, &countFileInDir, TRUE, recursiveInDirectory, filterPattern);    
    ptrFilenameInDir = new wchar_t*[countFileInDir];
   	
    // to retrieve fileName
    getFileList(internalPath, ptrFilenameInDir, &countFileInDir, FALSE, recursiveInDirectory, filterPattern);              
    
    // to retrieve num of files in files.dat/notes.dat
    countFileInFile = readFilenameFromFile(dataType, path, &ptrFilenameFromFile);
    
    // if there is no backup file...
    if (!ptrFilenameFromFile)
        return;


    // first element in file is timestamp
    timeInFile = wcstod(ptrFilenameFromFile[0], NULL);    
       
    //
    // to get the new or update items that are files
    //    
   
    for (i = 0; i < countFileInDir; i++) {   
        
        found = FALSE;

        for (j = 1; j <= countFileInFile; j++) {                              

            if (wcscmp(ptrFilenameInDir[i], ptrFilenameFromFile[j]) == 0) {            
            
                found = TRUE;
                getLastWriteTime(ptrFilenameInDir[i], wtime, &dtime);
                
                if (dtime > timeInFile) {
                    
                    if (dataType == constFiles) {                           
                        unsigned long sizeFile = 0;
                        getFileSize(ptrFilenameInDir[i], &sizeFile);
                        if (sizeFile == 0)
                            return;
                        fileContent = new char[sizeFile]; 

                        readByteFromFile(ptrFilenameInDir[i], (byte*)fileContent, &numBytes); 
                        // only file name without path. (e.g. example.bin)
                        FileData fileData;            
                        getFileName(ptrFilenameInDir[i], localTemp);
                        fileData.setName(localTemp);
                        fileData.setSize(numBytes);
                        fileData.setEnc(TEXT("base64"));
                        fileData.setBody(fileContent, numBytes);
                        fileContent = fileData.format();
                        numBytes = strlen(fileContent);                            
                    }
                    else if (dataType == constNotes) {     

                        pp = readTextFromBinaryFileChar(ptrFilenameInDir[i]);       
                        getFileName(ptrFilenameInDir[i], localTemp);
                        if (pp != NULL) {
                            getXMLNoteChar(&pp, localTemp);
                            fileContent = pp;
                            numBytes = strlen(fileContent);
                        }

                        /*
                        ptrFileContent = readTextFromBinaryFile(ptrFilenameInDir[i]);
                        
                         // only file name without path. (e.g. example.bin)
                        getFileName(ptrFilenameInDir[i], localTemp);

                        if (ptrFileContent != NULL) {
                           
                            getXMLNote(&ptrFileContent, localTemp);
                            fileContent = toMultibyte(ptrFileContent);
                            numBytes = strlen(fileContent);                                        
                            
                        }
                        */                       
                    }
                    else if (dataType == constFavorites) {
                        ptrFileContent = readTextFromBinaryFile(ptrFilenameInDir[i]);
                        
                        if (ptrFileContent != NULL) {                               
                            fileContent = toMultibyte(ptrFileContent);
                            numBytes = strlen(fileContent);
                        }

                        wsprintf(localTemp, ptrFilenameInDir[i]);                            

                        wchar_t* t = wcsstr(ptrFilenameInDir[i], internalPath);            
                        if (t != NULL) {
                             if (dbg) {	                       
	                            MessageBox (NULL, t, TEXT ("settingModItems"), MB_SETFOREGROUND |MB_OK);	            
                             }   
                            wsprintf(localTemp, t + wcslen(internalPath));
                        }
                        if (FILE_SYSTEM_SYNC_SOURCE == TRUE)
                            convertSlashAt(localTemp);

                    }                        
                                                                       
                    encodeKey (localTemp);
                    key_s = string(localTemp);
                    //encodeSpecialChar(key_s);                        
                                           
                    syncItem = new SyncItem((wchar_t*)key_s);	

                    // without CDATA
    
                        if (fileContent == NULL) {                        
                        syncItem->setData("", 1*sizeof(char));	
                    }
                    else {                            
                        syncItem->setData(fileContent, numBytes*sizeof(char));	                                                    
                    }
                    countUpdatedItems++;
                    
                    c->addItemToUpdatedItems(syncItem);

                    if (syncItem)
                        delete syncItem;
                    syncItem = NULL;

                    if (ptrFileContent != NULL)
                        delete [] ptrFileContent; 
                    ptrFileContent = NULL;
                    if (fileContent != NULL)
                        delete [] fileContent; 
                    fileContent = NULL;
                    pp = NULL;

                }
                else {

                }
                break;
            }

        }

        if (found == FALSE) {           

            if (dataType == constFiles) {
                
                unsigned long sizeFile = 0;
                getFileSize(ptrFilenameInDir[i], &sizeFile);
                if (sizeFile == 0)
                    return;
                fileContent = new char[sizeFile]; 

                readByteFromFile(ptrFilenameInDir[i], (byte*)fileContent, &numBytes); 
                //getFileName(ptrFilenameInDir[i], localTemp);
                FileData fileData;            
                getFileName(ptrFilenameInDir[i], localTemp);
                fileData.setName(localTemp);
                fileData.setSize(numBytes);
                fileData.setEnc(TEXT("base64"));
                fileData.setBody(fileContent, numBytes);
                fileContent = fileData.format();
                numBytes = strlen(fileContent);                
            }
            else if (dataType == constNotes) {    

                pp = readTextFromBinaryFileChar(ptrFilenameInDir[i]);       
                getFileName(ptrFilenameInDir[i], localTemp);
                if (pp != NULL) {
                    getXMLNoteChar(&pp, localTemp);
                    fileContent = pp;
                    numBytes = strlen(fileContent);
                }
                /*
                ptrFileContent = readTextFromBinaryFile(ptrFilenameInDir[i]);                
                getFileName(ptrFilenameInDir[i], localTemp);

                if (ptrFileContent != NULL) {                   
                    getXMLNote(&ptrFileContent, localTemp);
                    fileContent = toMultibyte(ptrFileContent);
                    numBytes = strlen(fileContent);                
                } 
                */
            }
            else if (dataType == constFavorites) {
                ptrFileContent = readTextFromBinaryFile(ptrFilenameInDir[i]);

                if (ptrFileContent != NULL) {
                    fileContent = toMultibyte(ptrFileContent);
                    numBytes = strlen(fileContent);                    
                }

                wsprintf(localTemp, ptrFilenameInDir[i]);

                wchar_t* t = wcsstr(ptrFilenameInDir[i], internalPath);            

                if (t != NULL) {                
                     wsprintf(localTemp, t + wcslen(internalPath));
                }
                if (FILE_SYSTEM_SYNC_SOURCE == TRUE) {
                    convertSlashAt(localTemp);
                }

            }

            encodeKey (localTemp);
            key_s = string(localTemp);
            //encodeSpecialChar(key_s);
            
            syncItem = new SyncItem((wchar_t*)key_s);
                    
            if (fileContent == NULL) {
                syncItem->setData("", 1*sizeof(char));	
            }
            else {
                syncItem->setData(fileContent, numBytes*sizeof(char));	                                        
            }
            countNewItems++;
            
            c->addItemToNewItems(syncItem);

            if (syncItem) 
                delete syncItem;
            syncItem = NULL;
           
            if (ptrFileContent != NULL)
                delete [] ptrFileContent; 
            ptrFileContent = NULL;            

            if (fileContent != NULL)
                delete [] fileContent; 
            fileContent = NULL;     
            pp = NULL;
        }

    }
    
    //
    // to get the delete items that are files
    //

    for (j = 1; j <= countFileInFile; j++) {

        found = FALSE;
        for (i = 0; i < countFileInDir; i++) {                        
            if (wcscmp( ptrFilenameFromFile[j], ptrFilenameInDir[i]) == 0) {
                found = TRUE;               
            }
        }        
        if (found == FALSE) {
            
            if (dataType == constFiles || dataType == constNotes) {
                getFileName(ptrFilenameFromFile[j], localTemp);
            }
            else if (dataType == constFavorites) {                
                wsprintf(localTemp, ptrFilenameFromFile[j]);
                wchar_t* t = wcsstr(ptrFilenameFromFile[j], internalPath);         
                if (t != NULL) {
                    wsprintf(localTemp, t + wcslen(internalPath));
                }
                if (FILE_SYSTEM_SYNC_SOURCE == TRUE)
                    convertSlashAt(localTemp);
            }

            encodeKey (localTemp);
            key_s = string(localTemp);
            //encodeSpecialChar(key_s);

            syncItem = new SyncItem((wchar_t*)key_s);	 
            
            c->addItemToDeletedItems(syncItem);

            if (syncItem)
                delete syncItem;
            syncItem = NULL;

            countDeletedItems++;

        }
    }
    
    // delete array and pointer...
    for (j = 0; j < countFileInDir; j++) {
        if (ptrFilenameInDir[j] != NULL)
            delete ptrFilenameInDir[j]; 
    }
    if (ptrFilenameInDir != NULL)
        delete [] ptrFilenameInDir;

    for (j = 0; j < countFileInFile; j++) {
        if (ptrFilenameFromFile[j] != NULL)
            delete ptrFilenameFromFile[j]; 
    }
    if (ptrFilenameFromFile != NULL)
        delete [] ptrFilenameFromFile;      

    LOG.debug("Exiting from setModifiedItemsFILE... ");
}

