/*
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "pim/FileObjectSyncSource.h"
#include "spds/SyncItemStatus.h"
#include "base/util/utils.h"
#include "base/Log.h"
#include "pim/FILEClient2Server.h"
#include "pim/FILEServer2Client.h"
#include "pim/FILEFileManagement.h"
#include "base/startcmd.h"
#include "Winbase.h"
#include "pim/SettingFunctions.h"
#include "spds/FileData.h"
#include "notify/timed_msgbox.h"

static int call;
static int cnew;
static int ckey;
static int cupdated;
static int cdeleted;

#define ASK_REFRESH_FILE_MESSAGE TEXT("All the local %s will be deleted before adding the server items. Are you sure you want to continue?")
#define ASK_REFRESH_FILE_TIMEOUT 10
#define BACKUP_FILE TEXT("FunambolBackup")

FileObjectSyncSource::FileObjectSyncSource(const wchar_t* name) : SyncSource(name) {
    c = NULL; 
    path = NULL;
    dir = NULL;
    setPath(TEXT("\\"));
    setDir(TEXT("\\"));    
    friendlyName[0] = 0;  // char 64
    newFromServer = 0;
    updatedFromServer = 0;
    deletedFromServer = 0;
}

FileObjectSyncSource::~FileObjectSyncSource() {
    if (c) 
        delete c; 
    if (path)
        delete [] path;
    if(dir)
        delete [] dir;
}

void FileObjectSyncSource::setType(SourceType t) {
    type = t;
}

SourceType FileObjectSyncSource::getType() {
    return type;
}

void FileObjectSyncSource::setPath(const wchar_t* p) {
    if (path)
        delete [] path;

    path = (p) ? wstrdup(p) : wstrdup(TEXT("\\"));
}

const wchar_t* FileObjectSyncSource::getPath() {
    return path;
}

void FileObjectSyncSource::setDir(const wchar_t* p) {
    if (dir)
        delete [] dir;

    dir = (p) ? wstrdup(p) : wstrdup(TEXT("\\"));
}

const wchar_t* FileObjectSyncSource::getDir() {
    return dir;
}
/*
* Return the first SyncItem of all. It is used in case of slow or refresh sync 
* and retrieve the entire data source content.
*/

SyncItem* FileObjectSyncSource::getFirstItem() {
        
    if (c == NULL) {
        c = new Container();
    }

    setAllItemsFILE(c, getType(), TEXT(""), getDir()); // 

    LOG.info("The client number of %s to sync are %i", friendlyName, c->getAllItemsSize());

    if (c->getAllItemsSize() == 0) {
        return NULL;
    }        
    call = 0;
    ArrayList* list = c->getAllItems();    
    return (SyncItem*)list->get(call);    

}

/*
* Return the next SyncItem of all. It is used in case of slow or refresh sync 
* and retrieve the entire data source content.
*/

SyncItem* FileObjectSyncSource::getNextItem() {        
    if (c == NULL)
        return NULL;

    if (c->getAllItemsSize() == 0)
        return NULL;

    call++;
    if (call == c->getAllItemsSize()) {
        return NULL;
    }        
    return (SyncItem*)c->getAllItems()->get(call);  
}

SyncItem* FileObjectSyncSource::getFirstItemKey() {
    if (c == NULL) {
        c = new Container();
    }
    ArrayList* list = NULL;

    setAllItemsFILEKey(c, getType(), TEXT(""), getDir()); // 
    if (c->getAllItemsSize() == 0) {
        goto finally;
    }        
    ckey = 0;
    list = c->getAllItems();   

    if (list && list->size() > 0) {
        for (ckey = 0; ckey < list->size(); ckey++) {
            deleteItem(*(SyncItem*)list->get(ckey));
        }
    }

    /*
    //
    // To implement the backup before a refresh from server
    //
    if (this->getPreferredSyncMode() == SYNC_REFRESH_FROM_SERVER)
    {
        wchar_t logMessage[128];
        wchar_t backupFolder[128];
        wcscpy(backupFolder, BACKUP_FILE);
        wchar_t backupFolderPath[255];
        wsprintf(backupFolderPath, TEXT("%s\\%s"),(wchar_t*)getProgramPath(),backupFolder);
        int res = CreateDirectory(backupFolderPath, NULL);
        int errCode;
        if (res == 0)
        {
            
            errCode = GetLastError();
            if (errCode == ERROR_ALREADY_EXISTS)
            {
                wsprintf(logMessage, TEXT("Backup directory already exists"));
                // todo
                const wchar_t* syncedFolder = getDir();
                for (int i = 0; i < list->size(); i++)
                {
                    SyncItem* item = (SyncItem*)list->get(i)->clone();
                    wchar_t file [128];
                    wsprintf(file, TEXT("%s\\%s"),syncedFolder, item->getKey());
                    delete item;
                    DeleteFile(file);                     
                }
                syncedFolder = NULL;

            }
            else
                wsprintf(logMessage, TEXT("Error creating directory for backup of the files %d"), errCode);
            logDebug(logMessage);
        }
        else
        {
            wchar_t sourceFolder [128];
            wsprintf(sourceFolder,TEXT("%s"),getDir());
            for (int i = 0; i < list->size(); i++)
            {
                SyncItem* item = (SyncItem*)list->get(i)->clone();
                wchar_t oldFile [128];
                wchar_t newFile [128];
                wsprintf(oldFile, TEXT("%s\\%s"),sourceFolder, item->getKey());
                wsprintf(newFile, TEXT("%s\\%s"),backupFolderPath, item->getKey());
                delete item;
                BOOL fileMoved = MoveFile(oldFile, newFile); 
                if (!fileMoved)
                {
                    errCode = GetLastError();
                    wsprintf(logMessage, TEXT("Error moving the file %s , error code is %d"), oldFile, errCode);
                    logDebug(logMessage);
                }
            }
        }
    }
    */
finally:
    if (c) { delete c; c = NULL; }
    return NULL;
}

SyncItem* FileObjectSyncSource::getNextItemKey() {

    return NULL;

}

/*
* This method is always called by the sync manager. So the setModifiedItemsPOOM is always called
*/

SyncItem* FileObjectSyncSource::getFirstNewItem() {
    if (c == NULL) {
        c = new Container();
    }

    setModifiedItemsFILE(c, getType(), getPath(), getDir() ); 
   
     //Log the number of item to sync from client
    LOG.info("The client number of new %s to sync are %i", friendlyName, c->getNewItemsSize());
    LOG.info("The client number of updated %s to sync are %i", friendlyName, c->getUpdatedItemsSize());
    LOG.info("The client number of deleted %s to sync are %i", friendlyName, c->getDeletedItemsSize());
   
    if (c->getNewItemsSize() == 0) {
        return NULL;
    }    
    cnew = 0;
    ArrayList* list = c->getNewItems();

    return (SyncItem*)list->get(cnew);    

}

SyncItem* FileObjectSyncSource::getNextNewItem() {    
   if (c == NULL)
        return NULL;

    cnew++;
    if (cnew == c->getNewItemsSize()) {
        return NULL;
    }        
    return (SyncItem*)c->getNewItems()->get(cnew);  
}

SyncItem* FileObjectSyncSource::getFirstUpdatedItem() {
    if (c == NULL) {
        c = new Container();
    }

    if (c->getUpdatedItemsSize() == 0) {
        return NULL;
    }    
    cupdated = 0;
    ArrayList* list = c->getUpdatedItems();

    return (SyncItem*)list->get(cupdated);          
}

SyncItem* FileObjectSyncSource::getNextUpdatedItem() {    
    if (c == NULL)
        return NULL;

    cupdated++;
    if (cupdated == c->getUpdatedItemsSize()) {
        return NULL;
    }        
    return (SyncItem*)c->getUpdatedItems()->get(cupdated);  
}

SyncItem* FileObjectSyncSource::getFirstDeletedItem() {
   
    if (c == NULL) {
        c = new Container();
    }

    if (c->getDeletedItemsSize() == 0) {
        return NULL;
    }    
    cdeleted = 0;
    ArrayList* list = c->getDeletedItems();

    return (SyncItem*)list->get(cdeleted);      
}

SyncItem* FileObjectSyncSource::getNextDeletedItem() {    
    if (c == NULL)
        return NULL;

    cdeleted++;
    if (cupdated == c->getDeletedItemsSize()) {
        return NULL;
    }        
    return (SyncItem*)c->getDeletedItems()->get(cdeleted);    
}

void FileObjectSyncSource::setItemStatus(const wchar_t* key, int status) {
    LOG.debug(T("key: %S, status: %i"), key, status);    
}

int FileObjectSyncSource::addItem(SyncItem& item) {
    
    int ret = STC_COMMAND_FAILED; // command Failed;
    
    if (!isMemoryAvailable()) {
        LOG.error(LOW_DEVICE_MEMORY);
        return ret;
    }

    FileData file;
    char* data      = NULL;
    wchar_t* oid    = NULL;
    int size = 0;
    int res = 0;
    size = item.getDataSize();    
    data = new char[size + 1];
    memset(data, 0, size + 1);
    memcpy(data, item.getData(), size);
    res = file.parse(data);
    const wchar_t *encod = file.getEnc();
    delete [] data;
    // TODO bug #304192
    if (res == -1)// the file was not sent by client as FileObject version 1.2
    {
        oid = FILEmanageNewItems(&item, getType(), getPath(), getDir());
    }
    else 
        if (wcslen(encod) > 0)
        {
            item.setKey(file.getName());
            item.setData(file.getBody(), file.getSize());
            oid = FILEmanageNewItems(&item, getType(), getPath(), getDir());
        }
        
        
    if (oid != NULL) {        
        ret = STC_ITEM_ADDED;
        LOG.debug(T("Added item: %S LUID: %S"), item.getKey(), oid);                
        item.setKey(oid);   

        delete [] oid; oid = NULL;

        newFromServer++;
    }

    return ret;
}

int FileObjectSyncSource::updateItem(SyncItem& item) {
    
    int ret = STC_COMMAND_FAILED; // command Failed;
    if (!isMemoryAvailable()) {
        LOG.error(LOW_DEVICE_MEMORY);
        return ret;
    }

    FileData file;
    char* data      = NULL;
    long h          = 0;
    wchar_t* encod  = NULL;
    int size = 0;
    int res = 0;
    size = item.getDataSize();    
    data = new char[size + 1];
    memset(data, 0, size + 1);
    memcpy(data, item.getData(), size);
    res = file.parse((data));
    encod = (wchar_t*)file.getEnc();
    delete [] data;
    if (res == -1)// the file was not sent by client as FileObject version 1.2
    {
        h = FILEmanageUpdatedItems(&item, getType(), getPath(), getDir());
    }
    else 
        if (wcslen(encod) > 0)
        {        
            item.setData(file.getBody(), file.getSize());
            h = FILEmanageUpdatedItems(&item, getType(), getPath(), getDir());
        }                   
    if (h == 0) {
        ret = STC_OK;
        LOG.debug(T("updated item: %S"), item.getKey());    

        updatedFromServer++;
    }
    return ret;
}

int FileObjectSyncSource::deleteItem(SyncItem& item) {
    
    int ret = STC_COMMAND_FAILED; // command Failed;
    long h = FILEmanageDeletedItems(&item, getType(), getPath(), getDir());
    
    if (h == 0) {
        ret = STC_OK;
        LOG.debug(T("deleted item: %S"), item.getKey());

        deletedFromServer++;
        
    }
    return ret;    
}

int FileObjectSyncSource::beginSync() {
    /*
    OL_CONTACTS     = 10,
    OL_CALENDAR     = 9,
    OL_TASK         = 13,
    OL_BRIEFCASE    = 600,
    OL_NOTES        = 601,
    OL_FAVORITES    = 602
    */
    wchar_t message[256];
    wchar_t* sourceNameW = NULL;
    
    // counter to get the number of items from server
    newFromServer = 0;
    updatedFromServer = 0;
    deletedFromServer = 0;

    message[0] = 0;
    
    if (getType() == OL_NOTES)
        strcpy(friendlyName, "notes");
    else if (getType() == OL_BRIEFCASE)
        strcpy(friendlyName, "files");
    
    sourceNameW = toWideChar(friendlyName);

    wsprintf(message, ASK_REFRESH_FILE_MESSAGE, sourceNameW);

    if (getSyncMode() == SYNC_REFRESH_FROM_SERVER) {
        int val = TimedMessageBox(NULL, message, TEXT("Alert"),
                             MB_YESNO | MB_ICONQUESTION | MB_SETFOREGROUND, ASK_REFRESH_FILE_TIMEOUT * 1000); // default ASK_TIMEOUT = 10

        if (val == IDYES) {
            LOG.debug("User said yes to delete local file or note source and refresh.");            
        }
        else {
            LOG.debug("User said no to delete local file or note and refresh");     
            return -1;
        }
    }

    if (sourceNameW) { delete [] sourceNameW; sourceNameW = NULL; }
    return 0;    
    
}

int FileObjectSyncSource::endSync() {
        
    writeCurrentFileItems(getType(), getPath(), getDir());

    //Log the number of item synced from server
    LOG.info("The server number of new %s synced are %i", friendlyName, newFromServer);
    LOG.info("The server number of updated %s synced are %i", friendlyName, updatedFromServer);
    LOG.info("The server number of deleted %s synced are %i", friendlyName, deletedFromServer);
         
    // remove backup file since the sync ended well
    /* 
    //
    // To implement the backup after a refresh from server
    //
    int errCode;
    int res;
    wchar_t logMessage[256];
    wchar_t backupFolder[128];
    wcscpy(backupFolder, BACKUP_FILE);
    wchar_t backupFolderPath[255];
    wsprintf(backupFolderPath, TEXT("%s\\%s\\%s"),(wchar_t*)getProgramPath(),backupFolder, TEXT("*"));
    WIN32_FIND_DATA* fileData = new WIN32_FIND_DATA;
    HANDLE hnd = FindFirstFile(backupFolderPath, fileData);
    if (hnd != INVALID_HANDLE_VALUE )
    {
        BOOL isEndOfDirectory = TRUE;
        while (isEndOfDirectory)
        {
            isEndOfDirectory = FindNextFile(hnd, fileData);
            wchar_t delFile[127];
            if (isEndOfDirectory)
            {
                
                wsprintf(delFile, TEXT("%s\\%s\\%s"),(wchar_t*)getProgramPath(),backupFolder, fileData->cFileName);
                res = DeleteFile(delFile);
                if (res == 0)
                {
                    errCode = GetLastError();
                    logDebug(TEXT("Error deleting a file. Error code is %d"), errCode);
                }
            }
            else
            {
                res = FindClose(hnd);
                if (res == 0)
                {
                    errCode = GetLastError();
                    logDebug(logMessage, TEXT("Error closing the handle to the file in backup folder. Error mesage is %d"), errCode);
                }
                hnd = FindFirstFile(backupFolderPath, fileData);
                if (hnd != INVALID_HANDLE_VALUE)
                {
                    wsprintf(delFile, TEXT("%s\\%s\\%s"),(wchar_t*)getProgramPath(),backupFolder, fileData->cFileName);                
                    DeleteFile(delFile);
                    FindClose(hnd);
                }
            }
        }
        wsprintf(backupFolderPath, TEXT("%s\\%s"),(wchar_t*)getProgramPath(),backupFolder);
        BOOL deleted = RemoveDirectory(backupFolderPath);
        if (deleted == FALSE)
        {
            errCode = GetLastError();
            wsprintf(logMessage, TEXT("Error deleting backup directory. Error code is: %d"), errCode);            
        }
        else
        {
            wsprintf(logMessage, TEXT("Backup folder removed."));            
        }
        logDebug(logMessage);
    }
    else
    {
        errCode = GetLastError();
        if (errCode == ERROR_NO_MORE_FILES)
        {        
            wsprintf(backupFolderPath, TEXT("%s\\%s"),(wchar_t*)getProgramPath(),backupFolder);
            BOOL deleted = RemoveDirectory(backupFolderPath);
            if (deleted == FALSE)
            {
                errCode = GetLastError();
                wsprintf(logMessage, TEXT("Error deleting backup directory. Error code is: %d"), errCode);                
            }
            else
            {
                wsprintf(logMessage, TEXT("Backup folder removed."));                
            }            
        }
        else
        {
            wsprintf(logMessage, TEXT("Unable to aquaire a handle to a file in the backup folder. Error code is %d"), errCode);            
        }
        logDebug(logMessage);
    }
    */
    return 0;
}

void FileObjectSyncSource::assign(FileObjectSyncSource& s) {
    setSyncMode(s.getSyncMode());
    setLastSync(s.getLastSync());
    setNextSync(s.getNextSync());
    setRemoteURI(s.getRemoteURI());
    setType(s.getType());
    setLastAnchor(s.getLastAnchor());
    setNextAnchor(s.getNextAnchor());
    setErrorHandler(s.getErrorHandler());
    setFilter(s.getFilter());
    setPath(getPath());
    setType(getType());
    setDir(getDir());
}

