/*
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//
// @author Marco Magistrali
//

#include "spdm/Constants.h"

#include "pim/SettingFunctions.h"
#include <winioctl.h>
#include "ptypes/ptypes.h"
#include "spdm/DMTree.h"
#include "spdm/DMTreeFactory.h"
#include "base/util/utils.h"

#include <commctrl.h>
#include <aygshell.h>
#include <sipapi.h>
#include <cfgmgrapi.h>

#define IOCTL_HAL_GET_DEVICEID CTL_CODE(FILE_DEVICE_HAL, 21, METHOD_BUFFERED, FILE_ANY_ACCESS)

extern "C" __declspec(dllimport) BOOL KernelIoControl(
                                                        DWORD dwIoControlCode, 
                                                        LPVOID lpInBuf, 
                                                        DWORD nInBufSize, 
                                                        LPVOID lpOutBuf, 
                                                        DWORD nOutBufSize, 
                                                        LPDWORD lpBytesReturned
);

USING_PTYPES

/*
* Get the unique Device Identifier from PPC 2002 - 2003
* see more at http://www.pocketpcdn.com/articles/serial_number2002.html
*
* @param deviceId     : the retrieved device id
*/

void GetSerialNumberFromKernelIoControl(wchar_t* deviceId) {
        
    DWORD dwOutBytes = 0;
    const int nBuffSize = 4096;
    byte arrOutBuff[nBuffSize];
    BOOL bRes = TRUE; 
    
    bRes = KernelIoControl(IOCTL_HAL_GET_DEVICEID, 0, 0, arrOutBuff, nBuffSize, &dwOutBytes);

    if (bRes) {
        string strDeviceInfo = string();
        for (unsigned int i = 0; i<dwOutBytes; i++) {                   
            wchar_t tmp [10];
            wsprintf(tmp, TEXT("%02X"), arrOutBuff[i]);
            strDeviceInfo = strDeviceInfo + tmp;

        }
        del(strDeviceInfo, 0, 40);
        del(strDeviceInfo, 2, 3);
        del(strDeviceInfo, 11, 16);
        del(strDeviceInfo, 17, 4);
        
        wsprintf(deviceId, TEXT("fwm-%s"), strDeviceInfo);
                        
    } else {
        deviceId[0] = 0;        
    }               
}

/* 
* Method to get communication settings from device. 
* @param name  : name of the property to retrieve value of
* @param value : value of the property
*/

DWORD WINAPI getConfigParameter (wchar_t* name, wchar_t* value) {

    BCHAR *tmp = NULL;    
    wchar_t* toRet = NULL;
    BCHAR  spds_context [128];
    bsprintf(spds_context, T("%s"), T(SPDS_CONTEXT));
    const char arg[] = "dummy";
    DMTree* dmt = DMTreeFactory::getDMTree(arg);
    ManagementNode *node = dmt->getManagementNode(spds_context); 

    if (wcscmp(name, SPDM_SYNCURL   ) == 0) {        
        tmp = node->getPropertyValue(PROPERTY_SYNC_URL);
    }
    
    else if (wcscmp(name, SPDM_DEVICEID) == 0) {
        
        tmp = node->getPropertyValue(PROPERTY_DEVICE_ID);
        if (strcmp(tmp, T("")) == 0) {
            wchar_t t[128];
            
            GetSerialNumberFromKernelIoControl(t);
            GetSerialNumberFromKernelIoControl(t);
            tmp = toMultibyte(t);
            
        }
    }
    
    else if (wcscmp(name, SPDM_PASSWORD  ) == 0) {   
        tmp = node->getPropertyValue(PROPERTY_PASSWORD);
    }
    else if (wcscmp(name, SPDM_USERNAME  ) == 0) {  
        tmp = node->getPropertyValue(PROPERTY_USERNAME);
    }
    else if (wcscmp(name, SPDM_SERVERNAME) == 0) {  
        tmp = node->getPropertyValue(PROPERTY_SERVER_NAME);
    } 
    else if (wcscmp(name, PROPERTY_SPDM_ENCRYPT) == 0) {  
        tmp = node->getPropertyValue(PROPERTY_ENCRYPTION);
    }
    else if (wcscmp(name, PROPERTY_SPDM_MAX_MSG_SIZE) == 0) {
        tmp = node->getPropertyValue(PROPERTY_MAX_MSG_SIZE);        
    }
    else if (wcscmp(name, PROPERTY_SPDM_READ_BUFFER_SIZE) == 0) {
        tmp = node->getPropertyValue(PROPERTY_READ_BUFFER_SIZE);        
    } 
    
    // overwrite the terminator char \0 of wchar_t* array to read value properly
    toRet = toWideChar(tmp);
    wcscpy(value, toRet);    
    value[wcslen(value)] = L' ';
    
    if (tmp)
        delete [] tmp;     
    tmp = NULL;
    if (toRet) 
        delete [] toRet;
    toRet = NULL;
    
    if (node) 
        delete node;
    node = NULL;

    return 0;
}

/* 
* Method to set communication settings into device. The parameters are retrieved by Visual Basic interface
*
* @param username     : user name
* @param password     : user password
* @param syncURL      : synchronization url
* @param serverName   : the server uri used to address the sync server
* @param errorMsg     : the returned message to vb interface
*/

DWORD WINAPI setConfigParameters(wchar_t *username, wchar_t* password, 
                                 wchar_t* syncURL, wchar_t* serverName, wchar_t* errorMsg) {
        
    wchar_t deviceId [DIM_DEVICE_ID];       
    
    GetSerialNumberFromKernelIoControl(deviceId);
    
    /*
    * Repeat function call GetSerialNumberFromKernelIoControl to fix
    * retrieve serial number from Asus MyPal 620.
    * It's seems this function doesn't work properly if the function
    * is called only one time. 
    */

    GetSerialNumberFromKernelIoControl(deviceId);

    if (wcsstr(syncURL, HTTP_HEADER) == NULL && wcsstr(syncURL, HTTPS_HEADER) == NULL ) {
        wchar_t tmp[256];
        wsprintf(tmp, HTTP_HEADER);
        wcscat(tmp, syncURL);
        wsprintf(syncURL, tmp);    
    }

    BCHAR  *tmp = NULL;
    BCHAR  spds_context [128];
    bsprintf(spds_context, T("%s"), T(SPDS_CONTEXT));
    const char arg[] = "dummy";
    DMTree* dmt = DMTreeFactory::getDMTree(arg);
    ManagementNode *node = dmt->getManagementNode(spds_context); 
   
    node->setPropertyValue(PROPERTY_USERNAME,   _wcc(username));
    node->setPropertyValue(PROPERTY_PASSWORD,   _wcc(password));
    node->setPropertyValue(PROPERTY_SYNC_URL,   _wcc(syncURL));
    node->setPropertyValue(PROPERTY_SERVER_NAME, _wcc(serverName));

    tmp = node->getPropertyValue(PROPERTY_DEVICE_ID);

    if (strcmp(tmp, T("")) == 0) {
        node->setPropertyValue(PROPERTY_DEVICE_ID, _wcc(deviceId));
    }

    wsprintf(errorMsg, TEXT("success"));     
    
    // overwrite the terminator char \0 of wchar_t* array to read value properly
    errorMsg[wcslen(errorMsg)] = L' ';
    
    if (node) 
        delete node;
    node = NULL;

    return 0;
}

DWORD WINAPI setConfigParameter(wchar_t* propertyName, 
                                  wchar_t* propertyValue, wchar_t* errorMsg) {
    
    
    BCHAR *tmp = NULL;        
    BCHAR  spds_context [128];
    bsprintf(spds_context, T("%s"), T(SPDS_CONTEXT));
    const char arg[] = "dummy";
    DMTree* dmt = DMTreeFactory::getDMTree(arg);
    ManagementNode *node = dmt->getManagementNode(spds_context); 
    
    if (wcscmp(propertyName, PROPERTY_SPDM_ENCRYPT) == 0) {
        node->setPropertyValue(PROPERTY_ENCRYPTION, _wcc(propertyValue));            
    } else if (wcscmp(propertyName, PROPERTY_SPDM_MAX_MSG_SIZE) == 0) {
        node->setPropertyValue(PROPERTY_MAX_MSG_SIZE, _wcc(propertyValue));            
    } else if (wcscmp(propertyName, PROPERTY_SPDM_READ_BUFFER_SIZE) == 0) {
        node->setPropertyValue(PROPERTY_READ_BUFFER_SIZE, _wcc(propertyValue));            
    }
    
    
    // overwrite the terminator char \0 of wchar_t* array to read value properly
    errorMsg[wcslen(errorMsg)] = L' ';
    
    if (node)
        delete node;
    node = NULL;

    return 0;
}

/*
* Method to get the synchronization setting parameters.
* 
* @param sourceName     : source name of which get parameter's value
* @param propertyName   : name of property
* @param propertyValue  : value of property
*/

DWORD WINAPI getSourceParameter (LPWSTR sourceName, LPWSTR propertyName,  LPWSTR propertyValue) {
    
    BCHAR *tmp = NULL;        
    wchar_t* toRet = NULL;
    BCHAR  spds_context [128];

    if (wcscmp(sourceName, SYNC_APPOINTMENTS) == 0) {          
        bsprintf(spds_context, T("%s"), T(APPOINTMENTS_CONTEXT));
    }
    else if (wcscmp(sourceName, SYNC_CONTACTS) == 0) {
        bsprintf(spds_context, T("%s"), T(CONTACTS_CONTEXT));        
    }
    else if (wcscmp(sourceName, SYNC_TASKS) == 0) {
        bsprintf(spds_context, T("%s"), T(TASKS_CONTEXT));        
    }
    else if (wcscmp(sourceName, SYNC_NOTES) == 0) {
        bsprintf(spds_context, T("%s"), T(NOTES_CONTEXT));        
    }
    else if (wcscmp(sourceName, SYNC_FILES) == 0) {
        bsprintf(spds_context, T("%s"), T(BRIEFCASE_CONTEXT));        
    }
    else if (wcscmp(sourceName, SYNC_FAVORITES) == 0) {
        bsprintf(spds_context, T("%s"), T(FAVORITES_CONTEXT));        
    }
    else if (wcscmp(sourceName, SYNC_MAILS) == 0) {
        bsprintf(spds_context, T("%s"), T(MAILS_CONTEXT));        
    }

    const char arg[] = "dummy";
    DMTree* dmt = DMTreeFactory::getDMTree(arg);
    ManagementNode *node = dmt->getManagementNode(spds_context); 
    
    if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_SYNC) == 0)  {          // sync
        tmp = node->getPropertyValue(PROPERTY_SOURCE_SYNC);
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_LAST) == 0) {  
        tmp = node->getPropertyValue(PROPERTY_SOURCE_LAST_SYNC);
    }
     else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_URI) == 0) {    // uri
        tmp = node->getPropertyValue(PROPERTY_SOURCE_URI);
    }  
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_SYNC_MODES) == 0) {   // sync modes      
       tmp = node->getPropertyValue(PROPERTY_SOURCE_SYNC_MODES);
    } 
    // dir. It is the parameter only for briefcase and notes
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_DIR) == 0) {    
        tmp = node->getPropertyValue(PROPERTY_SOURCE_DIR); 
        
        char* loc = strchr(tmp, '\\');
        if (loc != NULL) {
        }
        else {
            wchar_t t[128];
            SHGetSpecialFolderPath(NULL, t, CSIDL_PERSONAL, FALSE);                
            tmp = toMultibyte(t);
        }
    }  
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_INBOX) == 0) {    
         tmp = node->getPropertyValue(PROPERTY_SOURCE_INBOX);
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_OUTBOX) == 0) {    
         tmp = node->getPropertyValue(PROPERTY_SOURCE_OUTBOX);
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_TRASH) == 0) {    
         tmp = node->getPropertyValue(PROPERTY_SOURCE_TRASH);
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_SENT) == 0) {    
         tmp = node->getPropertyValue(PROPERTY_SOURCE_SENT);
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_DRAFT) == 0) {    
         tmp = node->getPropertyValue(PROPERTY_SOURCE_DRAFT); 
    } 
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_ATTACH_SIZE) == 0) {
        tmp = node->getPropertyValue(PROPERTY_SOURCE_ATTACH_SIZE); 
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_DOWNLOAD_AGE) == 0) {
        tmp = node->getPropertyValue(PROPERTY_SOURCE_DOWNLOAD_AGE); 
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_BODY_SIZE) == 0) {
        tmp = node->getPropertyValue(PROPERTY_SOURCE_BODY_SIZE); 
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_USE_SIF) == 0) {    
        tmp = node->getPropertyValue(PROPERTY_SOURCE_USE_SIF); 
    } 

    // overwrite the terminator char \0 of wchar_t* array to read value properly
    toRet = toWideChar(tmp);
    wcscpy(propertyValue, toRet);
    propertyValue[wcslen(propertyValue)] = L' ';
    
    if (tmp) 
        delete [] tmp;
    tmp = NULL;

    if (toRet)
        delete [] toRet;
    toRet = NULL;
    
    if (node)
        delete node;
    node = NULL;
        
    return 0;
}

/*
* Method to set the synchronization setting parameters.
* 
* @param sourceName     : source name of which get parameter's value
* @param propertyName   : name of property
* @param propertyValue  : value of property
* @param errorMsg       : returned error message
*/

DWORD WINAPI setSourceParameters (wchar_t* sourceName, wchar_t* propertyName, 
                                  wchar_t* propertyValue, wchar_t* errorMsg) {
    
    BCHAR  spds_context [128];                                      
    bsprintf(spds_context, ROOT_CONTEXT);
    
    if (wcscmp(sourceName, SYNC_APPOINTMENTS) == 0) {  
        bsprintf(spds_context, T("%s"), T(APPOINTMENTS_CONTEXT));
    }
    else if (wcscmp(sourceName, SYNC_CONTACTS) == 0) {
        bsprintf(spds_context, T("%s"), T(CONTACTS_CONTEXT));
    }
    else if (wcscmp(sourceName, SYNC_TASKS) == 0) {
        bsprintf(spds_context, T("%s"), T(TASKS_CONTEXT));
    }
    else if (wcscmp(sourceName, SYNC_NOTES) == 0) {
        bsprintf(spds_context, T("%s"), T(NOTES_CONTEXT));
    }
    else if (wcscmp(sourceName, SYNC_FILES) == 0) {
        bsprintf(spds_context, T("%s"), T(BRIEFCASE_CONTEXT));
    }
    else if (wcscmp(sourceName, SYNC_FAVORITES) == 0) {
        bsprintf(spds_context, T("%s"), T(FAVORITES_CONTEXT));
    }
    else if (wcscmp(sourceName, SYNC_MAILS) == 0) {
        bsprintf(spds_context, T("%s"), T(MAILS_CONTEXT));
    }    

    const char arg[] = "dummy";
    DMTree* dmt = DMTreeFactory::getDMTree(arg);
    ManagementNode *node = dmt->getManagementNode(spds_context); 

    if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_SYNC) == 0) {     
        node->setPropertyValue(PROPERTY_SOURCE_SYNC, _wcc(propertyValue));
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_LAST) == 0) {     
        node->setPropertyValue(PROPERTY_SOURCE_LAST_SYNC, _wcc(propertyValue));
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_URI) == 0) {
        node->setPropertyValue(PROPERTY_SOURCE_URI, _wcc(propertyValue));
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_DIR) == 0) {

        wchar_t t [256];
        wsprintf(t, propertyValue);

        wchar_t* loc = wcschr(t, L'\\');
        if (loc != NULL)
            wsprintf(t, loc);
        else
            wsprintf(t, TEXT("\\"));
        
        node->setPropertyValue(PROPERTY_SOURCE_DIR, _wcc(propertyValue));
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_INBOX) == 0) {
        node->setPropertyValue(PROPERTY_SOURCE_INBOX, _wcc(propertyValue));
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_OUTBOX) == 0) {
        node->setPropertyValue(PROPERTY_SOURCE_OUTBOX, _wcc(propertyValue));
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_SENT) == 0) {
        node->setPropertyValue(PROPERTY_SOURCE_SENT, _wcc(propertyValue));
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_DRAFT) == 0) {
        node->setPropertyValue(PROPERTY_SOURCE_DRAFT, _wcc(propertyValue));
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_TRASH) == 0) {
        node->setPropertyValue(PROPERTY_SOURCE_TRASH, _wcc(propertyValue));
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_ATTACH_SIZE) == 0) {
        node->setPropertyValue(PROPERTY_SOURCE_ATTACH_SIZE, _wcc(propertyValue));
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_DOWNLOAD_AGE) == 0) {
        node->setPropertyValue(PROPERTY_SOURCE_DOWNLOAD_AGE, _wcc(propertyValue));
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_BODY_SIZE) == 0) {
        node->setPropertyValue(PROPERTY_SOURCE_BODY_SIZE, _wcc(propertyValue));
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_USE_SIF) == 0) {
        node->setPropertyValue(PROPERTY_SOURCE_USE_SIF, _wcc(propertyValue));

        //
        // to set the encoding parameter automatically
        //
        if (wcscmp(propertyValue, TEXT("1")) == 0) {
            node->setPropertyValue(PROPERTY_SOURCE_ENCODING, T("b64"));
        } else {
            node->setPropertyValue(PROPERTY_SOURCE_ENCODING, T("plain/text"));
        }

    }
        
    wsprintf(errorMsg, TEXT("success"));
    errorMsg[wcslen(errorMsg)] = L' ';
    
    if (node)
        delete node;
    node = NULL;
    
    return 0;

}

DWORD WINAPI setClientConfiguration(wchar_t* sourceName, wchar_t* propertyName, 
                                     wchar_t* propertyValue, wchar_t* errorMsg)  {
    
    BCHAR  spds_context [128];                                      
    bsprintf(spds_context, T("%s"), ROOT_CONTEXT);
    
    const char arg[] = "dummy";
    DMTree* dmt = DMTreeFactory::getDMTree(arg);
    ManagementNode *node = dmt->getManagementNode(spds_context); 

    if (sourceName == NULL || wcscmp(sourceName, TEXT("")) == 0) {
        
        if (wcscmp(propertyName, PROPERTY_SPDM_LOG_LEVEL) == 0) {
            node->setPropertyValue(PROPERTY_LOG_LEVEL, _wcc(propertyValue));
        } else if (wcscmp(propertyName, PROPERTY_SPDM_PATH_INFO) == 0) {
            node->setPropertyValue(PROPERTY_PATH_INFO, _wcc(propertyValue));
        } else if (wcscmp(propertyName, PROPERTY_SPDM_PUSH_NOTIFICATION) == 0) {
            node->setPropertyValue(PROPERTY_PUSH_NOTIFICATION, _wcc(propertyValue));
        } else if (wcscmp(propertyName, PROPERTY_SPDM_POLLING_NOTIFICATION) == 0) {
            node->setPropertyValue(PROPERTY_POLLING_NOTIFICATION, _wcc(propertyValue));
        } else if (wcscmp(propertyName, PROPERTY_SPDM_SERVER_NOTIFIED) == 0) {
            node->setPropertyValue(PROPERTY_SERVER_NOTIFIED, _wcc(propertyValue));
        } else if (wcscmp(propertyName, PROPERTY_SPDM_NOTIFICATION_PORT) == 0) {
            node->setPropertyValue(PROPERTY_NOTIFICATION_PORT, _wcc(propertyValue));
        } else if (wcscmp(propertyName, PROPERTY_SPDM_SMS_WAP_PUSH) == 0) {
            node->setPropertyValue(PROPERTY_SMS_WAP_PUSH, _wcc(propertyValue));
        } 
                                   
                                   
    }
    if (node)
        delete node;
    node = NULL;

    return 0;
}

/*
* The method recall the other getClientConfiguration and set the variable isFromDB as true.
* This variable means that the caller is the VB and so it's necessary remove the final char '\0' 
* to permit the vb to understand the return string properly
*/

DWORD WINAPI getClientConfiguration (wchar_t* sourceName, wchar_t* propertyName, 
                                     wchar_t* propertyValue) {   

    return getClientConfigurationInternal (sourceName, propertyName, propertyValue, TRUE);
}

/*
* Method to get the client configurations. These property are set by installing process.
* These are extra parameters not used in synchronization process 
* but are needed for example to show or hide some fields in GUI interface.
* 
* @param sourceName     : source name of which get parameter's value
* @param propertyName   : name of property
* @param propertyValue  : value of property
*/

DWORD getClientConfigurationInternal (wchar_t* sourceName, wchar_t* propertyName, 
                                     wchar_t* propertyValue, BOOL isFromVB) {    

    const int VALUESIZE = 1024;
    int nc = 0;
    
    BCHAR  *tmp = NULL;
    wchar_t* toRet = NULL;
    
    BCHAR  spds_context [128];                                      
    bsprintf(spds_context, T("%s"), ROOT_CONTEXT);
    const char arg[] = "dummy";
    DMTree* dmt = DMTreeFactory::getDMTree(arg);
    ManagementNode *node = dmt->getManagementNode(spds_context); 

    wchar_t buf[1024]; 
       
    /*
    * if sourceName is empty then is a property of root context
    */
    if (sourceName == NULL || wcscmp(sourceName, TEXT("")) == 0) {
        
        if (wcscmp(propertyName, PROPERTY_SPDM_REMOTE_SETTINGS) == 0) {
            tmp = node->getPropertyValue(PROPERTY_REMOTE_SETTINGS); 
        }
        else if (wcscmp(propertyName, PROPERTY_SPDM_LOG_LEVEL) == 0) {
            tmp = node->getPropertyValue(PROPERTY_LOG_LEVEL);
        }
        else if (wcscmp(propertyName, PROPERTY_SPDM_ENCRYPT) == 0) {
            tmp = node->getPropertyValue(PROPERTY_ENCRYPTION);
        }
        else if (wcscmp(propertyName, PROPERTY_SPDM_PATH_INFO) == 0) {
            tmp = node->getPropertyValue(PROPERTY_PATH_INFO);
        }
        else if (wcscmp(propertyName, PROPERTY_SPDM_PUSH_NOTIFICATION) == 0) {
            tmp = node->getPropertyValue(PROPERTY_PUSH_NOTIFICATION);
        }
        else if (wcscmp(propertyName, PROPERTY_SPDM_NOTIFICATION_PORT) == 0) {
            tmp = node->getPropertyValue(PROPERTY_NOTIFICATION_PORT);
        }
        else if (wcscmp(propertyName, PROPERTY_SPDM_POLLING_NOTIFICATION) == 0) {
            tmp = node->getPropertyValue(PROPERTY_POLLING_NOTIFICATION);
        }
        else if (wcscmp(propertyName, PROPERTY_SPDM_SERVER_NOTIFIED) == 0) {
            tmp = node->getPropertyValue(PROPERTY_SERVER_NOTIFIED);
        }        
        else if (wcscmp(propertyName, PROPERTY_SPDM_SMS_WAP_PUSH) == 0) {
            tmp = node->getPropertyValue(PROPERTY_SMS_WAP_PUSH);
        }
        else
            tmp = T("");
        
        toRet = toWideChar(tmp);
        wcscpy(propertyValue, toRet);                                
        
        if (isFromVB != NULL && isFromVB == TRUE)
            propertyValue[wcslen(propertyValue)] = L' ';
        
        goto finally;
        
    }               

    wsprintf(buf, TEXT(""));                      

    wcscpy(propertyValue, buf);                                
    propertyValue[wcslen(propertyValue)] = L' ';

finally:    
    if (toRet) 
        delete [] toRet;
    toRet = NULL;

    if(tmp)
        delete [] tmp;
    tmp = NULL;
    
    if (node) {delete node; node = NULL; }
    return 0;    
}
