/*
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <windows.h>
#include <Winbase.h>
#include "base/Log.h"
#include "base/util/utils.h"
#include "base/base64.h"

#include "pim/Utils.h"
#include "pim/ActionLog.h"

#include "pim/SettingFunctions.h"

#include <tchar.h>
#include "spdm/ManagementNode.h"
#include "spdm/DMTree.h"
#include "spdm/DMTreeFactory.h"
#include "base/quoted-printable.h"
#include "spds/DataTransformerFactory.h"
#include "spds/DataTransformer.h"


// Timezones that are changing their rule in 2007
#define TZ_EASTERN_US TEXT("Eastern Standard Time")
#define TZ_CENTRAL_US TEXT("Central Standard Time")
#define TZ_MOUNTAIN TEXT("Mountain Standard Time")
#define TZ_PACIFIC_US TEXT("Pacific Standard Time")
#define TZ_ALASKA TEXT("Alaska Standard Time")


// DST conversion items
#define IS_LEAP_YEAR(y) (((y) & 3) == 0)

#define BASE_DOW          4                  // 1/1/1970 was a Thursday.
#define SECONDS_IN_A_DAY  (24L * 60L * 60L)  // Number of seconds in one day.

// Month to Year Day conversion array.
int M2YD[] = {
   0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334, 365
};
// Month to Leap Year Day conversion array.
int M2LYD[] = {
   0, 31, 60, 91, 121, 152, 182, 213, 244, 274, 305, 335, 366
};

USING_PTYPES

/**
* Open POOM Application object
*/
IPOutlookApp* getOutlookApp() {
    
    // Initialization and creation IPOutlookApp
    
    IPOutlookApp    *polApp; // outlook app    
    HRESULT hr;           
    if (FAILED (CoInitializeEx (NULL, 0))) 
        return NULL;
    hr = CoCreateInstance (CLSID_Application, NULL, CLSCTX_INPROC_SERVER, IID_IPOutlookApp, (LPVOID*)&polApp);
    if (FAILED(hr)) 
        return NULL;
    hr = polApp->Logon (NULL);
    if (FAILED (hr))
        return NULL;    
    if (FAILED(hr))
        return NULL;     
    
    return polApp;

}

/**
* Close POOM Application object
*/

void releaseOutlookApp(IPOutlookApp* polApp) {
    if (polApp) {
        polApp->Logoff ();
        polApp->Release ();
    }
}

/////////////////////////////////////////////////////////////////////////
//FIXME
void logError (const wchar_t *pFormat, ...) {
    va_list argList;
    va_start (argList, pFormat);
    LOG.printMessageW(LOG_DEBUG, pFormat, argList);
    va_end(argList);
}

void logInfo (const wchar_t *pFormat, ...) {
    if(LOG.isLoggable(LOG_LEVEL_INFO)){
        va_list argList;
        va_start (argList, pFormat);        
        LOG.printMessageW(LOG_INFO, pFormat, argList);
        va_end(argList);
    }
}

void logDebug (const wchar_t *pFormat, ...) {
    if(LOG.isLoggable(LOG_LEVEL_DEBUG)){
        va_list argList;
        va_start (argList, pFormat);        
        LOG.printMessageW(LOG_DEBUG, pFormat, argList);
        va_end(argList);
    }
}
/////////////////////////////////////////////////////////////////////////


/*
 * The method extracts the content of a tag into a XML message. It is supposed the
 * message is a valid XML message. It returns NULL in case the tag is not
 * found or the XML fragment is not in the expected form.
 * The returned pointer (if not NULL) is allocated with the new operator and
 * must be discarded with the operator delete.
 *
 * @param xml : the xml fragment
 * @param tag : the tag we want the content
 * @param pos : (OUTPUT) the position just after the closing tag (ignored if NULL)
 *
 */
wchar_t* getElementContent(wchar_t* xml, wchar_t* tag, unsigned int* pos) {
    wchar_t* p1       = NULL;
    wchar_t* p2       = NULL;
    wchar_t* ret      = NULL;
    BOOL charFound    = FALSE;

    wchar_t openTag[50];
    wchar_t closeTag[50];

    unsigned int xmlLength = wcslen(xml);
    unsigned int l = wcslen(tag);

    if (pos != NULL) {
        *pos = 0;
    }

    wsprintf(openTag, TEXT("<%s>"), tag);
    wsprintf(closeTag, TEXT("</%s>"), tag);

    p1 = wcsstr(xml, openTag);

    if (p1 == NULL) { // tag can have attributes
        //
        // This is abcxyz
        //
        //goto finally;

        // if p1 is null I try to discover the next '>' char to close the tag. If does not exist 
        // return NULL
        
        // try to find "<tagName/>". If found it return null.
        wsprintf(openTag, TEXT("<%s/>"), tag);
        p1 = wcsstr(xml, openTag);

        if (p1 != NULL) {
            ret = new wchar_t[2];
            wsprintf(ret, TEXT(""));
            goto finally;
        }
        
        // try to find "<tagName"
        wsprintf(openTag, TEXT("<%s"), tag);
        p1 = wcsstr(xml, openTag);

        if (p1 == NULL) {
            goto finally;
        }
        
        p1 = p1 + l + 1;   // <body_ 

        for (unsigned int k = 0; k < xmlLength; k++) { // Suppose max length as the xml string
            p1 = p1 + 1;
            if (*p1 == 0) {
                goto finally;
            }
            else if (*p1 == '>') {
                charFound = TRUE;
                p1 = p1 + 1;
                break;
            }            
        }
        if (!charFound)
            goto finally;
        
        
    } else {  // tag doesn't have attribute. Original version

        p1 = p1+l+2;
        
    }
    
    if (*p1 == 0) {
        //
        // This is abc<tag>\0
        //
        goto finally;
    }

    p2 = wcsstr(p1, closeTag);

    if (p2 == NULL) {
        //
        // This is abc<tag>xyz\0
        //
        goto finally;
    }

    ret = new wchar_t[p2-p1+1];

    wcsncpy(ret, p1, p2-p1);
    ret[p2-p1] = 0;

    if (pos != NULL) {
        *pos = p2-xml+l+3;
    }
   
finally:
   
    return ret;

}

/*
* The method is used to convert VARIANT_BOOL type in the same string type
* With variant bool TRUE = -1, FALSE = 0
* Translate in BOOL i which TRUE = 1 and FALSE = 0
* @param localTemp : the variable that contain old and new value 
*/

void normalizeBoolean(wchar_t* localTemp) {
        
    if (wcscmp(localTemp, TEXT("-1")) == 0) {
        wsprintf(localTemp, TEXT("1"));
    } else {
        wsprintf(localTemp, TEXT("0"));
    }
    /*
    if (wcscmp(localTemp, TEXT("-1")) == 0) {
        wsprintf(localTemp, TEXT("True"));
    } else {
        wsprintf(localTemp, TEXT("False"));
    }*/
    
}

/*
* The method replaces &lt; or &gt; with < or > in server response syncML message. 
* 
* @param s : the string to be replaced
*/
void replaceAmpLGt(string &s) {
        
    int position = 0;
    
    // search for &lt;
    position = pos(TEXT("&lt;"), s);
    while (position != -1) {
        del(s, position, 4);
        ins(TEXT("<"), s, position);
        position = pos(TEXT("&lt;"), s);
    }
    // now search for &lt;
    position = 0;
    position = pos(TEXT("&gt;"), s);
    while (position != -1) {
        del(s, position, 4);
        ins(TEXT(">"), s, position);
        position = pos(TEXT("&gt;"), s);
    }
    
}

/*
* The method removes \n from the start if there is.
* The start 1n was a bug that has been fixed in new SyncClient API version.
* It works on original array to save memory.
*
* @param ptr : the array to be modify
*/

void removeStartCarriage(wchar_t** ptr) {
   
    // if start with CRLF, 
    if ( ptr != NULL &&
        *ptr != NULL &&
        (int)((*ptr)[0]) == 13 && (int)((*ptr)[1]) == 10)
        *ptr = &((*ptr)[2]);


}

/*
* The method removes \r\n from the end if there is.
* This method is used to remove carriage\return in reading line from file notes.dat, 
* briefcase.dat and favorites.dat.
* So it will be possible compare properly file names retrived by file system
* and these file name
*
* @param ptr : the filename ended with \r\n or \n
*/

void removeEndCarriage(wchar_t** ptr) {
    int length = 0;
    if (*ptr != NULL)
        length = wcslen(*ptr);
    
            
    // if end with \n, 
    if ( ptr != NULL &&
        *ptr != NULL &&
        (int)((*ptr)[length -1]) == 10)
        (*ptr)[length -1] = 0;

    if (*ptr != NULL)
        length = wcslen(*ptr);
    
    // if there is also \r
    if ( ptr != NULL &&
        *ptr != NULL &&
        (int)((*ptr)[length -1]) == 13)
        (*ptr)[length -1] = 0;

}

void encodeSpecialChar(string &s) {
        
    int position = 0, previousPosition = 0;
       
    //
    // &
    //
    position = pos(TEXT("&"), s);
    while (position != -1) {        
        ins(TEXT("&amp;"), s, position);
        del(s, position + 5, 1);        
        position = position + 2;
        previousPosition = position;                
        position = pos(TEXT("&"), &s[position]);
        
        if (position == -1) {
            break;
        }
        position = previousPosition + position;
    }  

    //
    // <
    //    
    position = 0;
    previousPosition = 0;
    position = pos(TEXT("<"), s);
    while (position != -1) {        
        ins(TEXT("&lt;"), s, position);
        del(s, position + 4, 1);        
        position = position + 2;
        previousPosition = position;                
        position = pos(TEXT("<"), &s[position]);
        
        if (position == -1) {
            break;
        }
        position = previousPosition + position;
    }  

    //
    // >
    //    
    position = 0;
    previousPosition = 0;
    position = pos(TEXT(">"), s);
    while (position != -1) {        
        ins(TEXT("&gt;"), s, position);
        del(s, position + 4, 1);        
        position = position + 2;
        previousPosition = position;                
        position = pos(TEXT(">"), &s[position]);
        
        if (position == -1) {
            break;
        }
        position = previousPosition + position;
    } 
      
}


void decodeSpecialChar(string &s) {
        

    int position = 0, previousPosition = 0;
       
    //
    // &
    //
    position = pos(TEXT("&amp;"), s);
    while (position != -1) {        
        ins(TEXT("&"), s, position);
        del(s, position + 1, 5);                
        previousPosition = position;                
        position = pos(TEXT("&amp;"), &s[position]);
        
        if (position == -1) {
            break;
        }
        position = previousPosition + position;
    }  
    
    //
    // <
    //
    position = 0;
    previousPosition = 0;
    position = pos(TEXT("&lt;"), s);
    while (position != -1) {        
        ins(TEXT("<"), s, position); 
        del(s, position + 1, 4);               
        previousPosition = position;                
        position = pos(TEXT("&lt;"), &s[position]);
        
        if (position == -1) {
            break;
        }
        position = previousPosition + position;
    }  
    
    //
    // >
    //
    position = 0;
    previousPosition = 0;
    position = pos(TEXT("&gt;"), s);
    while (position != -1) {        
        ins(TEXT(">"), s, position);
        del(s, position + 1, 4);                
        previousPosition = position;                
        position = pos(TEXT("&gt;"), &s[position]);
        
        if (position == -1) {
            break;
        }
        position = previousPosition + position;
    } 
      
}

/*
* The method return current system time in VariantTime format 
* used in FILEFileManagement
*/

DATE getSystemTime() {

    SYSTEMTIME t;
    DATE date;
    BOOL isUTC = TRUE;
    wchar_t localTemp [DIM_LOCAL_TEMP];
    
    GetLocalTime (&t);
    
    // conversion from actual time to UTC
    // if propertyValue PROPERTY_SOURCEPROP_SYSTEMTIME_TO_LOCALTIME == 1 then 
    //wchar_t propertyValue[10];
    //getClientConfigurationInternal (NULL, PROPERTY_SOURCEPROP_SPDM_SYSTEMTIME_TO_LOCALTIME, propertyValue, FALSE); 
    
    //if (wcscmp(propertyValue, TEXT("1")) == 0) {
    //    isUTC = TRUE;    
    //}
    
    if (isUTC) {
        localTimeToUTC(t);
    }

    wchar_t month [10];
    wchar_t day   [10];
    wchar_t hour  [10];
    wchar_t minute[10];
    wchar_t second[10];
    
    // month
    if (t.wMonth < 10)
        wsprintf(month, TEXT("0%i"), t.wMonth);
    else
        wsprintf(month, TEXT("%i"), t.wMonth);
    
    // day
    if (t.wDay  < 10)
        wsprintf(day, TEXT("0%i"), t.wDay);
    else
        wsprintf(day, TEXT("%i"), t.wDay);
    
    // hour
    if (t.wHour  < 10)
        wsprintf(hour, TEXT("0%i"), t.wHour);
    else
        wsprintf(hour, TEXT("%i"), t.wHour);

    // minute
    if (t.wMinute < 10)
        wsprintf(minute, TEXT("0%i"), t.wMinute);
    else
        wsprintf(minute, TEXT("%i"), t.wMinute);

    // second
    if (t.wSecond < 10)
        wsprintf(second, TEXT("0%i"), t.wSecond);
    else
        wsprintf(second, TEXT("%i"), t.wSecond);
    
    if (isUTC) {
        wsprintf(localTemp, TEXT("%i%s%sT%s%s%sZ"), t.wYear, month, day, hour, minute, second);
    } else {
        wsprintf(localTemp, TEXT("%i%s%sT%s%s%s"), t.wYear, month, day, hour, minute, second);
    }
    //MessageBox (NULL, localTemp, TEXT ("Update"), MB_SETFOREGROUND |MB_OK); 
    systemTimeToDouble(localTemp, &date, NULL);
    
    return date;

}

/*
* The method return current system time from VariantTime format 
*
* @param tempDate : the variable to write date in
* @param date     : the date in variant time format
*/

void doubleToSystemTime(wchar_t* tempDate, DATE date) {
    doubleToSystemTimeHourZero(tempDate, date, FALSE);
}

/*
* Is like doubleToSystemTime but set hour to 0. It used directly for anniversary and birthday.
* if not the time represented by date is the same in which the date is written 
* on the device
*/
void doubleToSystemTimeHourZero(wchar_t* tempDate, DATE date, BOOL isHourToSetToZero) {
        
    SYSTEMTIME t;
    VariantTimeToSystemTime(date, &t); 
    
    if (isHourToSetToZero) {
        t.wHour = 0;
        t.wMinute = 0;
        t.wSecond = 0;
    }

    localTimeToUTC2(t);

    wsprintf(tempDate, TEXT("%d%02d%02dT%02d%02d%02dZ"),
             t.wYear, t.wMonth, t.wDay,
             t.wHour, t.wMinute, t.wSecond);
}


/*
* Is like doubleToSystemTime but set hour to 0. It used directly for anniversary and birthday.
* if not the time represented by date is the same in which the date is written 
* on the device
*/
void doubleToSystemTimeBirthday(wchar_t* tempDate, DATE date) {
        
    SYSTEMTIME t;
    
    VariantTimeToSystemTime(date, &t); 
    
    t.wHour = 0;
    t.wMinute = 0;
    t.wSecond = 0;

    wchar_t month [10];
    wchar_t day   [10];
    
    // month
    if (t.wMonth < 10)
        wsprintf(month, TEXT("0%i"), t.wMonth);
    else
        wsprintf(month, TEXT("%i"), t.wMonth);
    
    // day
    if (t.wDay  < 10)
        wsprintf(day, TEXT("0%i"), t.wDay);
    else
        wsprintf(day, TEXT("%i"), t.wDay);
    
    
    wsprintf(tempDate, TEXT("%i-%s-%s"), t.wYear, month, day);
    

}

/*
* The method transform a string date in format "yyyy-MM-dd" to DATE
*
* @param dataString : the string in yyyy-MM-dd format
* @param date       : the returned value into VariantTime format
* @action           : the parameter to set manually hour and minute. If NULL current value are used
*/
void systemTimeToDoubleBirthday(wchar_t* dataString, DATE* date) {
    systemTimeToDoubleBirthFormat(dataString, date, TEXT("START"));
}

/*
* It is used to transform yyyy-MM-dd format date into DATE. type parameter is to set
* the hours and minutes. 
* Parameter END to set hour 23:59
* Parameter START to set hour 00:00
* Set the hour to 00:00 is used for birthday, anniversary and startDate of appoitment
* when it is all day event. Set hour 23:59 for endDate when an appointment it is allDayEvent
*/
void systemTimeToDoubleBirthFormat(wchar_t* dataString, DATE* date, wchar_t* type) {

    wchar_t* token  = NULL;
    wchar_t* dummy  = NULL;
    wchar_t* Ztoken = NULL;
    
    wchar_t internalString[DIM_LOCAL_TEMP];
    wsprintf(internalString, dataString);
    wchar_t tmp[50];
    
    int ret = 0;
    
    SYSTEMTIME t;
    TIME_ZONE_INFORMATION tmz;        
    
    GetTimeZoneInformation(&tmz);
    
    Ztoken = wcschr(dataString, '-');
    if (Ztoken == NULL) {
        systemTimeToDouble(dataString, date, NULL);
        return;
    }
    
    //  TEXT("yyyy-MM-dd") 
    token = wcstok(internalString, TEXT("-"));

    if (token != NULL) {
            
        //yyyy
        wcsncpy(tmp, dataString, 4);
        tmp[4] = 0;
        swscanf(tmp, TEXT("%d"),&t.wYear);
        //MM
        wcsncpy(tmp, dataString + 5, 2);
        tmp[2] = 0;
        swscanf(tmp, TEXT("%d"),&t.wMonth);
        //dd
         wcsncpy(tmp, dataString + 8, 2);
        tmp[2] = 0;
        swscanf(tmp, TEXT("%d"),&t.wDay);
        
        if (wcscmp(type, TEXT("END")) == 0) {
        
            t.wHour = 23;
            t.wMinute = 59;
            t.wSecond = 0;
        } else {
        
            t.wHour = 0;
            t.wMinute = 0;
            t.wSecond = 0;
        } 
                   
        // set to default
        t.wMilliseconds = 0;                    
        t.wDayOfWeek = 0;                       
        
        ret = SystemTimeToVariantTime (&t, date);
       
            
    } else {
        
        (*date) = 949998.000000;
        
    }
}


/*
* The method transform a date in format "YYYYMMDDThhmmss" or TEXT("YYYYMMDDThhmmssZ") with hour 0-24 into VariantTime format.
*
* @param dataString : the string in "YYYYMMDDThhmmss" TEXT("YYYYMMDDThhmmssZ") format
* @param date       : the returned value into VariantTime format
* @action           : the parameter to set manually hour and minute. If NULL current value are used
*/
void systemTimeToDouble(wchar_t* dataString, DATE* date, wchar_t* action) {

    wchar_t* token  = NULL;
    wchar_t* dummy  = NULL;
    wchar_t* Ztoken = NULL;
    BOOL isUTC      = FALSE;
    wchar_t internalString[DIM_LOCAL_TEMP];
    wsprintf(internalString, dataString);
    wchar_t tmp[50];
    
    int ret = 0;
    
    SYSTEMTIME t;
    TIME_ZONE_INFORMATION tmz;        
    
    GetTimeZoneInformation(&tmz);
    
    //  TEXT("20041006T203457") or TEXT("20041006T203457Z")
    token = wcstok(internalString, TEXT("T"));
    Ztoken = wcschr(dataString, 'Z');
    if (Ztoken != NULL) {
        isUTC = TRUE;
    }
    
    if (token != NULL) {
            
        //yyyy
        wcsncpy(tmp, dataString, 4);
        tmp[4] = 0;
        swscanf(tmp, TEXT("%d"),&t.wYear);
        //MM
        wcsncpy(tmp, dataString + 4, 2);
        tmp[2] = 0;
        swscanf(tmp, TEXT("%d"),&t.wMonth);
        //dd
         wcsncpy(tmp, dataString + 6, 2);
        tmp[2] = 0;
        swscanf(tmp, TEXT("%d"),&t.wDay);

        if (action != NULL && wcscmp(action, TEXT("0000")) == 0) {
            t.wHour = 0;
            t.wMinute = 0;
            t.wSecond = 0;
            
        } else if (action != NULL && wcscmp(action, TEXT("2359")) == 0) {
            t.wHour = 23;
            t.wMinute = 59;
            t.wSecond = 59;
        } else {
            if (wcslen(dataString) > 9) {
                //HH
                wcsncpy(tmp, dataString + 9, 2);
                tmp[2] = 0;
                swscanf(tmp, TEXT("%d"),&t.wHour);
                                
                //mm
                 wcsncpy(tmp, dataString + 11, 2);
                tmp[2] = 0;
                swscanf(tmp, TEXT("%d"),&t.wMinute);
                //ss
                 wcsncpy(tmp, dataString + 13, 2);
                tmp[2] = 0;
                swscanf(tmp, TEXT("%d"),&t.wSecond);
                

            } else {
                t.wHour = 0;
                t.wMinute = 0;
                t.wSecond = 0;
            }

        }
        // set to default
        t.wMilliseconds = 0;                    
        t.wDayOfWeek = 0;                       
                
        
        if(isUTC) {
            UTCToLocalTime2(t);
        }
        ret = SystemTimeToVariantTime (&t, date);
       
            
    } else {
        
        (*date) = 949998.000000;
        
    }

}

/*
* return date and time of a date and if it is in utc or not
*/
BOOL getDateAndTimeAsIs(wchar_t* dataString, DATE* date) {

    wchar_t* token  = NULL;
    wchar_t* dummy  = NULL;
    wchar_t* Ztoken = NULL;
    BOOL isUTC      = FALSE;
    wchar_t internalString[DIM_LOCAL_TEMP];
    wsprintf(internalString, dataString);
    wchar_t tmp[50];
    
    int ret = 0;
    
    SYSTEMTIME t;
    TIME_ZONE_INFORMATION tmz;        
    
    GetTimeZoneInformation(&tmz);
    
    //  TEXT("20041006T203457") or TEXT("20041006T203457Z")
    token = wcstok(internalString, TEXT("T"));
    Ztoken = wcschr(dataString, 'Z');
    if (Ztoken != NULL) {
        isUTC = TRUE;
    }
    
    if (token != NULL) {
            
        //yyyy
        wcsncpy(tmp, dataString, 4);
        tmp[4] = 0;
        swscanf(tmp, TEXT("%d"),&t.wYear);
        //MM
        wcsncpy(tmp, dataString + 4, 2);
        tmp[2] = 0;
        swscanf(tmp, TEXT("%d"),&t.wMonth);
        //dd
         wcsncpy(tmp, dataString + 6, 2);
        tmp[2] = 0;
        swscanf(tmp, TEXT("%d"),&t.wDay);
        
        if (wcslen(dataString) > 9) {
            //HH
            wcsncpy(tmp, dataString + 9, 2);
            tmp[2] = 0;
            swscanf(tmp, TEXT("%d"),&t.wHour);
                            
            //mm
             wcsncpy(tmp, dataString + 11, 2);
            tmp[2] = 0;
            swscanf(tmp, TEXT("%d"),&t.wMinute);
            //ss
             wcsncpy(tmp, dataString + 13, 2);
            tmp[2] = 0;
            swscanf(tmp, TEXT("%d"),&t.wSecond);
            
        } else {
            t.wHour = 0;
            t.wMinute = 0;
            t.wSecond = 0;
        }

        
        // set to default
        t.wMilliseconds = 0;                    
        t.wDayOfWeek = 0;                       
                                
        ret = SystemTimeToVariantTime (&t, date);
       
            
    } else {
        
        (*date) = 949998.000000;
        
    }
    return isUTC;

}


/*
* The method returns the file name from the complete path. 
*
* @param path     : the complete path
* @param filename : the filename retrieved form path
*/

void getFileName(const wchar_t* path, wchar_t* filename) {
    wchar_t* p;

    p = wcsrchr(path, L'\\');

    if (p == NULL) {
        wcscpy(filename, path);
        return;
    }

    wcscpy(filename, p+1);
}

/*
* The method converts the path separator "\" with "@".
* Used for favorites items to permit to sync files and directory with a filesystem sync source.
* "\" char is not permitted in file name on windows machine
* This conversion happens on send items to server
*
* @param filename : the file name retrieved to be modified
*/

void convertAtSlash(wchar_t* filename) {
    
    int position = 0;
    string s = string();
    s += (wchar_t*)filename;

    position = pos(TEXT("@"), s);
    while (position != -1) {
        del(s, position, 1);
        ins(TEXT("\\"), s, position);
        position = pos(TEXT("@"), s);
    }        

    wsprintf(filename, (wchar_t*)s);
}

/*
* The method converts the separator "@" with "\".
* Used for favorites items to permit to sync files and directory with a filesystem sync source.
* "\" char is not permitted in file name on windows machine
* This conversion happens on returned items from server
*
* @param filename : the file name retrieved to be modified
*/

void convertSlashAt(wchar_t* filename) {
    
    int position = 0;
    string s = string();
    s += (wchar_t*)filename;

    // search for \;
    position = pos(TEXT("\\"), s);
    while (position != -1) {
        del(s, position, 1);
        ins(TEXT("@"), s, position);
        position = pos(TEXT("\\"), s);
    }
    

    wsprintf(filename, (wchar_t*)s);
}

/*
* The method get the complete directory name starting from path.
* 
* @param path     : the path and filename
* @param dirname  : the returned dir name
*/
void getDirName(const wchar_t* path, wchar_t* dirname) {

    string s = string();
    int position = 0;

    s += (wchar_t*) path;
    position = rpos(L'\\', s);
    
    if (position == -1) {
        wsprintf(dirname, path);
        return;
    }

    del(s, position, length(s) - position);
    if (dbg)
        MessageBox (NULL, (wchar_t*)s, TEXT ("1 in getDirName"), MB_SETFOREGROUND |MB_OK);
    
    getFileName((wchar_t*)s, dirname);
    
    if (dbg)
        MessageBox (NULL, dirname, TEXT ("dirName in getDirName"), MB_SETFOREGROUND |MB_OK);

}

/*
* return the length for the base64 array starting from length of the original array
*/
int lengthForB64(int len) {
    
    int modules = 0;
    int ret     = 0;

    modules = len % 3;
    if (modules == 0) {
        ret = 4 * (len / 3);

    } else {
        ret = 4 * ((len/3) + 1);

    }
    return ret;


}

/*
* encode in base 64 starting from char* that was trasnformed into utf8
*/
wchar_t* encodeBase64UTF8(wchar_t* toEncode) {
   
    if (toEncode == NULL)
        return NULL;

    char*    b64 = NULL;
    TransformationInfo info;
    char* tmp = toMultibyte(toEncode);    
    
    DataTransformer* dtb64 = DataTransformerFactory::getEncoder(T("b64"));

    if (dtb64 == NULL) {
        return NULL;
    }
    info.size = strlen(tmp);

    b64 = dtb64->transform((char*)tmp, info);
    
    wchar_t* ret = toWideChar(b64);
    
    /*
    char* tmp = toMultibyte(toEncode);
    int len = (wcslen(toEncode)/3+1)<<2;
    char* b64tmp = new char[len];
        memset(b64tmp, 0, len);
    int retLen = b64_encode(b64tmp, tmp, len);      
    
    wchar_t* ret = toWideChar(b64tmp);
    */
    if (tmp) 
        delete [] tmp; 
    
    if (b64)
        delete [] b64;
    
    if (dtb64) 
        delete dtb64;

    return ret;
}

/*
* decoding from b64 and trasforming decoding from UTF8.
* used for contact, calendar, task
*/
wchar_t* decodeBase64WUTF8(wchar_t* toDecode) {
    
    wchar_t* ret = NULL;
    char*    tmp = NULL;    
    int      len = 0;
    char* decodedData = NULL;

    if (toDecode == NULL)
        return NULL;
    
    tmp = toMultibyte(toDecode);
    TransformationInfo info;
    DataTransformer* dt = DataTransformerFactory::getDecoder(T("b64"));

    if (dt == NULL) {        
        return NULL;
    }
    
    decodedData = dt->transform(tmp, info);
        
    if (tmp) { delete [] tmp; tmp = NULL; }

    tmp = new char[info.size + 1];
    memset(tmp, 0, info.size + 1);

    strncpy(tmp, decodedData, info.size);
       
    ret = toWideChar(tmp);    
        
    if (tmp) {
        delete [] tmp;
    }
    if (decodedData) {
        delete [] decodedData;
    }
    return ret;

}


char* decodeBase64WUTF8(wchar_t* toDecode, int* myLen) {
    
    wchar_t* ret = NULL;
    char*    tmp = NULL;
    int      len = 0;

    if (toDecode == NULL)
        return NULL;

    tmp = toMultibyte(toDecode);

    len = b64_decode(tmp, tmp);
    *myLen = len;
    tmp[len] = 0;
    return tmp;    
}


/*
* This function gets a unique file name in the current directory based on the parameter filename.
* It checks in the specified dir if fileName exists. If none it return the same file name. 
* If one arleady exists it return a file name like - file (1) - according to the 
* rules applied by windows for PPC
* 
* @param fileName : the file name that can be modified if the same file arleady exists
*/

void getUniqueFileName(wchar_t* fileName) {
        
    LOG.debug("uniqueFileBegin");
    BOOL exists = FALSE;
    FILE* f = NULL;
    int i = 1;

    f = _wfopen(fileName, TEXT("r"));

    if (f == NULL) {
        return;         
    }
    
    fflush(f);
    fclose(f);

    wchar_t path [DIM_LOCAL_TEMP];
    wchar_t file [DIM_LOCAL_TEMP];
    wsprintf(path, fileName);

    getFileName(fileName, file); // file contains file name
    
    getDirName(fileName, path); // path

    wchar_t* p = NULL;
    
    string s = string();
    int position = 0;
    s += (wchar_t*)file;
    position = pos(PWI_EXTENSION, s);
    if (position != -1) {
        file[position] = 0;
    }      
   
    do {
        wchar_t localFile[DIM_LOCAL_TEMP];
        wsprintf(localFile, TEXT("%s%s%s (%i)%s") , path, TEXT("\\"), file, i, PWI_EXTENSION);
        
        f = _wfopen(localFile, TEXT("r"));
    
        if (f == NULL) {
            wsprintf(fileName, localFile);
            break;              
        }    
        fflush(f);
        fclose(f);

        i++;
    } while(1);
    LOG.debug("uniqueFileEnd");
}



void UTCToLocalTime(SYSTEMTIME &sysTime)
{
     FILETIME LocalFileTime, SystemFileTime;
     SystemTimeToFileTime(&sysTime, &SystemFileTime);
     FileTimeToLocalFileTime(&SystemFileTime, &LocalFileTime);
     FileTimeToSystemTime(&LocalFileTime, &sysTime);
}

void localTimeToUTC(SYSTEMTIME &sysTime)
{
     FILETIME LocalFileTime, SystemFileTime;
     SystemTimeToFileTime(&sysTime, &LocalFileTime);
     LocalFileTimeToFileTime(&LocalFileTime, &SystemFileTime);
     FileTimeToSystemTime(&SystemFileTime, &sysTime);
     
     /*LPTIME_ZONE_INFORMATION lpTimeZoneInformation;
     GetTimeZoneInformation(lpTimeZoneInformation);*/     

}

void SafeGetTimeZoneInformation(TIME_ZONE_INFORMATION *ptzi) {

   ZeroMemory(ptzi, sizeof(TIME_ZONE_INFORMATION));

   // Ask the OS for the standard/daylight rules for the current time zone.
   if ((GetTimeZoneInformation(ptzi) == 0xFFFFFFFF) ||
       (ptzi->StandardDate.wMonth > 12) || (ptzi->DaylightDate.wMonth > 12))
   {
      // If the OS fails us, we default to the United States' rules.
      ZeroMemory(ptzi, sizeof(TIME_ZONE_INFORMATION));
      ptzi->StandardDate.wMonth =  10;  // October
      ptzi->StandardDate.wDay   =   5;  // Last Sunday (DOW == 0)
      ptzi->StandardDate.wHour  =   2;  // At 2:00 AM
      ptzi->DaylightBias        = -60;  // One hour difference
      ptzi->DaylightDate.wMonth =   4;  // April
      ptzi->DaylightDate.wDay   =   1;  // First Sunday (DOW == 0)
      ptzi->DaylightDate.wHour  =   2;  // At 2:00 AM
   }
}


time_t GetTransitionTimeT(TIME_ZONE_INFORMATION *ptzi, int year, BOOL fStartDST) {

   // We only handle years within the range that time_t supports.  We need to
   // handle the very end of 1969 since the local time could be up to 13 hours
   // into the previous year.  In this case, our code will actually return a
   // negative value, but it will be compared to another negative value and is
   // handled correctly.  The same goes for the 13 hours past a the max time_t
   // value of 0x7FFFFFFF (in the year 2038).  Again, these values are handled
   // correctly as well.

   if ((year < 1969) || (year > 2038)) {
      return (time_t)0;
   }

   SYSTEMTIME *pst = fStartDST ? &ptzi->DaylightDate : &ptzi->StandardDate;

   // WORD wYear          Year (0000 == 0)
   // WORD wMonth         Month (January == 1)
   // WORD wDayOfWeek     Day of week (Sunday == 0)
   // WORD wDay           Month day (1 - 31)
   // WORD wHour          Hour (0 - 23)
   // WORD wMinute        Minute (0 - 59)
   // WORD wSecond        Second (0 - 59)
   // WORD wMilliseconds  Milliseconds (0 - 999)

   // Compute the number of days since 1/1/1970 to the beginning of this year.
   long daysToYear = ((year - 1970) * 365) // Tally up previous years.
                   + ((year - 1969) >> 2); // Add few extra for the leap years.

   // Compute the number of days since the beginning of this year to the
   // beginning of the month.  We will add to this value to get the actual
   // year day.
   long yearDay = IS_LEAP_YEAR(year) ? M2LYD[pst->wMonth - 1] :
                                       M2YD [pst->wMonth - 1];

   // Check for day-in-month format.
   if (pst->wYear == 0) {

      // Compute the week day for the first day of the month (Sunday == 0).
      long monthDOW = (daysToYear + yearDay + BASE_DOW) % 7;

      // Add the day offset of the transition day to the year day.
      if (monthDOW < pst->wDayOfWeek) {
         yearDay += (pst->wDayOfWeek - monthDOW) + (pst->wDay - 1) * 7;
      } else {
         yearDay += (pst->wDayOfWeek - monthDOW) + pst->wDay * 7;
      }

      // It is possible that we overshot the month, especially if pst->wDay
      // is 5 (which means the last instance of the day in the month). Check
      // if the year-day has exceeded the month and adjust accordingly.
      if ((pst->wDay == 5) &&
          (yearDay >= (IS_LEAP_YEAR(year) ? M2LYD[pst->wMonth] :
                                            M2YD [pst->wMonth])))
      {
         yearDay -= 7;
      }

   // If not day-in-month format, then we assume an absolute date.
   } else {

      // Simply add the month day to the current year day.
      yearDay += pst->wDay - 1;
   }

   // Tally up all our days, hours, minutes, and seconds since 1970.
   long seconds = ((SECONDS_IN_A_DAY * (daysToYear + yearDay)) +
                   (3600L * (long)pst->wHour) +
                   (60L * (long)pst->wMinute) +
                   (long)pst->wSecond);

   // If we are checking for the end of DST, then we need to add the DST bias
   // since we are in DST when we chack this time stamp.
   if (!fStartDST) {
      seconds += ptzi->DaylightBias * 60L;
   }

   return (time_t)seconds;
}


BOOL IsDST(TIME_ZONE_INFORMATION *ptzi, time_t localTime) {

   // If either of the months is 0, then this usually means that the time zone
   // does not use DST.
   if ((ptzi->StandardDate.wMonth == 0) || (ptzi->DaylightDate.wMonth == 0)) {
      return FALSE;
   }

   // time_t   is a 32-bit value for the seconds since January 1, 1970
   // FILETIME is a 64-bit value for the number of 100-nanosecond intervals
   //          since January 1, 1601

   // Compute the FILETIME for the given local time.
   DWORDLONG dwl = ((DWORDLONG)116444736000000000 +
                   ((DWORDLONG)localTime * (DWORDLONG)10000000));
   FILETIME ft = *(FILETIME*)&dwl;

   // Convert the FILETIME to a SYSTEMTIME.
   SYSTEMTIME st;
   ZeroMemory(&st, sizeof(st));
   FileTimeToSystemTime(&ft, &st);

   // Get our start and end daylight savings times.
   time_t timeStart = GetTransitionTimeT(ptzi, (int)st.wYear, TRUE);
   time_t timeEnd   = GetTransitionTimeT(ptzi, (int)st.wYear, FALSE);

   // Check what hemisphere we are in.
   if (timeStart < timeEnd) {

      // Northern hemisphere ordering.
      return ((localTime >= timeStart) && (localTime < timeEnd));

   } else if (timeStart > timeEnd) {

      // Southern hemisphere ordering.
      return ((localTime < timeEnd) || (localTime >= timeStart));
   }

   // If timeStart equals timeEnd then this time zone does not support DST.
   return FALSE;
}

BOOL IsTzInDstChangingList(WCHAR* dstName)
{
    if (wcsicmp(dstName, TZ_ALASKA) == 0)
       return TRUE; 
    if (wcsicmp(dstName, TZ_CENTRAL_US) == 0)
       return TRUE; 
    if (wcsicmp(dstName, TZ_EASTERN_US) == 0)
       return TRUE; 
    if (wcsicmp(dstName, TZ_MOUNTAIN) == 0)
       return TRUE; 
    if (wcsicmp(dstName, TZ_PACIFIC_US) == 0)
       return TRUE; 

    return FALSE;
}

BOOL IsDateAfterDstChanging(SYSTEMTIME& date)
{
    if (date.wYear >= 2007)
        return TRUE;
    else
        return FALSE;
}

time_t GetLocalTimeT()
{
    SYSTEMTIME currentSysTime;
    FILETIME currentFileTime;
    GetLocalTime(&currentSysTime);
    SystemTimeToFileTime(&currentSysTime, &currentFileTime);
    FILETIME currentFt= currentFileTime;
    __int64 llCurrent = 0;
    memcpy (&llCurrent, &currentFt, sizeof (__int64));
    llCurrent = (llCurrent - 116444736000000000) / 10000000;
    time_t currentT = (time_t) llCurrent;
    return currentT;
}


void UTCToLocalTime2(SYSTEMTIME &sysTime) {

    BOOL changedDST = FALSE;
    // FIXME: this value - regFlag - will be read from registry
    BOOL regFlag = TRUE; 
    FILETIME LocalFileTime, SystemFileTime;    
    if (IsDateAfterDstChanging(sysTime))
         changedDST = TRUE;
    LocalFileTime.dwLowDateTime = 0;
    LocalFileTime.dwHighDateTime = 0;
    SystemFileTime.dwLowDateTime = 0;
    SystemFileTime.dwHighDateTime = 0;

    SystemTimeToFileTime(&sysTime, &SystemFileTime);     
    FileTimeToLocalFileTime(&SystemFileTime, &LocalFileTime);     
    FileTimeToSystemTime(&LocalFileTime, &sysTime);
        
    // convert FILETIME to time_t
    FILETIME ftTmp = LocalFileTime;
    __int64 llTmp = 0;
    memcpy (&llTmp, &ftTmp, sizeof (__int64));
    llTmp = (llTmp - 116444736000000000) / 10000000;
    time_t t = (time_t) llTmp;

    TIME_ZONE_INFORMATION tzi;
    SafeGetTimeZoneInformation(&tzi);

    // if the date is from 2007 and the locale is one of US locales
    // change the standard/DST rules to the ones that will apply there from 2007
    if (regFlag && changedDST && IsTzInDstChangingList(tzi.StandardName))
     {
         // change the tzi info for StandardTime and DaylightTime
         tzi.DaylightDate.wMonth = 3;
         tzi.DaylightDate.wDay = 2;
         tzi.StandardDate.wMonth = 11;
         tzi.StandardDate.wDay = 1;
     }
    
    if ((IsDST(&tzi, t) && // Is date t in daylight saving time?
        !IsDST(&tzi, GetLocalTimeT()))) // and current date is not in DST?    
    {
        // Subtract the number of minutes for the DST bias.  This is 
        // normally -60, meaning that we add 60 minutes.
        t -= (tzi.DaylightBias * 60);     
    }
    
    if ((!IsDST(&tzi, t) && // Is date t in standard time?
        IsDST(&tzi, GetLocalTimeT()))) // and current date is in DST?    
    {
        // Add the number of minutes for the DST bias.  This is 
        // normally -60, meaning that we substract 60 minutes.
        t += (tzi.DaylightBias * 60);     
    }

    // Subtract the number of minutes for the StandardBias.
    t -= (tzi.StandardBias * 60);

    //t -= 3600;

    llTmp = t;
    llTmp = llTmp * 10000000 + 116444736000000000;
    memcpy (&ftTmp, &llTmp, sizeof (FILETIME));
    FileTimeToSystemTime(&ftTmp, &sysTime);
}

/**
 * Convert FILETIME to Unix time_t
 */
time_t FileTimeToUnixTime(FILETIME &ft)
{
     __int64 llTmp;
     memcpy (&llTmp, &ft, sizeof (__int64));
     llTmp = (llTmp - 116444736000000000) / 10000000;
     return (time_t) llTmp;
}

/**
 * Convert Unix time_t to FILETIME
 */
void UnixTimeToFileTime(time_t t, FILETIME &ft)
{
     __int64 llTmp = t;
     llTmp = llTmp * 10000000 + 116444736000000000;
     memcpy (&ft, &llTmp, sizeof (FILETIME));
}

void localTimeToUTC2(SYSTEMTIME &sysTime)
{
     BOOL changedDST = FALSE;
     // FIXME: this value - regFlag - will be read from registry
     BOOL regFlag = TRUE;
     FILETIME LocalFileTime, SystemFileTime;
     if (IsDateAfterDstChanging(sysTime))
         changedDST = TRUE;
     SystemTimeToFileTime(&sysTime, &LocalFileTime);
     LocalFileTimeToFileTime(&LocalFileTime, &SystemFileTime);
     FileTimeToSystemTime(&SystemFileTime, &sysTime);
         
     time_t t = FileTimeToUnixTime(SystemFileTime);

     TIME_ZONE_INFORMATION tzi;
     SafeGetTimeZoneInformation(&tzi);
     
     // if the date is from 2007 and the locale is one of US locales
     // change the standard/DST rules to the ones that will apply there from 2007

     if (regFlag && changedDST && IsTzInDstChangingList(tzi.StandardName))
     {
         // change the tzi info for StandardTime and DaylightTime
         tzi.DaylightDate.wMonth = 3;
         tzi.DaylightDate.wDay = 2;
         tzi.StandardDate.wMonth = 11;
         tzi.StandardDate.wDay = 1;
     }

     if (IsDST(&tzi, t) && // Is date t in daylight saving time?
         !IsDST(&tzi, GetLocalTimeT())) // and current date is not in DST?    
     {
        // Add the number of minutes for the DST bias.  This is 
        // normally -60, meaning that we subtract 60 minutes.
        t += (tzi.DaylightBias * 60);
     }
        
     if (!IsDST(&tzi, t) && // Is date t in standard time?
         IsDST(&tzi, GetLocalTimeT())) // and current date is in DST?    
     {
        // Add the number of minutes for the DST bias.  This is 
        // normally -60, meaning that we add 60 minutes.
        t -= (tzi.DaylightBias * 60);
     }
          
     // Add the number of minutes for the StandardBias.
     t += (tzi.StandardBias * 60);

     FILETIME ftTmp;
     UnixTimeToFileTime(t, ftTmp);
     FileTimeToSystemTime(&ftTmp, &sysTime);
}

wchar_t* FileTime2UTCTimeFormatter(FILETIME &ft, wchar_t* out) {
        
    SYSTEMTIME t;

    FileTimeToSystemTime(&ft, &t);

    wchar_t month [10];
    wchar_t day   [10];
    wchar_t hour  [10];
    wchar_t minute[10];
    wchar_t second[10];
    
    // month
    if (t.wMonth < 10)
        wsprintf(month, TEXT("0%i"), t.wMonth);
    else
        wsprintf(month, TEXT("%i"), t.wMonth);
    
    // day
    if (t.wDay  < 10)
        wsprintf(day, TEXT("0%i"), t.wDay);
    else
        wsprintf(day, TEXT("%i"), t.wDay);
    
    // hour
    if (t.wHour  < 10)
        wsprintf(hour, TEXT("0%i"), t.wHour);
    else
        wsprintf(hour, TEXT("%i"), t.wHour);

    // minute
    if (t.wMinute < 10)
        wsprintf(minute, TEXT("0%i"), t.wMinute);
    else
        wsprintf(minute, TEXT("%i"), t.wMinute);

    // second
    if (t.wSecond < 10)
        wsprintf(second, TEXT("0%i"), t.wSecond);
    else
        wsprintf(second, TEXT("%i"), t.wSecond);
    
    if (out == NULL)
        out = new wchar_t[20];

    wsprintf(out, TEXT("%i%s%sT%s%s%sZ"), t.wYear, month, day, hour, minute, second);          
    
    return out;

}

bool checkSIF(int dataType) {

	const char arg[] = "dummy";
    DMTree* dmt = DMTreeFactory::getDMTree(arg);		
	ManagementNode* node = NULL;
    
	if(!dmt)
		return false;

	if (dataType == olFolderContacts)
		node = dmt->getManagementNode(CONTACTS_CONTEXT);
	else if(dataType == olFolderCalendar)
		node = dmt->getManagementNode(APPOINTMENTS_CONTEXT);
	else if(dataType == olFolderTasks)
		node = dmt->getManagementNode(TASKS_CONTEXT);

	if(!node)
		return false;

	char* val = node->getPropertyValue("useSIF");

    if (dmt)  { delete dmt; dmt = NULL; }
    if (node) { delete node; node = NULL; }
	
	if(!val)
		return false;
	
	if(!strcmp(val, "1"))
		return true;
	else
		return false;
}

/*
* Used for filter in mail sync source.
*
*/
#define DAY_LOWPART     711000000       
#define DAY_HIGHPART    201             // 201*2^32 =~ 8.6e11

wchar_t* getDayBefore(int numDayBefore) {
    
    SYSTEMTIME t;
    SYSTEMTIME t2;
    wchar_t* ret = NULL;   
    FILETIME ft;
    FILETIME ft2;
    LARGE_INTEGER time; 
    LARGE_INTEGER time2; 
    LARGE_INTEGER Days; 
    
    if (numDayBefore < 0)
        goto finally;

    // GetLocalTime (&t);
    GetSystemTime (&t);
    
    SystemTimeToFileTime(&t, &ft);    
    
    Days.LowPart = DAY_LOWPART; 
    Days.HighPart = DAY_HIGHPART; 
    Days.QuadPart = Days.QuadPart * numDayBefore;
    
    time.LowPart = ft.dwLowDateTime; 
    time.HighPart = ft.dwHighDateTime;
    
    time2.QuadPart = time.QuadPart - Days.QuadPart;
    
    ft2.dwLowDateTime = time2.LowPart;
    ft2.dwHighDateTime = time2.HighPart;
    
    FileTimeToSystemTime(&ft2, &t2);
    
    ret = new wchar_t[32];
    memset(ret, 0, 32*sizeof(wchar_t));        
    
    wchar_t month [10];
    wchar_t day   [10];        

    // month
    if (t.wMonth < 10)
        wsprintf(month, TEXT("0%i"), t2.wMonth);
    else
        wsprintf(month, TEXT("%i"), t2.wMonth);
    
    // day
    if (t.wDay  < 10)
        wsprintf(day, TEXT("0%i"), t2.wDay);
    else
        wsprintf(day, TEXT("%i"), t2.wDay);
    
 
    wsprintf(ret, TEXT("%i%s%sT000000Z"), t2.wYear, month, day);                      

finally:    
    return ret;
   
}

bool convertRrule(IRecurrencePattern* pRecPat, wchar_t* rrule, DATE startDate) {
	bool ret = true;

	wchar_t* str = new wchar_t[100];
	wcscpy(str, rrule);

	wchar_t seps[] = TEXT(" ");
	wchar_t* token;

	token = wcstok( str, seps );

	//first will be D|W|MP|MD|YM|YD<interval>
	long recType = -1;
	long interval;
	long occurences;
	DATE endDate;
	long dayOfMonth = -1;
	long weekOfMonth = -1;
	long monthOfYear = -1;

	if(token[0] == TEXT('D')) {
		recType = olRecursDaily;
		token++;
	}
	else if(token[0] == TEXT('W')) {
		recType = olRecursWeekly;
		token++;
		
	}
	else if(token[0] == TEXT('M') && token[1] == TEXT('D')) {
		recType = olRecursMonthly;
		token = token + 2;
	}
	else if(token[0] == TEXT('M') && token[1] == TEXT('P')) {
		recType = olRecursMonthNth;
		token = token + 2;
	}
	//not supported
	else if(token[0] == TEXT('Y') && token[1] == TEXT('D'))
		recType = -1;
	else if(token[0] == TEXT('Y') && token[1] == TEXT('M')) {		
		recType = olRecursYearly; //!!!
		token = token + 2;
	}

	interval = _wtol(token);
	if(!interval || recType == -1) {
		if(str)
			delete [] str;
		return false;
	}

	wchar_t* days = new wchar_t[20];
	wcscpy(days, TEXT(""));

	wchar_t* mOfYear = new wchar_t[20];
	wcscpy(mOfYear, TEXT(""));

	while( token != NULL ) {
		token = wcstok( NULL, seps );
		if(token) {		
			if(recType == olRecursDaily) {
				//this toke should be even number of occurrences or the end date of the pattern				
				if(wcschr(token, TEXT('#'))) {
					pRecPat->put_RecurrenceType(recType);
					token++;
					occurences = _wtol(token);
					if(occurences == 0)
						pRecPat->put_NoEndDate(VARIANT_TRUE);
					else
						pRecPat->put_Occurrences(occurences);
					pRecPat->put_Interval(interval);
					
				}
				else if(token[8] == TEXT('T')) {
					pRecPat->put_RecurrenceType(recType);
					systemTimeToDouble(token, &endDate, NULL);
					pRecPat->put_PatternEndDate(endDate);
					pRecPat->put_Interval(interval);
				}
			}
			else if(recType == olRecursWeekly) {				
				if(wcschr(token, TEXT('#'))) {
					pRecPat->put_RecurrenceType(recType);
					token++;
					occurences = _wtol(token);
					if(occurences == 0)
						pRecPat->put_NoEndDate(VARIANT_TRUE);
					else
						pRecPat->put_Occurrences(occurences);
					pRecPat->put_Interval(interval);
					if(wcscmp(days, TEXT(""))) 
						pRecPat->put_DayOfWeekMask(calculateDayOfWeek(days));
					else
						pRecPat->put_DayOfWeekMask(getWeekDay(startDate));
				}
				else if(token[8] == TEXT('T')) {
					pRecPat->put_RecurrenceType(recType);
					systemTimeToDouble(token, &endDate, NULL);
					pRecPat->put_PatternEndDate(endDate);
					pRecPat->put_Interval(interval);
					if(wcscmp(days, TEXT(""))) 
						pRecPat->put_DayOfWeekMask(calculateDayOfWeek(days));
					else
						pRecPat->put_DayOfWeekMask(getWeekDay(startDate));
				}
				else if(isDayOfWeek(token)) {
 					wcscat(days, token);
					wcscat(days, TEXT(" "));
				}
			}
			else if(recType == olRecursMonthly) {
				//VCalendar supports as Monthly by day rules: MD<interval> listofdates|listofdates(from end of month) <end tag>
				//On outlook recurrence pattern only MD<interval> day <end tag> can be represented
				if(wcschr(token, TEXT('#'))) {
					pRecPat->put_RecurrenceType(recType);
					token++;
					occurences = _wtol(token);
					if(occurences == 0)
						pRecPat->put_NoEndDate(VARIANT_TRUE);
					else
						pRecPat->put_Occurrences(occurences);
					pRecPat->put_Interval(interval);
					pRecPat->put_DayOfMonth(dayOfMonth);
				}
				else if(token[8] == TEXT('T')) {
					pRecPat->put_RecurrenceType(recType);
					systemTimeToDouble(token, &endDate, NULL);
					pRecPat->put_PatternEndDate(endDate);
					pRecPat->put_Interval(interval);
					pRecPat->put_DayOfMonth(dayOfMonth);
				}
				else {
					if(dayOfMonth != -1) {
						if(str)
							delete [] str;
						return false;
					}
					dayOfMonth = _wtol(token);
					if(dayOfMonth == 0) {
						if(str)
							delete [] str;
						return false;
					}
				}
			}
			else if(recType == olRecursMonthNth) {
				if(wcschr(token, TEXT('#'))) {
					pRecPat->put_RecurrenceType(recType);
					token++;
					occurences = _wtol(token);
					if(occurences == 0)
						pRecPat->put_NoEndDate(VARIANT_TRUE);
					else
						pRecPat->put_Occurrences(occurences);
					pRecPat->put_Interval(interval);
					pRecPat->put_Instance(weekOfMonth);
					pRecPat->put_DayOfWeekMask(calculateDayOfWeek(days));
				}
				else if(token[8] == TEXT('T')) {
					pRecPat->put_RecurrenceType(recType);
					systemTimeToDouble(token, &endDate, NULL);
					pRecPat->put_PatternEndDate(endDate);
					pRecPat->put_Interval(interval);
					pRecPat->put_Instance(weekOfMonth);
					pRecPat->put_DayOfWeekMask(calculateDayOfWeek(days));
				}
				else if(isDayOfWeek(token)) {
 					wcscat(days, token);
					wcscat(days, TEXT(" "));
				}
				else { 
					if(token[1] != TEXT('+') && token[1] != TEXT('-')) {
						if(str)
							delete [] str;
						return false;
					}
					wchar_t sWeek[] = TEXT("\0\0");
					sWeek[0] = token[0];
					weekOfMonth = _wtol(sWeek);
					if(token[1] == TEXT('-'))
						weekOfMonth = 5 - weekOfMonth;
					if(weekOfMonth > 5 || weekOfMonth < 1) {
						if(str)
							delete [] str;
						return false;
					}
				}
			}
			else if(recType == olRecursYearly) {
				//expected sequence will be YM1 month <end tag>
				if(wcschr(token, TEXT('#'))) {
					pRecPat->put_RecurrenceType(recType);
					token++;
					occurences = _wtol(token);
					if(occurences == 0)
						pRecPat->put_NoEndDate(VARIANT_TRUE);
					else
						pRecPat->put_Occurrences(occurences);
					pRecPat->put_Interval(interval);
					pRecPat->put_MonthOfYear(monthOfYear);
					pRecPat->put_DayOfMonth(dayOfMonth);
				}
				else if(token[8] == TEXT('T')) {
					pRecPat->put_RecurrenceType(recType);
					systemTimeToDouble(token, &endDate, NULL);
					pRecPat->put_PatternEndDate(endDate);
					pRecPat->put_Interval(interval);
					pRecPat->put_MonthOfYear(monthOfYear);
					pRecPat->put_DayOfMonth(dayOfMonth);
				}
				else {
					if(wcscmp(mOfYear, TEXT(""))) {
						if(str)
							delete [] str;
						return false;
					}
 					wcscat(mOfYear, token);
					monthOfYear = _wtol(mOfYear);
					SYSTEMTIME st;
					VariantTimeToSystemTime(startDate, &st);
					dayOfMonth = st.wDay;
				}
			}
		}		
	}

	return ret;
}
wchar_t* extractRrule(IRecurrencePattern* pRecPat) {
	
	wchar_t* recurrence = new wchar_t[100];
	wcscpy(recurrence, TEXT(""));
	DATE patStartDate;
	DATE patEndDate;
	wchar_t* datev = new wchar_t[30];
				
	long recType;
	pRecPat->get_RecurrenceType(&recType);
	pRecPat->get_PatternEndDate(& patEndDate);
	pRecPat->get_PatternStartDate(& patStartDate);
	
	VARIANT_BOOL noEnd;
	pRecPat->get_NoEndDate(&noEnd);

	long interval;
	pRecPat->get_Interval(&interval);

	long occurences;
	pRecPat->get_Occurrences(&occurences);
	
	long dayofweek;
	pRecPat->get_DayOfWeekMask(&dayofweek);

	long dayofmonth;
	pRecPat->get_DayOfMonth(& dayofmonth);

	//week of month
	long instance;
	pRecPat->get_Instance(&instance);
	
	long duration;
	pRecPat->get_Duration(&duration);

	long monthofyear;
	pRecPat->get_MonthOfYear(&monthofyear);

	switch(recType) {
			case olRecursDaily:
				if(noEnd == VARIANT_TRUE)
					occurences = 0;  //occurences are -1 in this case but we use 0 to indicate event that does not end
				if(interval >0)
						wsprintf(recurrence, TEXT("D%ld #%ld"), interval, occurences);
				else if (dayofweek > 0) {
					wchar_t* days = extractDayOfWeek(dayofweek);
					if (days) {
						doubleToSystemTimeHourZero(datev, patEndDate, FALSE);
						wsprintf(recurrence, TEXT("W1 %s %s #%ld"), days, datev, occurences);
						delete [] days;
					}
				}
				break;
			case olRecursWeekly:
				if (dayofweek > 0) {
					wchar_t* days = extractDayOfWeek(dayofweek);
					if (days) {
						if(noEnd == VARIANT_TRUE)
							wsprintf(recurrence, TEXT("W%ld %s #0"), interval, days);
						else {
							doubleToSystemTimeHourZero(datev, patEndDate, FALSE);
							wsprintf(recurrence, TEXT("W%ld %s%s #%ld"), interval, days, datev, occurences);
							delete [] days;
						}
					}
				}
				break;
			case olRecursMonthly:
				if(dayofmonth > 0) 
					if(noEnd == VARIANT_TRUE)
							wsprintf(recurrence, TEXT("MD%ld %ld #0"), interval, dayofmonth);
						else {
							doubleToSystemTimeHourZero(datev, patEndDate, FALSE);
							wsprintf(recurrence, TEXT("MD%ld %ld %s #%ld"), interval, dayofmonth, datev, occurences);
						}
				break;
			case olRecursMonthNth:
				if(instance > 0 && dayofweek >0) {
					wchar_t* days = extractDayOfWeek(dayofweek);
					if(days)
						if(noEnd == VARIANT_TRUE)
								wsprintf(recurrence, TEXT("MP%ld %ld+ %s #0"), interval, instance, days);
							else {
								doubleToSystemTimeHourZero(datev, patEndDate, FALSE);
								wsprintf(recurrence, TEXT("MP%ld %ld+ %s%s #%ld"), interval, instance, days, datev, occurences);
							}
					if(days)
						delete [] days;
				}
				break;
			case olRecursYearly:
				if(dayofmonth > 0 && monthofyear > 0) 
					if(noEnd == VARIANT_TRUE) 
						wsprintf(recurrence, TEXT("YM1 %ld #0"), monthofyear);
					else
						wsprintf(recurrence, TEXT("YM1 %ld #%ld"), monthofyear, occurences);
				break;
			case olRecursYearNth:
				if(dayofweek > 0 && instance >0) {
					wchar_t* days = extractDayOfWeek(dayofweek);
					if(days)
						if(noEnd == VARIANT_TRUE)
							wsprintf(recurrence, TEXT("MP12 %ld+ %s #0"), instance, days);
							else {
								doubleToSystemTimeHourZero(datev, patEndDate, FALSE);
								wsprintf(recurrence, TEXT("MP12 %ld+ %s%s #%ld"), instance, days, datev, occurences);
							}
					if(days)
						delete [] days;
				}
				break;	
		}

		delete [] datev; datev = NULL;	

	return recurrence;
}

wchar_t* extractDayOfWeek(long l) {
	/*
		enum OlDaysOfWeek {
		  olSunday    = 1,
		  olMonday    = 2,
		  olTuesday   = 4,
		  olWednesday = 8,
		  olThursday  = 16,
		  olFriday    = 32,
		  olSaturday  = 64,
		};

		*/
	if (l<0 || l>128)
		return NULL;

	//SU MO TU WE TH FR SA

	wchar_t* ret = new wchar_t[21];
	wcscpy(ret, TEXT(""));
	
	if(l & olSunday)
		wcscat(ret, TEXT("SU "));
	if(l & olMonday)
		wcscat(ret, TEXT("MO "));
	if(l & olTuesday)
		wcscat(ret, TEXT("TU "));
	if(l & olWednesday)
		wcscat(ret, TEXT("WE "));
	if(l & olThursday)
		wcscat(ret, TEXT("TH "));
	if(l & olFriday)
		wcscat(ret, TEXT("FR "));
	if(l & olSaturday)
		wcscat(ret, TEXT("SA "));

	return ret;
	
}
long calculateDayOfWeek(wchar_t* in) {

	long ret = 0;
	
	wchar_t* index;
	index = NULL;
	index = wcsstr(in, TEXT("SU"));
	if(index)
		ret += olSunday;

	index = NULL;
	index = wcsstr(in, TEXT("MO"));
	if(index)
		ret += olMonday;

	index = NULL;
	index = wcsstr(in, TEXT("TU"));
	if(index)
		ret += olTuesday;

	index = NULL;
	index = wcsstr(in, TEXT("WE"));
	if(index)
		ret += olWednesday;

	index = NULL;
	index = wcsstr(in, TEXT("TH"));
	if(index)
		ret += olThursday;

	index = NULL;
	index = wcsstr(in, TEXT("FR"));
	if(index)
		ret += olFriday;

	index = NULL;
	index = wcsstr(in, TEXT("SA"));
	if(index)
		ret += olSaturday;

	return ret; 
}

bool isDayOfWeek(wchar_t* string) {
	
	bool ret = false;
	wchar_t* weekDay[] = {TEXT("SU"), TEXT("MO"), TEXT("TU"), TEXT("WE"), TEXT("TH"), TEXT("FR"), TEXT("SA")};

	for(int i = 0; i < 7 ; i++) 
		if(!wcscmp(string, weekDay[i]))
			return true;

	return ret;
}

long getWeekDay(DATE  dt) {
	long ret = 0;

	SYSTEMTIME st;
	VariantTimeToSystemTime(dt, &st);

    ret = (long)pow(2, st.wDayOfWeek);

	return ret;
}


BOOL CALLBACK EnumWindowsProc(HWND hwnd, LPARAM lParam)
{
    DWORD pid = NULL;
    DWORD threadId = GetWindowThreadProcessId(hwnd, &pid);
    if (pid == (DWORD)lParam)
    {
        SendMessage(hwnd,WM_CLOSE, 0, 0);
    }

    return TRUE;
}


DWORD getProcessPid(const wchar_t *processName)
{
    PROCESSENTRY32 lppe; 
    BOOL next = TRUE;
    HANDLE hProcess = 0;
    DWORD pid;

    HANDLE hSnapshot = CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0);
    
    if (hSnapshot == INVALID_HANDLE_VALUE) 
        return 0;
    
    lppe.dwSize = sizeof( PROCESSENTRY32 );

    if (!Process32First(hSnapshot, &lppe))
        return 1;

    if (wcsstr(lppe.szExeFile, processName) != NULL) {
        pid = lppe.th32ProcessID;
    }
    
    do {
        next = Process32Next(hSnapshot, &lppe);

        if (!next)
            break;
        
        if (wcsstr(lppe.szExeFile, processName) != NULL ) {
            pid = lppe.th32ProcessID;
            break;
        }

    } while(next);

    CloseHandle(hSnapshot);
    CloseHandle(hProcess);
    return pid;
}


DWORD stopProcess(DWORD pid)
{
    DWORD out = EnumWindows(&EnumWindowsProc, pid);

    return out;
}

DWORD stopProcessByName(const wchar_t *name)
{
    DWORD out = 0;
    DWORD pid = getProcessPid(name);
    out = stopProcess(pid);
    return out;
}

DWORD getMemoryAvailable() {
    MEMORYSTATUS mem;
    GlobalMemoryStatus(&mem);
    return mem.dwAvailPhys;
    
}

BOOL isMemoryAvailable(DWORD min) {

    if (min == 0) 
        min = 2500000; // 2.5 Mb

    return getMemoryAvailable() > min ? TRUE : FALSE;    
    
}

// Windows Mobile 5 has OS version = 5
// The previous is < 5 (2003 is 4)
//
BOOL isWindowsMobile5() {
    
    if (windowsMobileVersion == 0) {
        OSVERSIONINFO OSVersionInfo;
        OSVersionInfo.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
        GetVersionEx(&OSVersionInfo);
        
        windowsMobileVersion = OSVersionInfo.dwMajorVersion;
    }
    
    if (windowsMobileVersion < 5)
        return false;
    else
        return true;

}


