/*
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "pim/WindowsSyncSource.h"
#include "spds/SyncItemStatus.h"
#include "base/util/utils.h"
#include "base/Log.h"
#include "pim/POOMClient2Server.h"
#include "pim/POOMServer2Client.h"
#include "spdm/errors.h"
#include "notify/timed_msgbox.h"

#define ASK_REFRESH_PIM_MESSAGE TEXT("All the local %s will be deleted before adding the server items. Are you sure you want to continue?")
#define ASK_REFRESH_PIM_TIMEOUT 10

static int call;
static int cnew;
static int ckey;
static int cupdated;
static int cdeleted;
           
WindowsSyncSource::WindowsSyncSource(const wchar_t* name) : SyncSource(name) {
    c = NULL;        
    setPath(TEXT("\\"));
    syncItem = NULL;
    itemError = FALSE;
    friendlyName[0] = 0;  // char 64
    newFromServer = 0;
    updatedFromServer = 0;
    deletedFromServer = 0;

}

WindowsSyncSource::~WindowsSyncSource() {
    if (c) 
        delete c; 
    c = NULL;
    if (syncItem)
        delete syncItem;
    syncItem = NULL;
}

void WindowsSyncSource::setType(SourceType t) {
    type = t;
}

SourceType WindowsSyncSource::getType() {
    return type;
}


void WindowsSyncSource::setPath(const wchar_t* p) {
    if (p == NULL)
        path = wstrdup(TEXT(""));
    else
        path = wstrdup(p);
    
}

wchar_t* WindowsSyncSource::getPath() {
    return path;
}

/*
* Return the first SyncItem of all. It is used in case of slow or refresh sync 
* and retrieve the entire data source content.
*/

SyncItem* WindowsSyncSource::getFirstItem() {
    
    if (syncItem) {
        delete syncItem; syncItem = NULL;
    }
    
    if (c == NULL) {
        c = new Container();
    }

    setAllItemsPOOM(c, getType()); // olFolderContact = 10
    
    LOG.info("The client number of %s to sync are %i", friendlyName, c->getAllItemsSize());

    if (c->getAllItemsSize() == 0) {
        return NULL;
    }        
    call = 0;
    
    ArrayList* list = c->getAllItems();    
    syncItem = (SyncItem*)((SyncItem*)list->get(call))->clone();    
    fillSyncItem(syncItem, getType());
    return syncItem;

}

/*
* Return the next SyncItem of all. It is used in case of slow or refresh sync 
* and retrieve the entire data source content.
*/

SyncItem* WindowsSyncSource::getNextItem() {        
    
    if (syncItem) {
        delete syncItem; syncItem = NULL;
    }
    
    if (c == NULL)
        return NULL;

    if (c->getAllItemsSize() == 0)
        return NULL;

    call++;
    if (call == c->getAllItemsSize()) {
        return NULL;
    }     
    syncItem = (SyncItem*)((SyncItem*)c->getAllItems()->get(call))->clone();  
    fillSyncItem(syncItem, getType());
    return syncItem;
    
}

/*
* This method is always called by the sync manager. So the setModifiedItemsPOOM is always called
*/

SyncItem* WindowsSyncSource::getFirstNewItem() {
    
    if (syncItem) {
        delete syncItem; syncItem = NULL;
    }
    
    if (c == NULL) {
        c = new Container();
    }

    setModifiedItemsPOOM(c, getType(), getPath()); // olFolderContact = 10

    //Log the number of item to sync from client
    LOG.info("The client number of new %s to sync are %i", friendlyName, c->getNewItemsSize());
    LOG.info("The client number of updated %s to sync are %i", friendlyName, c->getUpdatedItemsSize());
    LOG.info("The client number of deleted %s to sync are %i", friendlyName, c->getDeletedItemsSize());
    
    if (c->getNewItemsSize() == 0) {
        return NULL;
    }    
    cnew = 0;
    /*
    ArrayList* list = c->getNewItems();
    syncItem = (SyncItem*)list->get(cnew);    
    fillSyncItem(syncItem, getType());
    return syncItem;
    */
    ArrayList* list = c->getNewItems();
    return (SyncItem*)list->get(cnew);    
    

}

SyncItem* WindowsSyncSource::getFirstItemKey() {
    
    ArrayList* list = NULL;

    if (c == NULL) {
        c = new Container();
    }

    setAllItemsPOOM(c, getType()); // olFolderContact = 10
    if (c->getAllItemsSize() == 0) {
        goto finally;
    }        

    list  = c->getAllItems();    
  
    if (list && list->size() > 0) {
        for (ckey = 0; ckey < list->size(); ckey++) {
            deleteItem(*(SyncItem*)list->get(ckey));
        }
    }

finally:

    if (c) { delete c; c = NULL; }
    return NULL;
    /*
    if (syncItem) {
        delete syncItem; syncItem = NULL;
    }
    
    if (c == NULL) {
        c = new Container();
    }

    setAllItemsPOOM(c, getType()); // olFolderContact = 10
    if (c->getAllItemsSize() == 0) {
        return NULL;
    }        
    ckey = 0;
    ArrayList* list = c->getAllItems();    
    syncItem = (SyncItem*)((SyncItem*)list->get(ckey))->clone();    
    return syncItem;
    */
    

}

SyncItem* WindowsSyncSource::getNextItemKey() {
    return NULL;
    /*
    if (syncItem) {
        delete syncItem; syncItem = NULL;
    }
    
    if (c == NULL)
        return NULL;

    if (c->getAllItemsSize() == 0)
        return NULL;

    ckey++;
    if (ckey == c->getAllItemsSize()) {
        return NULL;
    }     
    syncItem = (SyncItem*)((SyncItem*)c->getAllItems()->get(ckey))->clone();
    return syncItem;
    */
}

SyncItem* WindowsSyncSource::getNextNewItem() {    
    
    if (syncItem) {
        delete syncItem; syncItem = NULL;
    }
    
    if (c == NULL)
        return NULL;

    cnew++;
    if (cnew == c->getNewItemsSize()) {
        return NULL;
    }
    /*
    syncItem = (SyncItem*)c->getNewItems()->get(cnew);    
    fillSyncItem(syncItem, getType());
    return syncItem;  
    */
    return (SyncItem*)c->getNewItems()->get(cnew);    
}

SyncItem* WindowsSyncSource::getFirstUpdatedItem() {
    
    if (syncItem) {
        delete syncItem; syncItem = NULL;
    }
    
    if (c == NULL) {
        c = new Container();
    }

    if (c->getUpdatedItemsSize() == 0) {
        return NULL;
    }    
    cupdated = 0;
    /*
    ArrayList* list = c->getUpdatedItems();
    syncItem = (SyncItem*)list->get(cupdated);          
    fillSyncItem(syncItem, getType());
    return syncItem;
    */

    ArrayList* list = c->getUpdatedItems();
    return (SyncItem*)list->get(cupdated);          
    
}

SyncItem* WindowsSyncSource::getNextUpdatedItem() {    
   
    if (syncItem) {
        delete syncItem; syncItem = NULL;
    }
    
    if (c == NULL)
        return NULL;

    cupdated++;
    if (cupdated == c->getUpdatedItemsSize()) {
        return NULL;
    }        
    /*
    syncItem = (SyncItem*)c->getUpdatedItems()->get(cupdated);         
    fillSyncItem(syncItem, getType());
    return syncItem;
    */
    
    return (SyncItem*)c->getUpdatedItems()->get(cupdated);         

}

SyncItem* WindowsSyncSource::getFirstDeletedItem() {
   
    if (c == NULL) {
        c = new Container();
    }

    if (c->getDeletedItemsSize() == 0) {
        return NULL;
    }    
    cdeleted = 0;
    ArrayList* list = c->getDeletedItems();

    return (SyncItem*)list->get(cdeleted);      
}

SyncItem* WindowsSyncSource::getNextDeletedItem() {    
    if (c == NULL)
        return NULL;

    cdeleted++;
    if (cupdated == c->getDeletedItemsSize()) {
        return NULL;
    }        
    return (SyncItem*)c->getDeletedItems()->get(cdeleted);    
}

void WindowsSyncSource::setItemStatus(const wchar_t* key, int status) {    
    LOG.debug("key: %ls, status: %i", key, status);     
    if (status > 299 && status != 418 && status != 419)
        itemError = TRUE;
    else
        itemError = FALSE;
}

int WindowsSyncSource::addItem(SyncItem& item) {
    
    int ret = STC_COMMAND_FAILED; // command Failed;
    long oid = 0;
    if (!isMemoryAvailable()) {
        LOG.error(LOW_DEVICE_MEMORY);
        return ret;
    }

    long h = manageNewItems(&item, getType(), &oid);
    
    wchar_t tmp[256];
    if (h == 0) {        
        ret = STC_ITEM_ADDED;
        LOG.debug("Added item: %ls LUID: %i", item.getKey(), oid);        
        wsprintf(tmp, TEXT("%i"), oid);
        item.setKey(tmp);  

        newFromServer++;
    }

    return ret;
}

int WindowsSyncSource::updateItem(SyncItem& item) {
    
    int ret = STC_COMMAND_FAILED; // command Failed;
    
    if (!isMemoryAvailable()) {
        LOG.error(LOW_DEVICE_MEMORY);
        return ret;
    }

    long h = manageUpdatedItems(&item, getType());
    
    if (h == 0) {
        ret = STC_OK;        
        LOG.debug("updated item: %ls", item.getKey());  

        updatedFromServer++;
    }

    return ret;
}

int WindowsSyncSource::deleteItem(SyncItem& item) {
    
    int ret = STC_COMMAND_FAILED; // command Failed;
    long h = manageDeletedItems(&item, getType());
    
    if (h == 0) {
        ret = STC_OK;        
        LOG.debug("deleted item: %ls", item.getKey());   
        
        deletedFromServer++;
    }
    return ret;    
}

int WindowsSyncSource::beginSync() {
    /*
    OL_CONTACTS     = 10,
    OL_CALENDAR     = 9,
    OL_TASK         = 13,
    OL_BRIEFCASE    = 600,
    OL_NOTES        = 601,
    OL_FAVORITES    = 602
    */
    wchar_t message[256];
    wchar_t* sourceNameW = NULL;
    
    // counter to get the number of items from server
    newFromServer = 0;
    updatedFromServer = 0;
    deletedFromServer = 0;

    message[0] = 0;    

    if (getType() == OL_CONTACTS)
        strcpy(friendlyName, "contacts");
    else if (getType() == OL_CALENDAR)
        strcpy(friendlyName, "calendars");
    else if (getType() == OL_TASK)
        strcpy(friendlyName, "tasks");
    
    sourceNameW = toWideChar(friendlyName);

    wsprintf(message, ASK_REFRESH_PIM_MESSAGE, sourceNameW);

    if (getSyncMode() == SYNC_REFRESH_FROM_SERVER) {
        int val = TimedMessageBox(NULL, message, TEXT("Alert"),
                             MB_YESNO | MB_ICONQUESTION | MB_SETFOREGROUND, ASK_REFRESH_PIM_TIMEOUT * 1000); // default ASK_TIMEOUT = 10

        if (val == IDYES) {
            LOG.debug("User said yes to delete local PIM and refresh.");            
        }
        else {
            LOG.debug("User said no to delete local PIM and refresh");     
            return -1;
        }
    }

    if (sourceNameW) {delete [] sourceNameW; sourceNameW = NULL; }
    return 0;    
}

int WindowsSyncSource::endSync() {
        
    writeCurrentItems(getType(), getPath());
    if (TRUE == itemError) {
        return ERR_ITEM_ERROR;
    }
    else {
        //Log the number of item synced from server
        LOG.info("The server number of new %s synced are %i", friendlyName, newFromServer);
        LOG.info("The server number of updated %s synced are %i", friendlyName, updatedFromServer);
        LOG.info("The server number of deleted %s synced are %i", friendlyName, deletedFromServer);
         
        return 0;

    }
}

void WindowsSyncSource::assign(WindowsSyncSource& s) {
    setSyncMode(s.getSyncMode());
    setLastSync(s.getLastSync());
    setNextSync(s.getNextSync());
    setRemoteURI(s.getRemoteURI());
    setType(s.getType());
    setLastAnchor(s.getLastAnchor());
    setNextAnchor(s.getNextAnchor());
    setErrorHandler(s.getErrorHandler());
    setFilter(s.getFilter());
    setPath(getPath());
    setType(getType());
}