/*
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "pim/mail/MailServer2Client.h"
#include "base/util/utils.h"
#include "base/util/BasicTime.h"
#include "pim/difftime.h"

#include "base/Log.h"
HRESULT SetReplyTo(LPMAPISESSION lpSession, LPMESSAGE lpMessage, LPSTR pszReplyTo);

//------------------------------------------------------------- Typedef

enum {
	eeePR_ENTRYID,
	eeePR_MESSAGE_FLAGS,
    eeePR_LAST_MODIFICATION_TIME,
	eeeNUM_COLS};

// These tags represent the message information we would like to pick up
static SizedSPropTagArray(eeeNUM_COLS, endSync) = 
{	
	eeeNUM_COLS,
    PR_ENTRYID,                       // entry id
	PR_MESSAGE_FLAGS,    
    PR_LAST_MODIFICATION_TIME        //

};

 enum {
	ePR_ENTRYID,
	ePR_MESSAGE_FLAGS,
    //MM        
    ePR_LAST_MODIFICATION_TIME,
    ePR_SUBJECT,
    ePR_SUBJECT_PREFIX,
    ePR_MESSAGE_SIZE,    
    ePR_SENDER_NAME,        
    ePR_SENDER_EMAIL_ADDRESS,    
    ePR_HASATTACH,
    //MM
	NUM_COLS};

// These tags represent the message information we would like to pick up
static SizedSPropTagArray(NUM_COLS, properties) = 
{	
	NUM_COLS,
    PR_ENTRYID,                         // entry id
	PR_MESSAGE_FLAGS,
    //MM        
    PR_LAST_MODIFICATION_TIME,          //
    PR_SUBJECT,                         // mail Subject
    PR_SUBJECT_PREFIX,
    PR_MESSAGE_SIZE,                    // sum of the size of every property of the object
    PR_SENDER_NAME,                     // "Marco "
    PR_SENDER_EMAIL_ADDRESS,            // From  "Marco <magi@libero.it>    
    PR_HASATTACH

};

enum {
	eePR_ENTRYID,
    eePR_LAST_MODIFICATION_TIME,
	eePR_DISPLAY_NAME,     
    eePR_MESSAGE_FLAGS,
    eeNUM_COLS};

// These tags represent the message information we would like to pick up
static SizedSPropTagArray(eeNUM_COLS, Columns) = 
{	
	eeNUM_COLS,
	PR_ENTRYID,
    PR_LAST_MODIFICATION_TIME,
	PR_DISPLAY_NAME,
    PR_MESSAGE_FLAGS
};

//----------------------------------------------------- Local functions
static ULONG CountRecipsInString(LPCWSTR psz)
{
  ULONG cRecips = 0;

  if (psz != NULL)
  {
    if (*psz != '\0')
      ++cRecips;

    while (*psz != L'\0')
    {
      if (*psz == L';' || *psz == L',') 
        ++cRecips;
      ++psz;
    }
  }

  return cRecips;
}


//------------------------------------------------------ Constructors

MailServer2Client::MailServer2Client()
{
    server_folderToSync = NULL;
}

MailServer2Client::~MailServer2Client()
{
    if (server_folderToSync) {
        delete [] server_folderToSync; server_folderToSync = NULL;
    }
}

//----------------------------------------------------------- Methods

HRESULT MailServer2Client::AddAttachment(LPMESSAGE pmsg, MailClientData* m)
{
    LOG.debug("Enter into AddAttachment function");
	HRESULT hr = S_OK;
    BodyPart* bodyPart = NULL;
    bodyPart = m->getEmailData()->getEmailItem().getFirstAttachment();

    if (bodyPart == NULL) {
        LOG.debug("Exiting from AddAttachment function: no body part found");
        return hr;
    }

    do {
        
        wchar_t* tmpFileName = toWideChar(bodyPart->getContent());
        wchar_t* filename    = toWideChar(bodyPart->getFilename());

        SPropValue  prop[3];
		IAttach*    pAttachment = NULL;
		ULONG       ulAttachmentNum = 0;
		FILE*       fileAttach = NULL;;
        
        memset(prop, 0, sizeof(prop));
        fileAttach = _wfopen(tmpFileName, TEXT("rb"));	
        
        if (fileAttach == NULL) {
            hr = -1;
            break;
        }
        
        hr = pmsg->CreateAttach(NULL, 0, &ulAttachmentNum, &pAttachment);
		EXIT_ON_FAILED(hr);

		// Set the initial properties on the attachment 
        // no data actually there yet
		prop[0].ulPropTag = PR_ATTACH_METHOD;
		prop[0].Value.ul = ATTACH_BY_VALUE;

		prop[1].ulPropTag = PR_ATTACH_DATA_BIN;
        
        prop[2].ulPropTag = PR_ATTACH_FILENAME;
		prop[2].Value.LPSZ = filename;
        
        hr = pAttachment->SetProps(3, prop, NULL);
		EXIT_ON_FAILED(hr);

		// Now get the data property and fill it in
		//
		IStream* pStream;

		hr = pAttachment->OpenProperty (PR_ATTACH_DATA_BIN,
									 NULL,
									 0,
									 MAPI_MODIFY | MAPI_CREATE,
									 (LPUNKNOWN *)&pStream);
		EXIT_ON_FAILED(hr);
        
        //DWORD cbFile = GetFileSize(fileAttach, NULL);
        DWORD cbFile = fgetsize(fileAttach);
        if (cbFile == 0xFFFFFFFF) // error
            break;
        
		ULONG        cbWritten;
		ULONG        cbReadSize;
		const ULONG  ChunkSize = 4096;
		BYTE         Buffer[ChunkSize];

		do {
			// cbReadSize = fileAttach.Read(Buffer, ChunkSize);
            cbReadSize = fread(Buffer, sizeof(BYTE), ChunkSize, fileAttach ); 
                
			if (cbReadSize > 0)
			{
				hr = pStream->Write(Buffer, cbReadSize, &cbWritten);
			}
		} while (!FAILED(hr) && (cbReadSize == ChunkSize));

		fflush(fileAttach);
        fclose(fileAttach);
		pStream->Commit(STGC_DEFAULT);
		pStream->Release();

		// Set the size property
		//
		prop[0].ulPropTag = PR_ATTACH_SIZE;
		prop[0].Value.ul = cbFile + sizeof(prop[2]);
		hr = pAttachment->SetProps(1, prop, NULL);                
        
        if (tmpFileName != NULL) {
            DeleteFile(tmpFileName);
        }


    } while ((bodyPart = m->getEmailData()->getEmailItem().getNextAttachment()) != NULL);

FuncExit:  
    LOG.debug("Exiting from AddAttachment function");
    return hr;   
}


static TCHAR szSMTP[] = L"SMTP";

void AddRecipients(LPWSTR pszSrc, ULONG ulType, ADRLIST* plst, LPWSTR& pszLocalRecips, LPSPropValue& pval)
{   
    LOG.debug("Enter into AddRecipients function");
    if (pszSrc == NULL) {
        LOG.debug("Exiting from AddRecipients function: no recipients");
        return;
    }
    LPWSTR psz = pszSrc;
    while (*psz != L'\0') {
        while (*psz == L' ' && *psz != '\0')
            ++psz;

        if (*psz == L'\0')
            break;

        LPWSTR pszEnd = psz;
        // while (*pszEnd != L' ' && (*pszEnd != ';' || *pszEnd != ',') && *pszEnd != '\0')
        while (*pszEnd != ',' && *pszEnd != '\0')
            ++pszEnd;

        int cch = pszEnd - psz;

        wcsncpy(pszLocalRecips, psz, cch);
        *(pszLocalRecips + cch) = L'\0';

        plst->aEntries[plst->cEntries].cValues = 3;
        plst->aEntries[plst->cEntries].rgPropVals = pval;

        // Set the type (To, Cc, Bcc)...
        pval->ulPropTag = PR_RECIPIENT_TYPE;
        pval->Value.ul = ulType;
        ++pval;

        // Set the address type (we only do SMTP)...
        pval->ulPropTag = PR_ADDRTYPE;
        pval->Value.lpszW = szSMTP;
        ++pval;

        // Set the address...
        pval->ulPropTag = PR_EMAIL_ADDRESS;
        pval->Value.lpszW = pszLocalRecips;
        ++pval;

        ++plst->cEntries;

        pszLocalRecips += wcslen(pszLocalRecips) + 1;

        if (*pszEnd != L'\0')
            ++pszEnd;

        psz = pszEnd;
    }
    LOG.debug("Exiting from AddRecipients function");
}

HRESULT MailServer2Client::SetMessageRecipients (LPMESSAGE pmsg, MailClientData* m) {

    LOG.debug("Enter into SetMessageRecipients function");
    ULONG cRecipients = 0;
    ULONG cchTotalRecips = 0;

    ADRLIST* plst = NULL;
    LPSPropValue pval = NULL;
    BYTE* pb = NULL;
    LPWSTR pszLocalRecips;
    HRESULT hr;
    wchar_t *to = NULL, *cc = NULL, *bcc = NULL; 
    if (m->getEmailData()->getEmailItem().getTo()) {
        to = toWideChar(m->getEmailData()->getEmailItem().getTo());
        cRecipients += CountRecipsInString(to);
        cchTotalRecips += wcslen(to) + 3;
    }

    if (m->getEmailData()->getEmailItem().getCc()) {
        cc = toWideChar(m->getEmailData()->getEmailItem().getCc());
        cRecipients += CountRecipsInString(cc);
        cchTotalRecips += wcslen(cc) + 3;
    }

    if (m->getEmailData()->getEmailItem().getBcc()) {
        bcc = toWideChar(m->getEmailData()->getEmailItem().getBcc());
        cRecipients += CountRecipsInString(bcc);
        cchTotalRecips += wcslen(bcc) + 3;
    }
  
    //
    // Allocate one big block of memory to hold all the strings.
    // The block is arranged as follows:
    //
    // ADRLIST        SPropValue's     Copy of addresses
    // |---------------------|--------------------|-------------------|
    //

    DWORD cb = sizeof(ADRLIST) + ((sizeof(ADRENTRY) + sizeof(SPropValue) * 3) * cRecipients);

    pb = new BYTE[cb + (cchTotalRecips * sizeof(WCHAR))];
    if (pb == NULL)
    return E_OUTOFMEMORY;

    ZeroMemory(pb, cb + (cchTotalRecips * sizeof(WCHAR)));

    plst = (ADRLIST*) pb;

    pszLocalRecips = (LPWSTR) (pb + cb);

    pb += sizeof(ADRLIST) + (sizeof(ADRENTRY) * cRecipients);

    pval = (SPropValue*) pb;

    AddRecipients(to, MAPI_TO, plst, pszLocalRecips, pval);
    AddRecipients(cc, MAPI_CC, plst, pszLocalRecips, pval);
    AddRecipients(bcc, MAPI_BCC, plst, pszLocalRecips, pval);

    hr = pmsg->ModifyRecipients(MODRECIP_ADD, plst);

    delete[] (BYTE*) plst;
    if (to) {delete [] to; to = NULL;}
    if (cc) {delete [] cc; cc = NULL;}
    if (bcc) {delete [] bcc; bcc = NULL;}

    LOG.debug("Exiting from SetMessageRecipients function");
    return hr;
}

void MailServer2Client::bt2ft(const BasicTime& b, FILETIME& f)
{
    SYSTEMTIME local, s1, s2;

    local.wYear         = b.getYear();
    local.wMonth        = b.getMonth();
    local.wDay          = b.getDay();
    local.wDayOfWeek    = DayOfWeek(local.wYear, local.wMonth, local.wDay);    // b.getWeekday(); // 0 sunday, 1monday
    local.wHour         = b.getHour();
    local.wMinute       = b.getMin();
    local.wSecond       = b.getSec();
    local.wMilliseconds = 0;

    s1 = DT_AddDiff(nano100SecInHour, -(b.getTzHour()), &local); 
    if (b.getTzMin() != 0 && b.getTzHour() != 0)
        s2 = DT_AddDiff(nano100SecInMin, -(abs(b.getTzHour())/b.getTzHour())*(b.getTzMin()), &s1); 
    else s2 = DT_AddDiff(nano100SecInMin,b.getTzMin(),&s1);
    SystemTimeToFileTime(&s2, &f); 

}

/* 
* OnlyFlags says the client has to set only the PR_MESSAGE_FLAGS of the message the PR_MSG_STATUS.
* This is for the update mail. In our case the draft modification are sent from server
* as a delete of the old one and a new insert of the new one.
*/
HRESULT MailServer2Client::SetMessageProps(IMAPISession* pSession, LPMESSAGE pmsg, MailClientData* m, BOOL onlyFlags) {
  
    LOG.debug("Enter into SetMessageProps function");
    SPropValue rgprops[9] = {0};
    ULONG    cProps   = 0;
    HRESULT   hr;
    BodyPart bp;
    wchar_t *subj = NULL,   *from = NULL, 
            *messId = NULL, *headers = NULL;    

    LPSTREAM  pstm    = NULL;
    ULONG messageFlag = (ULONG) 0x00000000;        

    char* h = NULL;    
       
    //
    // Set this flag only for Inbox and Draft. This shows the From in the
    // visualization of the mail into the pocket pc    
    
    if (wcscmp(m->getParent(), TEXT("I")) == 0 || 
        wcscmp(m->getParent(), TEXT("T")) == 0) {
        
        messageFlag = messageFlag | MSGFLAG_UNMODIFIED;

    } else if (wcscmp(m->getParent(), TEXT("D")) == 0) {

        messageFlag = messageFlag | MSGFLAG_FROMME | MSGFLAG_UNSENT;

    } else if (wcscmp(m->getParent(), TEXT("S")) == 0) {

        messageFlag = messageFlag | MSGFLAG_FROMME ;
    }
          
            
    if (m->getEmailData()->getRead())
        messageFlag = messageFlag | MSGFLAG_READ;
 
    
    // Set the flags and a subject if they exist.
    rgprops[cProps].ulPropTag = PR_MESSAGE_FLAGS;
    rgprops[cProps].Value.ul = messageFlag;      
    ++cProps;
  
    rgprops[cProps].ulPropTag = PR_MSG_STATUS;
    rgprops[cProps].Value.ul = MSGSTATUS_RECTYPE_SMTP; // | MSGSTATUS_PARTIAL | MSGSTATUS_HAS_PR_CE_MIME_TEXT; // 
    ++cProps;
        
    if (!onlyFlags) {

        MailMessage &msg = m->getEmailData()->getEmailItem();       
        if (msg.attachmentCount() > 0) {
            rgprops[cProps].ulPropTag = PR_HASATTACH;
            rgprops[cProps].Value.ul = msg.attachmentCount();
            ++cProps;        
        }

        // Set the recipients up.
        hr = SetMessageRecipients(pmsg, m);      
        EXIT_ON_FAILED(hr);        
        
        //hr = SetReplyTo(pSession, pmsg, "Sync4j@yahoogroups.com");      
        //EXIT_ON_FAILED(hr);        
        

        if (msg.getSubject() != NULL) {
            rgprops[cProps].ulPropTag = PR_SUBJECT;
            subj = toWideChar(msg.getSubject());
            rgprops[cProps].Value.lpszW = subj;
            ++cProps;
        }
        if (msg.getFrom() != NULL) {
            rgprops[cProps].ulPropTag = PR_SENDER_EMAIL_ADDRESS;
            from = toWideChar(msg.getFrom());
            rgprops[cProps].Value.lpszW = from;
            ++cProps;
        }

        FILETIME ft;
        
        // Received time (Received: header) to view externally. currently it isn't of use
        bt2ft(msg.getReceived(), ft);
        rgprops[cProps].ulPropTag = PR_MESSAGE_DELIVERY_TIME;
        rgprops[cProps].Value.ft = ft;
        ++cProps;

        // Sent time (Date: header). to view internally. this is right
        bt2ft(msg.getDate(), ft);
        rgprops[cProps].ulPropTag = PR_CLIENT_SUBMIT_TIME; // PR_DELIVER_TIME; // // PR_LAST_MODIFICATION_TIME;
        rgprops[cProps].Value.ft = ft;
        ++cProps;                
        
        // TODO: MODIFY THE  rgprops TOO!|!!!!!!!!
        //
        rgprops[cProps].ulPropTag = PR_CUSTOM_MESSAGE_ID;
        messId = toWideChar(msg.getMessageId());
        if (messId) {
            rgprops[cProps].Value.lpszW  = messId;
            ++cProps;       
        }

        rgprops[cProps].ulPropTag = PR_CUSTOM_MESSAGE_HEADER;
        h = msg.getHeaders();
        headers = toWideChar(h);
        if (headers) {
            rgprops[cProps].Value.lpszW  = headers;
            ++cProps;       
        }    

        // Body  
        bp = msg.getBody();      
        if (bp.getContent()) {
            // Stream the body in...
            hr = pmsg->OpenProperty(PR_BODY, NULL, 0, MAPI_MODIFY | MAPI_CREATE, 
              (LPUNKNOWN*)&pstm);
            EXIT_ON_FAILED(hr);
            wchar_t *content = toWideChar(bp.getContent(), bp.getCharset());
            pstm->Write(content, (wcslen(content)+1)*sizeof(wchar_t), NULL);
            pstm->Release();
            delete [] content;
        }

        // Attachment
        AddAttachment(pmsg, m);
    }
    hr = pmsg->SetProps(cProps, rgprops, NULL);
    EXIT_ON_FAILED(hr);

FuncExit:
    if(subj)
        delete [] subj;
    if(from)
        delete [] from;
    if (messId) {
         delete [] messId; messId = NULL;
    }
    if (headers) {
         delete [] headers; headers = NULL;
    }
    if (h) {
        delete [] h; h = NULL;
    }
    LOG.debug("Exiting from SetMessageProps function");
    return hr;
}

void MailServer2Client::setFolderToSync(const wchar_t* t){
    if (server_folderToSync) {
        delete [] server_folderToSync; server_folderToSync = NULL;
    }
    server_folderToSync = wstrdup(t);
}

const wchar_t* MailServer2Client::getFolderToSync() {
    return server_folderToSync;
}



/*
* Return the ID of the inserted message
*/

wchar_t* MailServer2Client::addMessage(MailClientData* m) {

    LOG.debug("Enter into addMessage function");
    HRESULT           hr;
    IMAPISession *    pSession    =  NULL;
    IMAPITable*       pTable      =  NULL;
    SRowSet*          psrs        =  NULL;
    IMsgStore*        pStore      =  NULL;
    int               i           =  0;
    LPADRBOOK lpAddrBook = NULL;
    ULONG*   rgTags;
    folderNumber = 0;
    BOOL accountFound             = FALSE;

    if (server_folderToSync == NULL) {
        rgTags = new ULONG[2];
        rgTags[0] = 1; 
        rgTags[1] = PR_CE_IPM_INBOX_ENTRYID; 
        folderNumber = 1;
    }
    else {
        folderNumber = wcslen(server_folderToSync);
        rgTags = new ULONG[folderNumber + 1];         
        rgTags[0] = folderNumber;
        for (int i = 1; i <= folderNumber; i++) 
            rgTags[i] = getFolderId(server_folderToSync[i-1]);    // PR_CE_IPM_INBOX_ENTRYID; 
                                                      // PR_CE_IPM_DRAFTS_ENTRYID;        
    }
    
    ULONG             rgMsgTags[] =  { 1, PR_ENTRYID };
    ULONG             cValues     =  0;
    ULONG             returned    =  0;
    
    SPropValue*       rgprops     =  NULL;
    SPropValue*       rgpropsMsg  =  NULL;
    LPMAPIFOLDER      pfldrInbox  =  NULL;
    LPMAPIFOLDER      pfldrOutbox =  NULL;

    IMessage*         pmsg        =  NULL;
    ULONG             cmsgValue   =  0;
    SPropValue*       msgprops    =  NULL;
            
	LPSRowSet	pRows = NULL;    
    SPropValue*       rgprops2    =  NULL;
    ULONG             cValues2    =  0;
    wchar_t* ret                  = NULL;
	bool msgset = false;      
    
    // First log on to the store.
    hr = MAPILogonEx (NULL, NULL, NULL, NULL, &pSession);
    EXIT_ON_FAILED (hr);

    // Get the message stores table
    hr = pSession->GetMsgStoresTable (MAPI_UNICODE, &pTable);
    EXIT_ON_FAILED (hr);
        
    hr = pTable->SetColumns((LPSPropTagArray)&Columns, 0);
    EXIT_ON_FAILED(hr);
       

    while (!msgset){
        // Get a row
        hr = pTable->QueryRows (1, 0, &psrs);
        EXIT_ON_FAILED (hr);
        
        // Did we hit the end of the table?
        if (psrs->cRows != 1) {
          break;
        }
        //
        // the display name is the name of the account.
        //
        wchar_t* displayName = psrs->aRow[0].lpProps[eePR_DISPLAY_NAME].Value.lpszW;
        
        //
        // The name of the MessageStore that we use for the own Transport
        //
        if (wcscmp(displayName, ACCOUNT_NAME) == 0) {
            accountFound = TRUE;
            // Make sure we got the props we need
            if ((psrs->aRow[0].cValues < 1) || (psrs->aRow[0].lpProps[0].ulPropTag != PR_ENTRYID))
            {
              LOG.error("Store missing PR_ENTRYID!");
              hr = E_FAIL;
              break;
            }

            // Open this message store
            hr = pSession->OpenMsgStore (NULL, 
                           psrs->aRow[0].lpProps[0].Value.bin.cb, 
                          (ENTRYID *) psrs->aRow[0].lpProps[0].Value. 
                           bin.lpb, NULL, 0, &pStore);
            EXIT_ON_FAILED (hr);
      
            // Now get the Inbox folder.
            hr = pStore->GetProps ((SPropTagArray *) rgTags, MAPI_UNICODE, &cValues, &rgprops);
            EXIT_ON_FAILED (hr);
            
            if(!rgprops){
                LOG.error("Error getting folder from message store.");
                goto FuncExit;
            }
            
            //
            // watch at the folder that we want to sync
            //
            for (i = 0; i < folderNumber; i++) {
                wchar_t* tmp = c2wc(server_folderToSync[i]);

                if (wcscmp(tmp, m->getParent()) != 0) {
                    if (tmp) delete [] tmp; tmp = NULL;
                    continue;
                }
            
                // open the message store
                hr = pStore->OpenEntry (rgprops[i].Value.bin.cb, 
                            (LPENTRYID)rgprops[i].Value.bin.lpb, 
                            NULL, 
                            MAPI_MODIFY, 
                            NULL, 
                            (IUnknown **) &pfldrInbox);
                EXIT_ON_FAILED (hr);
            
                if (!pfldrInbox){
                    LOG.error("MailServer2Client::addMessage: pfldrInbox NULL!");
                    goto FuncExit;
                }
                
                if (doesMessageExist(m->getEmailData()->getEmailItem().getMessageId(), pfldrInbox)) {
                    ret = new wchar_t[64];
                    wcscpy(ret, ITEM_ARLEADY_EXISTS);
                }
                else {
                    hr = pfldrInbox->CreateMessage (NULL, 0, &pmsg);
                    EXIT_ON_FAILED (hr);
          
                    if (!pmsg){
                        LOG.error("MailServer2Client::addMessage: pmsg NULL!");
                        goto FuncExit;
                    }
                                    
                    // Now set the props on the message....
                    hr = SetMessageProps (pSession, pmsg, m);
                    EXIT_ON_FAILED(hr);
                    
                    LOG.debug("Message props set.");

                    //
                    // Get the ID of the new message
                    //
                    hr = pmsg->GetProps ((SPropTagArray *) rgMsgTags, MAPI_UNICODE, &cValues2, &rgprops2);
                    EXIT_ON_FAILED(hr);
                    
                    if (rgprops2[eePR_ENTRYID].ulPropTag == PR_ENTRYID) {
                        ret = convertBinaryToWChar(rgprops2[eePR_ENTRYID].Value.bin, server_folderToSync[i]);
                        LOG.debug("Message enry id: %ls", ret);
                    }
                    else {
                        LOG.error("Error getting msg entry id.");
                    }
                    
                }
                //
                // If there is a mail in Outbox with the same messageId of a mail just inserted in Sent, 
                // then delete the Outbox one.
                //
                
                if (server_folderToSync[i] == 'S') {
                    if (messageExistInOubox_deleteIt(pStore, m->getEmailData()->getEmailItem().getMessageId()) ){
                        LOG.debug("Message in outbox succesfully deleted. Message id: %s", m->getEmailData()->getEmailItem().getMessageId());
                    } else {
                        LOG.debug("No message in outbox to delete. Message id: %s", m->getEmailData()->getEmailItem().getMessageId());
                    }

                }
                
                msgset = true;
                break;  // found.
            }        
        }
                
        // Clean up
        MAPIFreeBuffer (rgprops);    
        MAPIFreeBuffer (rgprops2);    
        MAPIFreeBuffer (rgpropsMsg);  
        FreeProws   (psrs);     
        
        rgprops   =  NULL;  
        rgpropsMsg =  NULL; 
        psrs    =  NULL;    

        RELEASE_OBJ (pmsg);
        RELEASE_OBJ (pfldrInbox);
        RELEASE_OBJ (pStore);
    }
    if (accountFound == FALSE) {
        LOG.error("No Funambol account found");
    }
FuncExit:
  
    MAPIFreeBuffer (rgprops);
    MAPIFreeBuffer (rgpropsMsg);
    FreeProws   (psrs);

    RELEASE_OBJ (pmsg);
    RELEASE_OBJ (pfldrInbox);
    RELEASE_OBJ (pStore);
    RELEASE_OBJ (pTable);
    RELEASE_OBJ (pSession);

    if (rgTags)
        delete rgTags;
    LOG.debug("Exiting from addMessage function");
    return ret;
}


/*
* Return the ID of the inserted message
*/

HRESULT MailServer2Client::updateMessage(MailClientData* m) {
    LOG.debug("Enter into updateMessage function");
    HRESULT           hr;
    IMAPISession *    pSession    =  NULL;
    IMAPITable*       pTable      =  NULL;
    SRowSet*          psrs        =  NULL;
    IMsgStore*        pStore      =  NULL;
    int               i           =  0;
    ULONG*   rgTags;
    folderNumber = 0;
    BOOL accountFound             = FALSE;

    if (server_folderToSync == NULL) {
        rgTags = new ULONG[2];
        rgTags[0] = 1; 
        rgTags[1] = PR_CE_IPM_INBOX_ENTRYID; 
        folderNumber = 1;
    }
    else {
        folderNumber = wcslen(server_folderToSync);
        rgTags = new ULONG[folderNumber + 1];         
        rgTags[0] = folderNumber;
        for (int i = 1; i <= folderNumber; i++) 
            rgTags[i] = getFolderId(server_folderToSync[i-1]);    // PR_CE_IPM_INBOX_ENTRYID; 
                                                      // PR_CE_IPM_DRAFTS_ENTRYID;        
    }
    
    ULONG             rgMsgTags[] =  { 1, PR_ENTRYID };
    ULONG             cValues     =  0;
    ULONG             returned    =  0;
    
    SPropValue*       rgprops     =  NULL;
    SPropValue*       rgpropsMsg  =  NULL;
    LPMAPIFOLDER      pfldrInbox  =  NULL;

    IMessage*         pmsg        =  NULL;
    ULONG             cmsgValue   =  0;
    SPropValue*       msgprops    =  NULL;
            
	LPSRowSet	      pRows        = NULL;    
    SPropValue*       rgprops2     =  NULL;
    ULONG             cValues2     =  0;
    wchar_t*          ret         = NULL;
    wchar_t*          displayName  = NULL;
	bool msgset = false;
    
    // First log on to the store.
    hr = MAPILogonEx (NULL, NULL, NULL, NULL, &pSession);
    EXIT_ON_FAILED (hr);

    // Get the message stores table
    hr = pSession->GetMsgStoresTable (MAPI_UNICODE, &pTable);
    EXIT_ON_FAILED (hr);
    
    hr = pTable->SetColumns((LPSPropTagArray)&Columns, 0);
    EXIT_ON_FAILED(hr);
    
    while (!msgset){
        // Get a row
        hr = pTable->QueryRows (1, 0, &psrs);
        EXIT_ON_FAILED (hr);
        
        // Did we hit the end of the table?
        if (psrs->cRows != 1) {
            break;
        }
        //
        // the display name is the name of the account.
        //
        displayName = psrs->aRow[0].lpProps[eePR_DISPLAY_NAME].Value.lpszW;
        
        //
        // The name of the MessageStore that we use for the own Transport
        //
        if (wcscmp(displayName, ACCOUNT_NAME) == 0) {
            accountFound = TRUE;
            // Make sure we got the props we need
            if ((psrs->aRow[0].cValues < 1) || (psrs->aRow[0].lpProps[0].ulPropTag != PR_ENTRYID))
            {
                MessageBox (NULL, L"Store missing PR_ENTRYID!", L"Doh", MB_OK);
                hr = E_FAIL;
                break;
            }

            // Open this message store
            hr = pSession->OpenMsgStore (NULL, 
                           psrs->aRow[0].lpProps[0].Value.bin.cb, 
                          (ENTRYID *) psrs->aRow[0].lpProps[0].Value. 
                           bin.lpb, NULL, 0, &pStore);
            EXIT_ON_FAILED (hr);
      
            // Now get the Inbox folder.
            hr = pStore->GetProps ((SPropTagArray *) rgTags, MAPI_UNICODE, &cValues, &rgprops);
            EXIT_ON_FAILED (hr);
      
            ASSERT (rgprops);
                   
            //
            // watch at the folder that we want to sync
            //
            for (i = 0; i < folderNumber; i++) {
                wchar_t* t = c2wc(server_folderToSync[i]);

                if (wcscmp(t, m->getParent()) != 0) {
                    if (t) {
                        delete [] t; t = NULL; 
                    }
                    continue; 
                }
            
                // open the message store
                hr = pStore->OpenEntry (rgprops[i].Value.bin.cb, 
                            (LPENTRYID)rgprops[i].Value.bin.lpb, 
                            NULL, 
                            MAPI_MODIFY, 
                            NULL, 
                            (IUnknown **) &pfldrInbox);
                EXIT_ON_FAILED (hr);
            
                ASSERT (pfldrInbox);  
                
                // Decode the id
                char* eid = NULL;
                size_t len = decodeEntryId(m->getEntryId(), &eid);
                if(!eid){
                    LOG.error("updateMessage: error decoding entryId");
                    return -1;
                }
                
                ENTRYLIST entryList;
                
                entryList.cValues = 0;
			    MAPIAllocateBuffer(sizeof(SBinary), (LPVOID*)&(entryList.lpbin));
                entryList.lpbin[entryList.cValues].cb = len;
                MAPIAllocateBuffer(len, (LPVOID*)&(entryList.lpbin[entryList.cValues].lpb) );
                memmove(entryList.lpbin[entryList.cValues].lpb, eid, len);
                entryList.cValues++;
                
                hr = pSession->OpenEntry(entryList.lpbin[0].cb, 
                    (ENTRYID*)entryList.lpbin[0].lpb, 
                    NULL, 
                    MAPI_BEST_ACCESS, 
                    &returned, 
                    (IUnknown **) &pmsg);

                EXIT_ON_FAILED (hr);        
                if(!pmsg){
                    LOG.error("Pmsg is null. ");
                    goto FuncExit;
                }

                delete [] eid;
                
                // Now set the props on the message....
                hr = SetMessageProps (pSession, pmsg, m, true); // update only the flags
                EXIT_ON_FAILED(hr);                       
               
                MAPIFreeBuffer (entryList.lpbin);   
                
                msgset = true;
                break;

            }        
        }      
        
        // Clean up
        MAPIFreeBuffer (rgprops);    
        MAPIFreeBuffer (rgprops2);    
        MAPIFreeBuffer (rgpropsMsg);  
        FreeProws   (psrs);     

        rgprops   =  NULL;  
        rgpropsMsg =  NULL; 
        psrs    =  NULL;    

        RELEASE_OBJ (pmsg);
        RELEASE_OBJ (pfldrInbox);
        RELEASE_OBJ (pStore);
                
    }
    if (accountFound == FALSE) {
        LOG.error("No Funambol account found");
    }
FuncExit:
  
    MAPIFreeBuffer (rgprops);
    MAPIFreeBuffer (rgpropsMsg);
    FreeProws   (psrs);

    RELEASE_OBJ (pmsg);
    RELEASE_OBJ (pfldrInbox);
    RELEASE_OBJ (pStore);
    RELEASE_OBJ (pTable);
    RELEASE_OBJ (pSession);

    if (rgTags)
        delete rgTags;
    LOG.debug("Exiting from updateMessage function");
    return hr;
}



/*
* Return the ID of the inserted message
*/

HRESULT MailServer2Client::deleteMessage(MailClientData* m) {
    LOG.debug("Enter into deleteMessage function");
    HRESULT           hr;
    IMAPISession *    pSession    =  NULL;
    IMAPITable*       pTable      =  NULL;
    SRowSet*          psrs        =  NULL;
    IMsgStore*        pStore      =  NULL;
    int               i           =  0;
    ULONG*   rgTags;
    folderNumber = 0;
    BOOL accountFound             = FALSE;

    if (server_folderToSync == NULL) {
        rgTags = new ULONG[2];
        rgTags[0] = 1; 
        rgTags[1] = PR_CE_IPM_INBOX_ENTRYID; 
        folderNumber = 1;
    }
    else {
        folderNumber = wcslen(server_folderToSync);
        rgTags = new ULONG[folderNumber + 1];         
        rgTags[0] = folderNumber;
        for (int i = 1; i <= folderNumber; i++) 
            rgTags[i] = getFolderId(server_folderToSync[i-1]);    // PR_CE_IPM_INBOX_ENTRYID; 
                                                      // PR_CE_IPM_DRAFTS_ENTRYID;        
    }
    
    ULONG             rgMsgTags[] =  { 1, PR_ENTRYID };
    ULONG             cValues     =  0;
    ULONG             returned    =  0;
    
    SPropValue*       rgprops     =  NULL;
    SPropValue*       rgpropsMsg  =  NULL;
    LPMAPIFOLDER      pfldrInbox  =  NULL;

    IMessage*         pmsg        =  NULL;
    ULONG             cmsgValue   =  0;
    SPropValue*       msgprops    =  NULL;
            
	LPSRowSet	      pRows        = NULL;    
    SPropValue*       rgprops2     =  NULL;
    ULONG             cValues2     =  0;
    wchar_t*          ret         = NULL;
    wchar_t*          displayName  = NULL;
	bool msgset = false;
     
    // First log on to the store.
    hr = MAPILogonEx (NULL, NULL, NULL, NULL, &pSession);
    EXIT_ON_FAILED (hr);

    // Get the message stores table
    hr = pSession->GetMsgStoresTable (MAPI_UNICODE, &pTable);
    EXIT_ON_FAILED (hr);
   
    hr = pTable->SetColumns((LPSPropTagArray)&Columns, 0);
    EXIT_ON_FAILED(hr);
     
    while (!msgset){
        // Get a row
        hr = pTable->QueryRows (1, 0, &psrs);
        EXIT_ON_FAILED (hr);
        
        // Did we hit the end of the table?
        if (psrs->cRows != 1) {
            break;
        }
        //
        // the display name is the name of the account.
        //
        displayName = psrs->aRow[0].lpProps[eePR_DISPLAY_NAME].Value.lpszW;
    
        //
        // The name of the MessageStore that we use for the own Transport
        //
        if (wcscmp(displayName, ACCOUNT_NAME) == 0) {
            accountFound = TRUE;
            // Make sure we got the props we need
            if ((psrs->aRow[0].cValues < 1) || (psrs->aRow[0].lpProps[0].ulPropTag != PR_ENTRYID))
            {
              LOG.error("Store missing PR_ENTRYID!");
              hr = E_FAIL;
              break;
            }

            // Open this message store
            hr = pSession->OpenMsgStore (NULL, 
                           psrs->aRow[0].lpProps[0].Value.bin.cb, 
                          (ENTRYID *) psrs->aRow[0].lpProps[0].Value. 
                           bin.lpb, NULL, 0, &pStore);
            EXIT_ON_FAILED (hr);
      
            // Now get the Inbox folder.
            hr = pStore->GetProps ((SPropTagArray *) rgTags, MAPI_UNICODE, &cValues, &rgprops);
            EXIT_ON_FAILED (hr);
      
            ASSERT (rgprops);
                   
            //
            // watch at the folder that we want to sync
            //
            for (i = 0; i < folderNumber; i++) {
                wchar_t* t = c2wc(server_folderToSync[i]);
              
                if (wcscmp(t, m->getParent()) != 0) {
                    if (t) {
                        delete [] t; t = NULL; 
                    }
                    continue; 
                }
            
                // open the message store
                hr = pStore->OpenEntry (rgprops[i].Value.bin.cb, 
                            (LPENTRYID)rgprops[i].Value.bin.lpb, 
                            NULL, 
                            MAPI_MODIFY, 
                            NULL, 
                            (IUnknown **) &pfldrInbox);
                EXIT_ON_FAILED (hr);
            
                ASSERT (pfldrInbox);  
                            
                // Decode the id
                char* eid = NULL;
                size_t len = decodeEntryId(m->getEntryId(), &eid);
                if(!eid){
                    LOG.error("updateMessage: error decoding entryId");
                    return -1;
                }
                
                ENTRYLIST entryList;
                
                entryList.cValues = 0;
			    MAPIAllocateBuffer(sizeof(SBinary), (LPVOID*)&(entryList.lpbin));
                entryList.lpbin[entryList.cValues].cb = len;
		        MAPIAllocateBuffer(len, (LPVOID*)&(entryList.lpbin[entryList.cValues].lpb) );
		        memmove(entryList.lpbin[entryList.cValues].lpb, eid, len);
		        entryList.cValues++;
                
                delete [] eid;

                hr = pfldrInbox->DeleteMessages (&entryList, 0, NULL, 0);
                EXIT_ON_FAILED (hr);
               
                MAPIFreeBuffer (entryList.lpbin);  

                msgset = true;
                break;
            }        
        }        
        // Clean up
        MAPIFreeBuffer (rgprops);    
        MAPIFreeBuffer (rgprops2);    
        MAPIFreeBuffer (rgpropsMsg);  
        FreeProws   (psrs);     
        
        rgprops   =  NULL;  
        rgpropsMsg =  NULL; 
        psrs    =  NULL;    

        RELEASE_OBJ (pmsg);
        RELEASE_OBJ (pfldrInbox);
        RELEASE_OBJ (pStore);
    }
    if (accountFound == FALSE) {
        LOG.error("No Funambol account found");
        goto FuncExit;
    }
FuncExit:
  
    MAPIFreeBuffer (rgprops);
    MAPIFreeBuffer (rgpropsMsg);
    FreeProws   (psrs);

    RELEASE_OBJ (pmsg);
    RELEASE_OBJ (pfldrInbox);
    RELEASE_OBJ (pStore);
    RELEASE_OBJ (pTable);
    RELEASE_OBJ (pSession);

    if (rgTags)
        delete rgTags;
    LOG.debug("Exiting from deleteMessage function");
    return hr;
}

bool MailServer2Client::messageExistInOubox_deleteIt(IMsgStore* pStore, IN const char* msgid) {
	
    if (msgid == NULL) 
        return false;

    wchar_t* msgID = toWideChar(msgid);
    ULONG           cValues     =   0;
    SPropValue  *   rgprops     =   NULL;
    HRESULT hr                  = S_OK;
    bool bFound                 = false;

	SBinary	sbEntry = {0};
	LPMAPITABLE pContentsTable = NULL;
    LPMAPIFOLDER pfldrOutbox =   NULL;
	LPSRowSet pRows = NULL;
	SRow aRow = {0};
	
    ULONG rgTags[] = { 1, PR_IPM_OUTBOX_ENTRYID };

	enum {
		ePR_ENTRYID,			// 0		
		ePR_CUSTOM_MESSAGE_ID,			// 2
		NUM_COLS				// number of columns (put it to the end)
	};

	// These tags represent the message information we would like to pick up
	SizedSPropTagArray(NUM_COLS, Columns) =
	{
		NUM_COLS,
		PR_ENTRYID,
		PR_CUSTOM_MESSAGE_ID
	};
    
    hr = pStore->GetProps((SPropTagArray *)rgTags, MAPI_UNICODE, &cValues, &rgprops);
    EXIT_ON_FAILED(hr);

    ASSERT(rgprops);
    ASSERT(rgprops[0].ulPropTag == PR_IPM_OUTBOX_ENTRYID);

    hr = pStore->OpenEntry (rgprops[0].Value.bin.cb, 
                            (LPENTRYID)rgprops[0].Value.bin.lpb, 
                            NULL, 
                            MAPI_MODIFY, 
                            NULL, 
                            (IUnknown **) &pfldrOutbox);

    EXIT_ON_FAILED(hr);

	hr = pfldrOutbox->GetContentsTable(0, &pContentsTable);
	EXIT_ON_FAILED(hr);

	hr = pContentsTable->SetColumns((LPSPropTagArray)&Columns, 0);
	EXIT_ON_FAILED(hr);

	// iterate message one by one (each time only read 1)
	while(!FAILED(hr = pContentsTable->QueryRows(1, 0, &pRows))) {
		// this is the end of the content table
		if(pRows->cRows == 0) break;

		// check this Row
		aRow = pRows->aRow[0];

		// Correctly returned?
		if ((&aRow.lpProps[ePR_ENTRYID])->ulPropTag == PR_ENTRYID			
			&& (&aRow.lpProps[ePR_CUSTOM_MESSAGE_ID])->ulPropTag == PR_CUSTOM_MESSAGE_ID) {

			// Is CUSTOM_MESSAGE_ID found?
            wchar_t* mm = (&aRow.lpProps[ePR_CUSTOM_MESSAGE_ID])->Value.lpszW;
            
            // the message-id there is in the client has the O/AABBCC....
            // the message-id sent by server could have only AABBCC...

            if (wcsstr(mm, msgID) != 0) { 
				                
                wchar_t* entryId = convertBinaryToWChar(aRow.lpProps[ePR_ENTRYID].Value.bin, 'O');
                char* eid = NULL;
                size_t len = decodeEntryId(entryId, &eid);
                if(!eid){
                    LOG.error("messageExistInOubox_deleteIt: error decoding entryId");
                    return false;
                }
                
                ENTRYLIST entryList;
                
                entryList.cValues = 0;
			    MAPIAllocateBuffer(sizeof(SBinary), (LPVOID*)&(entryList.lpbin));
                entryList.lpbin[entryList.cValues].cb = len;
		        MAPIAllocateBuffer(len, (LPVOID*)&(entryList.lpbin[entryList.cValues].lpb) );
		        memmove(entryList.lpbin[entryList.cValues].lpb, eid, len);
		        entryList.cValues++;
                
                delete [] eid;
                delete [] entryId;

                hr = pfldrOutbox->DeleteMessages (&entryList, 0, NULL, 0);
                EXIT_ON_FAILED (hr);
                bFound = true;

				break;
			}
		} // if

		// Free pRows for each row
		FreeProws(pRows);
		pRows = NULL;
	} // while	

FuncExit:
	if(pRows) {
		// Free each element in the pRows buffer
		FreeProws(pRows);
		pRows = NULL;
	}
	RELEASE_OBJ(pContentsTable);
    
    if (msgID)  {
        delete [] msgID; msgID = NULL;
    }

	return bFound;
}


bool MailServer2Client::doesMessageExist (IN const char* msgid, LPMAPIFOLDER pfldrInbox) {
	
    if (msgid == NULL) 
        return false;
    
    wchar_t* msgID = toWideChar(msgid);
	HRESULT hr = S_OK;
	bool bFound = false;
	SBinary	sbEntry = {0};
	LPMAPITABLE pContentsTable = NULL;
	LPSRowSet pRows = NULL;
	SRow aRow = {0};
	
	enum {
		ePR_ENTRYID,			// 0		
		ePR_CUSTOM_MESSAGE_ID,			// 2
		NUM_COLS				// number of columns (put it to the end)
	};

	// These tags represent the message information we would like to pick up
	SizedSPropTagArray(NUM_COLS, Columns) =
	{
		NUM_COLS,
		PR_ENTRYID,
		PR_CUSTOM_MESSAGE_ID
	};

	hr = pfldrInbox->GetContentsTable(0, &pContentsTable);
	EXIT_ON_FAILED(hr);

	hr = pContentsTable->SetColumns((LPSPropTagArray)&Columns, 0);
	EXIT_ON_FAILED(hr);

	// iterate message one by one (each time only read 1)
	while(!FAILED(hr = pContentsTable->QueryRows(1, 0, &pRows))) {
		// this is the end of the content table
		if(pRows->cRows == 0) break;

		// check this Row
		aRow = pRows->aRow[0];

		// Correctly returned?
		if ((&aRow.lpProps[ePR_ENTRYID])->ulPropTag == PR_ENTRYID			
			&& (&aRow.lpProps[ePR_CUSTOM_MESSAGE_ID])->ulPropTag == PR_CUSTOM_MESSAGE_ID) {

			// Is CUSTOM_MESSAGE_ID found?
            wchar_t* mm = aRow.lpProps[ePR_CUSTOM_MESSAGE_ID].Value.lpszW;
            if (wcscmp(mm, msgID) == 0) {
				bFound = true;
				break;
			}
		} // if

		// Free pRows for each row
		FreeProws(pRows);
		pRows = NULL;
	} // while	

FuncExit:
	if(pRows) {
		// Free each element in the pRows buffer
		FreeProws(pRows);
		pRows = NULL;
	}
	RELEASE_OBJ(pContentsTable);
    
    if (msgID)  {
        delete [] msgID; msgID = NULL;
    }

	return bFound;
}



HRESULT SetReplyTo(LPMAPISESSION lpSession, IMessage* lpMessage, LPSTR pszReplyTo)
{
  
    HRESULT hRes = S_OK;
    LPADRLIST lpAddrList = NULL;
    LPADRBOOK lpAddrBook = NULL;
    SPropValue pspvReply[2];
    LPFLATENTRYLIST lpReplyEntryList = NULL;
    LPFLATENTRY lpReplyEntry = NULL;
    int cValues = 2;
    ULONG i = 0;

    // Allocate an address list structure.
    int cb = CbNewADRLIST(1);

    hRes = MAPIAllocateBuffer(cb, (LPVOID *)&lpAddrList);   
    if (FAILED(hRes)) goto Cleanup;

    ZeroMemory(lpAddrList, cb);

    hRes = MAPIAllocateBuffer(
        cValues * sizeof(SPropValue), 
        (LPVOID FAR *)&lpAddrList->aEntries[0].rgPropVals);
    if (FAILED(hRes)) goto Cleanup;

    // Open the address book to resolve Reply To recipient.
    hRes = lpSession->OpenAddressBook(0, NULL, 0, &lpAddrBook);
    if (FAILED(hRes)) goto Cleanup;

    // Set up address list structure.
    lpAddrList->cEntries = 1;
    lpAddrList->aEntries[0].rgPropVals[0].ulPropTag = PR_DISPLAY_NAME;   
    lpAddrList->aEntries[0].rgPropVals[0].Value.lpszA = pszReplyTo;

    lpAddrList->aEntries[0].rgPropVals[1].ulPropTag = PR_RECIPIENT_TYPE;
    lpAddrList->aEntries[0].rgPropVals[1].Value.l = MAPI_TO;
    lpAddrList->aEntries[0].cValues = cValues;

    // Resolve name.
    hRes = lpAddrBook->ResolveName(0, 0, NULL, lpAddrList);
    if (FAILED(hRes)) goto Cleanup;

    // Get the Entry ID.
    for (i = 0; i < lpAddrList->aEntries[0].cValues; i++)
    {
        if (lpAddrList->aEntries[0].rgPropVals[i].ulPropTag == PR_ENTRYID)
        {
            // Allocate a new FLATENTRY structure for 
            // the PR_REPLY_RECIPIENT_ENTRIES property.
            cb = CbNewFLATENTRY(
                   lpAddrList->aEntries[0].rgPropVals[i].Value.bin.cb);
            hRes = MAPIAllocateBuffer(cb, (LPVOID *)&lpReplyEntry);
            if (FAILED(hRes)) goto Cleanup;
            ZeroMemory((VOID *)lpReplyEntry, cb);

            // Copy the bits of the Entry ID into the FLATENTRY structure.
            CopyMemory(lpReplyEntry->abEntry, 
                       lpAddrList->aEntries[0].rgPropVals[i].Value.bin.lpb,
                       lpAddrList->aEntries[0].rgPropVals[i].Value.bin.cb);
            lpReplyEntry->cb = lpAddrList->
                                 aEntries[0].rgPropVals[i].Value.bin.cb;

            // Allocate a new FLATENTRYLIST structure.
            // This is what is stored in PR_REPLY_RECIPIENT_ENTRIES.
            cb = CbNewFLATENTRYLIST(cb);
            hRes = MAPIAllocateBuffer(cb, (LPVOID *)&lpReplyEntryList);
            if (FAILED(hRes)) goto Cleanup;
            ZeroMemory((VOID *)lpReplyEntryList, cb);

            // Copy the bits of the FLATENTRY into the FLATENTRYLIST.
            lpReplyEntryList->cEntries  = 1;
            lpReplyEntryList->cbEntries = CbFLATENTRY(lpReplyEntry);
            CopyMemory(lpReplyEntryList->abEntries, 
                       lpReplyEntry, CbFLATENTRY(lpReplyEntry));

            // Force exit from the loop.
            i = lpAddrList->aEntries[0].cValues;
        }
    }

    // Set PR_REPLY_RECIPIENT_NAMES.
    pspvReply[0].ulPropTag = PR_REPLY_RECIPIENT_NAMES;
    pspvReply[0].Value.lpszA = pszReplyTo;

    // Set PR_REPLY_RECIPIENT_ENTRIES.
    pspvReply[1].ulPropTag = PR_REPLY_RECIPIENT_ENTRIES;
    // Allocate memory in the lpb to hold the FLATENTRYLIST
    hRes = MAPIAllocateBuffer(CbFLATENTRYLIST(lpReplyEntryList), 
                              (LPVOID *)&pspvReply[1].Value.bin.lpb);
    // Copy the memory into the SPropValue.
    CopyMemory(pspvReply[1].Value.bin.lpb,
               lpReplyEntryList,
               CbFLATENTRYLIST(lpReplyEntryList));
    pspvReply[1].Value.bin.cb = CbFLATENTRYLIST(lpReplyEntryList);

    // Set the properties.
    hRes = lpMessage->SetProps(2,
                               pspvReply,
                               NULL);

Cleanup:
    if (lpReplyEntry) MAPIFreeBuffer(lpReplyEntry);
    if (lpReplyEntryList) MAPIFreeBuffer(lpReplyEntryList);
    //if (lpAddrList) FreePadrlist(lpAddrList);
    if (lpAddrBook) lpAddrBook->Release();
    return hRes;
}
