/*
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef INCL_MAIL_CLIENT2SERVER
#define INCL_MAIL_CLIENT2SERVER

#include "base/fscapi.h"
#include "base/util/ArrayList.h"
#include "base/util/ArrayElement.h"
#include "pim/mail/MailInfo.h"
#include "pim/utils.h"
#include "pim/mail/MailUtils.h"
#include "spds/constants.h"
#include "spds/SyncItem.h"
#include "spds/MailMessage.h"
#include "pim/mail/MailClientData.h"
#include "pim/mail/MailExtraData.h"
#include "spds/constants.h"

#include <mapidefs.h>
#include <mapicode.h>
#include <mapitags.h>
#include <mapix.h>
#include <mapiutil.h>
#include <cemapi.h>

#define ONLY_ALL_MESSAGE_LIST       -1

class MailClient2Server{

private:
    wchar_t* folderToSync;
    BOOL isMailInOutbox;

    /**
     * Avoid object copy
     */
    MailClient2Server(MailClient2Server& o);

    MailClient2Server& operator=(MailClient2Server& o);

public:
    
    /**
     * Constructor: create a Container
     */
    MailClient2Server();    

    /**
     * Destructor: 
     */
    ~MailClient2Server() ;        
   
    /**
    * folder to sync. It keeps a copy of folder to sync from sync source
    */
    void setFolderToSync(const wchar_t* t);
    wchar_t* getFolderToSync();
    
    void setIsMailInOutbox(BOOL value);
    BOOL getIsMailInOutbox();
        
    /*
    * number of folder to sync
    */
    int folderNumber;

    long arrayIndex;
    long elementIndex;
    long allTot;
    
    ENTRYLIST*   entryListArray; //array of ENTRYLIST
    //ENTRYLIST   entryList;
    
    ENTRYLIST*   newMessagesArray;
    ENTRYLIST*   updatedMessagesArray;
    ArrayList**  deletedMessagesArray;

    // ENTRYLIST   newMessages;
    // ENTRYLIST   updatedMessages;  
    // ArrayList*  deletedMessages;

    //MailMessage* mailMessage;
    
    /*
     * Function to complete the object to sync. It returns a complete object representing and email.
     * The isUpdate parameter says the mailClientData to fill is an updated item. In this case it full 
     * completes the object only if it is Draft folder else it fills only the syncml email header info 
     * (currently the LastModificationTime and the read parameter)
     */
    MailClientData* completeObject(IMessage* pmsg, wchar_t folder, BOOL isUpdate, BOOL keyOnly = FALSE);
    // wchar_t* convertBinaryToWChar(SBinary sbEntry, wchar_t folder);

    void logOff(IMAPISession* pSession);
    IMAPISession* logOn();

    /**
    * It returns a new MailContainer object if there is. NULL if there isn't.
    * Note that it returns an object created with the new operator and must be fred by the caller
    */
    MailClientData* getFirstItem(BOOL keyOnly = FALSE); 
    MailClientData* getNextItem(BOOL keyOnly = FALSE); 

    MailClientData* getFirstNewItem();
    MailClientData* getNextNewItem();

    MailClientData* getFirstUpdatedItem();
    MailClientData* getNextUpdatedItem();

    MailClientData* getFirstDeletedItem();
    MailClientData* getNextDeletedItem();
        
    /*
    * Return the ArrayList of MailExtraData containing the extra info related to the mail that aren't set into
    * pocket pc mail object but are necessary for the server (i.e. message-id)
    */
    HRESULT setSync (int mode, const wchar_t* path, ArrayList& extra);
    HRESULT GetAllMessages(ENTRYLIST& entryList, LPMAPIFOLDER m_pInBoxFolder, 
                           const wchar_t* path, wchar_t folderIdentification);

    HRESULT GetModificatedMessages(ENTRYLIST& newMessages, ENTRYLIST& updatedMessages, 
                                   ArrayList** deletedMessages, LPMAPIFOLDER m_pInBoxFolder, 
                                   const wchar_t* path, wchar_t folderIdentification,
                                   ArrayList& extra);

    HRESULT GetRecipients(IMessage* pmsg, char** to, char** cc, char** bcc); 
    HRESULT GetBody(IMessage* pmsg, wchar_t** body);
    HRESULT GetAttachments(IMessage* pmsg, MailClientData* mcd);

    MailInfo* readPreviousMail(const wchar_t* path, wchar_t folderId, ArrayList& extra);
    void writeCurrentMails(const wchar_t* path, ArrayList& extra);

};

#endif
