/*
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <winsock2.h>
#include <ws2tcpip.h>
#include <windows.h>
#include <tchar.h>
#include <strsafe.h>

#include "logutils.h"

// Declared in notlstnr.cpp
void S4N_DEBUG_Init(void);
DWORD S4N_DEBUG_accept(SOCKET sock);


#define DEFAULT_FAMILY  AF_UNSPEC
#define DEFAULT_SOCKTYPE SOCK_STREAM
#define DEFAULT_PORT  "4745"
#define BUFFER_SIZE   23    // length of "WinCE Echo Test Packet"

int _tmain (int argc, TCHAR* argv[])
{
    SOCKET sock, SockServ[FD_SETSIZE];
    int nFamily = DEFAULT_FAMILY;
    int nSockType = DEFAULT_SOCKTYPE;
    char *szPort = DEFAULT_PORT;
    SOCKADDR_STORAGE ssRemoteAddr;
    int i, nNumSocks, cbRemoteAddrSize;
    WSADATA wsaData;
    ADDRINFO Hints, *AddrInfo = NULL, *AI;
    fd_set fdSockSet;

    if(WSAStartup(MAKEWORD(2,2), &wsaData))
    {
        // WSAStartup failed
        return 1;
    }

    S4N_DEBUG_Init();

    sock = INVALID_SOCKET;

    for(i = 0; i < FD_SETSIZE; i++)
        SockServ[i] = INVALID_SOCKET;

    //
    // Get a list of available addresses to serve on
    //

    memset(&Hints, 0, sizeof(Hints));
    Hints.ai_family = nFamily;
    Hints.ai_socktype = nSockType;
    Hints.ai_flags = AI_NUMERICHOST | AI_PASSIVE;

    if(getaddrinfo(NULL, szPort, &Hints, &AddrInfo))
    {
        Print(TEXT("ERROR: getaddrinfo failed with error %d\r\n"), WSAGetLastError());
        goto Cleanup;
    }

    //
    // Create a list of serving sockets, one for each address
    //

    i = 0;
    for(AI = AddrInfo; AI != NULL; AI = AI->ai_next) 
    {
        if (i == FD_SETSIZE) 
        {
            // getaddrinfo returned more addresses than we could use
            break;
        }

        if((AI->ai_family == PF_INET) || (AI->ai_family == PF_INET6)) // only want PF_INET or PF_INET6
        {
            SockServ[i] = socket(AI->ai_family, AI->ai_socktype, AI->ai_protocol);
            if (SockServ[i] != INVALID_SOCKET)
            {
                if (bind(SockServ[i], AI->ai_addr, AI->ai_addrlen) == SOCKET_ERROR)
                    closesocket(SockServ[i]);
                else 
                {
                    if(nSockType == SOCK_STREAM)
                    {
                        if (listen(SockServ[i], 5) == SOCKET_ERROR)
                        {
                            closesocket(SockServ[i]);
                            continue;
                        }
                    }

                    DebugPrint( 
                        TEXT("Socket 0x%08x ready for connection with %hs family, %hs type, on port %hs\r\n"), 
                        SockServ[i], 
                        (AI->ai_family == AF_INET) ? "AF_INET" : ((AI->ai_family == AF_INET6) ? "AF_INET6" : "UNKNOWN"),
                        (AI->ai_socktype == SOCK_STREAM) ? "TCP" : ((AI->ai_socktype == SOCK_DGRAM) ? "UDP" : "UNKNOWN"),
                        szPort);
                    i++;
                }
            }
        }
    }
    
    freeaddrinfo(AddrInfo);
    
    if (i == 0) 
    {
        Print(TEXT("ERROR: Unable to serve on any address. Error = %d\r\n"), WSAGetLastError());
        goto Cleanup;
    }

    //
    // Wait for incomming data/connections
    //

    nNumSocks = i;

    while(1){
        FD_ZERO(&fdSockSet);

        for (i = 0; i < nNumSocks; i++){    // want to check all available sockets
            FD_SET(SockServ[i], &fdSockSet);
        }

        if (select(nNumSocks, &fdSockSet, 0, 0, NULL) == SOCKET_ERROR)
        {
            Print(TEXT("ERROR: select() failed with error = %d\r\n"), WSAGetLastError());
            goto Cleanup;
        }

        for (i = 0; i < nNumSocks; i++)    // check which socket is ready to process
        {
            if (FD_ISSET(SockServ[i], &fdSockSet))    // proceed for connected socket
            {
                FD_CLR(SockServ[i], &fdSockSet);
                if(nSockType == SOCK_STREAM)
                {
                    cbRemoteAddrSize = sizeof(ssRemoteAddr);
                    sock = accept(SockServ[i], (SOCKADDR*)&ssRemoteAddr, &cbRemoteAddrSize);
                    if(sock == INVALID_SOCKET) 
                    {
                        Print(TEXT("ERROR: accept() failed with error = %d\r\n"), WSAGetLastError());
                        goto Cleanup;
                    }

                    DebugPrint(TEXT("Accepted TCP connection from socket 0x%08x\r\n"), sock);

                    // Process Data
                    DWORD ret = S4N_DEBUG_accept(sock);
                    if(ret){
                        Print(TEXT("acceptConnection returned: %d"), ret);
                    }

                }
                else
                {
                    sock = SockServ[i];
                    Print(TEXT("UDP data available on socket 0x%08x\r\n"), sock);
                }
                break;        // Only need one socket
            }
        }
    }

Cleanup:

    for(i = 0; i < nNumSocks && SockServ[i] != INVALID_SOCKET; i++)
        closesocket(SockServ[i]);

    if(sock != INVALID_SOCKET)
    {
        shutdown(sock, SD_BOTH);
        closesocket(sock);
    }

    WSACleanup();

    return 0;
}
