/*
 * Copyright (C) 2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "pushmessage.h"
#include "logutils.h"

int handleWapPushMessage(LPPUSHMSG pPushMsg, wchar_t*** sources)
{
    int ret             = 0;       
    wchar_t* srcname    = NULL;
    SyncNotification    np;
    SN_Errors           errcode;    
    SyncAlert*          sync;    
    
    if (pPushMsg->pbBody == NULL)  {    
        LOG.error("Wap push: sms body is empty!");
        ret =  -1;
        goto finally;
    }
    
    // only to debug
    //SaveFile(pPushMsg);
    
    //
    // used in contact... TO BE FIXED
    //
    //errcode = np.parse((const char*)pPushMsg->pbBody + 13, pPushMsg->cbBodyLength - 13);
    errcode = np.parse((const char*)pPushMsg->pbBody, pPushMsg->cbBodyLength);
    
    PrintMessage(np);
    
    if (np.getNumSyncs() > 0) {
        *sources = new wchar_t*[np.getNumSyncs()];        
    }         
    for (int i = 0; i < np.getNumSyncs(); i++) {
        *sources[i] = NULL;        
        sync = np.getSyncAlert(i);
        
        if( !sync ){
            LOG.error("Wap push: no SyncAlert in SyncNotification");
            ret = -1;
            goto finally;
        }
        srcname = getSourceName(APPLICATION_URI, sync->getServerURI());        
        if (srcname) {
            *sources[i] = new wchar_t[128];
            wcscpy(*sources[i], srcname);
            delete [] srcname;
        } else {
            LOG.error("SMS notification: no source found from server notification request: %s", sync->getServerURI());    
        }
        DebugPrint(*sources[i]);
    }
    
    for (int i = 0; i < np.getNumSyncs(); i++) {
        if (*sources[i] == NULL) {
            ret = -1;
            LOG.error("SMS notification: a source sent by server is not compliant with the clients");    
        }
    }

    if (pPushMsg->szHeaders != NULL) {
        //MessageBox (NULL, pPushMsg->szHeaders, TEXT ("client_push"), MB_SETFOREGROUND |MB_OK);        
    }

finally:
    return ret;
}



static wchar_t *getSourceName(const char *root, const char *uri)
{
    int namelen = strlen(root) + strlen(CONTEXT_SPDS_SOURCES) + 5;
    int i = 0;
    char *nodeName = new char[namelen];
    wchar_t *ret = NULL;

    if(!nodeName)
        return NULL;

    sprintf(nodeName, "%s/%s", root, CONTEXT_SPDS_SOURCES);

	DMTree* dmt = DMTreeFactory::getDMTree(root);

    ManagementNode *node = dmt->getManagementNode(nodeName);
    if ( ! node ) {
        lastErrorCode = ERR_INVALID_CONTEXT;
        sprintf(lastErrorMsg, ERRMSG_INVALID_CONTEXT, nodeName);
        DebugPrint(TEXT("getSourceName: error: %s"), lastErrorMsg);
        goto finally;
    }
    for(i=0; i<node->getChildrenMaxCount(); i++) {
        ManagementNode *src = node->getChild(i);
        char *tmp = src->getPropertyValue(PROPERTY_SOURCE_URI);
        //DebugPrint(TEXT("i:%d uri:%s tmp:%s\n"), i, uri, tmp);
        if (strcmp(uri, tmp) == 0) {
            char *val = src->getPropertyValue(PROPERTY_SOURCE_NAME);
            ret = toWideChar(val);
            delete [] val;
            break;
        }   
        delete [] tmp;
    }
    
finally:
    if (dmt) {
        delete dmt;
    }
    if (node) {
        delete node;
    }
    return ret;
}
